Package: DuplexDiscovereR
Title: Analysis of the data from RNA duplex probing experiments
Authors@R: 
        c(person("Egor", "Semenchenko", , 
        email = "yegor.smb@gmail.com", 
        role = c("aut", "cre","cph"),
        comment = c(ORCID = "0009-0007-5306-076X")),
        person("Volodymyr", "Tsybulskyi", ,
        email = "Volodymyr.Tsybulskyi@mdc-berlin.de", role = c("ctb"),
        comment = c(ORCID = "0009-0002-4141-6291")),
        person("Irmtraud M.", "Meyer", ,
        email = "irmtraud.m.meyer@cantab.net", role = c("aut","cph"),
             comment = c(ORCID = "0000-0002-4048-3479")))
Description: DuplexDiscovereR is a package designed for analyzing data
        from RNA cross-linking and proximity ligation protocols such as
        SPLASH, PARIS, LIGR-seq, and others. DuplexDiscovereR accepts
        input in the form of chimerically or split-aligned reads. It
        includes procedures for alignment classification, filtering,
        and efficient clustering of individual chimeric reads into
        duplex groups (DGs). Once DGs are identified, the package
        predicts RNA duplex formation and their hybridization energies.
        Additional metrics, such as p-values for random ligation
        hypothesis or mean DG alignment scores, can be calculated to
        rank final set of RNA duplexes. Data from multiple experiments
        or replicates can be processed separately and further compared
        to check the reproducibility of the experimental method.
License: GPL-3
URL: https://github.com/Egors01/DuplexDiscovereR/
BugReports: https://github.com/Egors01/DuplexDiscovereR/issues/
Encoding: UTF-8
NeedsCompilation: no
Version: 1.6.0
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
BiocType: Software
biocViews: Sequencing, Transcriptomics, StructuralPrediction,
        Clustering, SplicedAlignment
Imports: Gviz, Biostrings, rtracklayer, GenomicAlignments,
        GenomicRanges, GenomeInfoDb, ggsci, igraph, rlang, scales,
        stringr, dplyr, tibble, tidyr, purrr, methods, grDevices,
        stats, utils, vctrs
Depends: R (>= 4.5), InteractionSet
LazyData: false
Suggests: knitr, UpSetR, BiocStyle, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
VignetteEngine: knitr
Config/testthat/edition: 3
Config/pak/sysreqs: cmake libglpk-dev make libbz2-dev libicu-dev
        libjpeg-dev liblzma-dev libpng-dev libuv1-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:03:52 UTC
RemoteUrl: https://github.com/bioc/DuplexDiscovereR
RemoteRef: RELEASE_3_23
RemoteSha: 6f2f2e9d95170bb05e2344d6385a940c45c98bbd
Packaged: 2026-04-29 21:04:38 UTC; root
Author: Egor Semenchenko [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0007-5306-076X>),
  Volodymyr Tsybulskyi [ctb] (ORCID:
    <https://orcid.org/0009-0002-4141-6291>),
  Irmtraud M. Meyer [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-4048-3479>)
Maintainer: Egor Semenchenko <yegor.smb@gmail.com>
Built: R 4.6.0; ; 2026-04-29 21:54:24 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DuplexDiscovererResults)
export(DuplexTrack)
export(annotateGI)
export(availableDisplayPars)
export(calculateLigationPvalues)
export(classifyTwoArmChimeras)
export(clusterDuplexGroups)
export(collapseIdenticalReads)
export(collapse_duplex_groups)
export(compareMultipleInteractions)
export(computeGISelfOverlaps)
export(convert_gi_to_ranges)
export(dd_get_chimeric_reads)
export(dd_get_chimeric_reads_stats)
export(dd_get_duplex_groups)
export(dd_get_reads_classes)
export(dd_get_run_stats)
export(getChimericJunctionTypes)
export(getRNAHybrids)
export(getSpliceJunctionChimeras)
export(get_char_count_cigar)
export(makeDfFromGi)
export(makeGiFromDf)
export(runDuplexDiscoPreproc)
export(runDuplexDiscoverer)
export(trimAroundJunction)
export(writeGiToSAMfile)
exportMethods(show)
import(Gviz)
import(InteractionSet)
import(dplyr)
import(grid)
import(rlang)
import(rtracklayer)
import(stringr)
import(tibble)
importFrom(Biostrings,RNAStringSet)
importFrom(Biostrings,letterFrequency)
importFrom(Biostrings,readBStringSet)
importFrom(GenomicAlignments,cigarWidthAlongReferenceSpace)
importFrom(GenomicRanges,GRanges)
importFrom(Gviz,availableDisplayPars)
importFrom(Gviz,getScheme)
importFrom(dplyr,first)
importFrom(ggsci,pal_igv)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hsv)
importFrom(grDevices,rgb2hsv)
importFrom(igraph,V)
importFrom(igraph,cluster_fast_greedy)
importFrom(igraph,cluster_louvain)
importFrom(igraph,decompose)
importFrom(igraph,edge.attributes)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,membership)
importFrom(igraph,simplify)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,getClass)
importFrom(methods,getClassDef)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(purrr,set_names)
importFrom(scales,rescale)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(stats,setNames)
importFrom(tidyr,replace_na)
importFrom(tidyr,unite)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
importFrom(vctrs,vec_cast)

# DuplexDiscovereR

<!-- badges: start -->
<!-- badges: end -->

A package for the analysis of the data from RNA duplex probing experiments
## About

DuplexDiscovereR is a package for analysing data from RNA cross-linking and 
proximity ligation protocols such as SPLASH, PARIS, LIGR-seq and other
experimental methods that capture information about RNA-RNA interactions 
as chimeric fragments after high-throughput sequencing.
It provides functions for uniform processing and visualisation of the raw chimeric 
read data. The methods implemented in DuplexDiscovereR are compatible with 
Bioconductor's GenomicRanges and InteractionSet classes, allowing for the 
integration of RNA-RNA interaction probing results into transcriptomic
analyses. 

## License

The `DuplexDiscovereR` code is distributed under [GNU Public License 3.0](https://www.gnu.org/licenses/gpl-3.0.html). 
The documentation, including the manual pages and vignettes,
is distributed under a [CC BY-SA 4.0 license](https://creativecommons.org/licenses/by-sa/4.0/deed.en).


Some functionality of `DuplexDiscovereR` depends on the [RNAduplex](https://www.tbi.univie.ac.at/RNA/RNAduplex.1.html) from the
[ViennaRNA](https://www.tbi.univie.ac.at/RNA/). ViennaRNA is distributed under its own licence.


## Installation

To install this package, use R (version "4.4"):

```
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("DuplexDiscovereR")
```

You can also install the package from Github with 
```r
library(devtools)
install_github('Egors01/DuplexDiscovereR')
```
 [RNAduplex](https://www.tbi.univie.ac.at/RNA/RNAduplex.1.html) can be installed by foolowing instructions on [ViennaRNA web-page](https://www.tbi.univie.ac.at/RNA/)

## Getting started

Package tutorial is avaliable on [Bioconductor](https://www.bioconductor.org/packages/release/bioc/vignettes/DuplexDiscovereR/inst/doc/DuplexDiscovereR.html) 

It can be loaded from R by 

```r
browseVigenttes("DuplexDiscovereR")
```


## Citataion
Semenchenko E, Tsybulsky V, Meyer IM (2025). "DuplexDiscovereR: a computational method for the analysis of experimental duplex RNA-RNA interaction data". Nucleic Acids Research. 
https://doi.org/10.1093/nar/gkaf266
```
@article{DuplexDiscovereR,
  title={DuplexDiscovereR: a computational method for the analysis of experimental duplex RNA-RNA interaction data},
  author={Semenchenko E, Tsybulsky V, Meyer IM},
  journal={Nucleic Acids Research},
  doi = {10.1093/nar/gkaf266}
  year={2018},
  publisher={Oxford University Press}
}
```

## ----style, echo = FALSE, results = 'asis'------------------------------------
BiocStyle::markdown(css.files = c("custom.css"))

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##",
    crop = FALSE
)

## ----install_chunk, eval=FALSE------------------------------------------------
# if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")
# 
# BiocManager::install("DuplexDiscovereR")
# 
# library(DuplexDiscovereR)
# ?DuplexDiscovereR

## ----install_chunk2, eval=FALSE-----------------------------------------------
# if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")
# 
# library(devtools)
# devtools::install_github("Egors01/DuplexDiscovereR")

## ----dataload,echo=T,eval=T,message=FALSE-------------------------------------
library(DuplexDiscovereR)
data(RNADuplexesSampleData)

## ----dataload2----------------------------------------------------------------
system.file("extdata", "scripts", "DD_data_generation.R", package = "DuplexDiscovereR")

## ----run_workflow, echo=T,eval=T,message=FALSE--------------------------------
genome_fasta <- NULL
result <- DuplexDiscovereR::runDuplexDiscoverer(
    data = RNADuplexesRawChimSTAR,
    junctions_gr = SampleSpliceJncGR,
    anno_gr = SampleGeneAnnoGR,
    df_counts = RNADuplexesGeneCounts,
    sample_name = "example_run",
    lib_type = "SE",
    table_type = "STAR",
    fafile = genome_fasta,
)

## ----observe_results1,eval=TRUE-----------------------------------------------
result

## ----observe_results2,eval=TRUE-----------------------------------------------
gi_clusters <- dd_get_duplex_groups(result)
head(gi_clusters, 2)

## ----observe_results3,eval=TRUE-----------------------------------------------
gi_reads <- dd_get_chimeric_reads(result)
head(gi_reads, 2)

## ----observe_results4a,eval=TRUE----------------------------------------------
df_reads <- dd_get_reads_classes(result)
print(dim(df_reads))
print(dim(RNADuplexesRawChimSTAR))

## ----observe_results4b,eval=TRUE----------------------------------------------
table(df_reads$read_type)

## ----observe_results5,eval=TRUE-----------------------------------------------
table(gi_reads$junction_type)

## ----write_table, eval=TRUE---------------------------------------------------
clusters_dt <- makeDfFromGi(gi_clusters)
write.table(clusters_dt, file = tempfile(pattern = "dgs_out", fileext = ".tab"))

## ----write_sam, eval=TRUE, message = FALSE------------------------------------
writeGiToSAMfile(gi_clusters, file_out = tempfile(pattern = "dgs_out", fileext = ".sam"), genome = "hg38")

## ----vis_1, eval=TRUE,message=FALSE-------------------------------------------
library(Gviz)
# define plotting region
plotrange <- GRanges(
    seqnames = "chr22",
    ranges = IRanges(
        start = c(37877619),
        end = c(37878053)
    ),
    strand = "+"
)
# make genes track
anno_track <- Gviz::GeneRegionTrack(SampleGeneAnnoGR,
    name = "chr22 Genes",
    range = plotrange
)
# construct DuplexTrack
duplex_track <- DuplexTrack(
    gi = gi_clusters,
    gr_region = plotrange,
    labels.fontsize = 12,
    arcConstrain = 4,
    annotation.column1 = "gene_name.A",
    annotation.column2 = "gene_name.B"
)
plotTracks(list(anno_track, duplex_track),
    sizes = c(1, 3),
    from = start(plotrange) - 100,
    to = end(plotrange) + 100
)

## ----comp1, eval=TRUE---------------------------------------------------------
clust_reads <- gi_reads
clust_reads <- clust_reads[!is.na(clust_reads$dg_id)]

# Create separate pseudo-sample objects for each group
set.seed(123)
group_indices <- sample(rep(1:3, length.out = length(clust_reads)))

group1 <- clust_reads[group_indices == 1]
group2 <- clust_reads[group_indices == 2]
group3 <- clust_reads[group_indices == 3]

## ----comp2, eval=TRUE---------------------------------------------------------
group1 <- collapse_duplex_groups(group1)
group2 <- collapse_duplex_groups(group2)
group3 <- collapse_duplex_groups(group3)

## ----comp3, eval=TRUE, message=FALSE------------------------------------------
a <- list("sample1" = group1, "sample2" = group2, "sample3" = group3)
res_comp <- compareMultipleInteractions(a)
names(res_comp)

## ----comp4, eval=TRUE---------------------------------------------------------
# first, check if UpSetR is installed
if (!requireNamespace("UpSetR", quietly = TRUE)) {
    stop("Install 'UpSetR' to use this function.")
}
library(UpSetR)
upset(res_comp$dt_upset, text.scale = 1.5)

## ----preproc ex1, message=FALSE-----------------------------------------------
data("RNADuplexesSampleData")

new_star <- runDuplexDiscoPreproc(
    data = RNADuplexesRawChimSTAR,
    table_type = "STAR",
    library_type = "SE",
)

## ----preproc ex2, message=FALSE-----------------------------------------------
new_bedpe <- runDuplexDiscoPreproc(
    data = RNADuplexesRawBed,
    table_type = "bedpe",
    library_type = "SE", return_gi = TRUE
)

## ----preproc ex3, message=FALSE-----------------------------------------------
# keep only readname in metadata
mcols(RNADuplexSampleGI) <- mcols(RNADuplexSampleGI)["readname"]
new_gi <- runDuplexDiscoPreproc(data = RNADuplexSampleGI, return_gi = TRUE)
head(new_gi, 1)

## ----ex_classif1--------------------------------------------------------------
gi <- getChimericJunctionTypes(new_gi)
gi <- getSpliceJunctionChimeras(gi, sj_gr = SampleSpliceJncGR)

## ----ex_classif2--------------------------------------------------------------
table(gi$junction_type)

## ----ex_cassif3---------------------------------------------------------------
table(gi$splicejnc)

## -----------------------------------------------------------------------------
keep <- which((gi$junction_type == "2arm") & (gi$splicejnc == 0))
gi <- gi[keep]

## ----clust1, message=FALSE----------------------------------------------------
gi <- clusterDuplexGroups(gi)

## ----ex_clust2----------------------------------------------------------------
table(is.na(gi$dg_id))

## ----ex_clist3----------------------------------------------------------------
gi_dgs <- collapse_duplex_groups(gi)
head(gi_dgs, 1)
# number of DGs
length(gi_dgs)

## ----dupl1--------------------------------------------------------------------
res_collapse <- collapseIdenticalReads(gi)
# returns new object
gi_unique <- res_collapse$gi_collapsed

## ----dupl2--------------------------------------------------------------------
head(res_collapse$stats_df, 1)

## ----dupl3, message=FALSE-----------------------------------------------------
# cluster  gi_unqiue
gi_unique_dg <- collapse_duplex_groups(clusterDuplexGroups(gi_unique))
# check if the get the same amount of reads as in basic clustering
sum(gi_unique_dg$n_reads) == sum(gi_dgs$n_reads)

## ----cl1----------------------------------------------------------------------
graph_df <- computeGISelfOverlaps(gi_unique)
head(graph_df)

## ----cl2a---------------------------------------------------------------------
graph_df <- graph_df[graph_df$ratio.A > 0.5 & graph_df$ratio.B > 0.5, ]

## ----cl2b---------------------------------------------------------------------
rescale_vec <- function(x) {
    return((x - min(x)) / (max(x) - min(x)))
}

graph_df$weight_new <- graph_df$ratio.A + graph_df$ratio.B
graph_df$weight_new <- rescale_vec(graph_df$weight_new)
head(graph_df, 2)

## ----cl3, message=FALSE-------------------------------------------------------
gi_clust_adj <- clusterDuplexGroups(gi_unique, graphdf = graph_df, weight_column = "weight_new")
gi_clust_adj_dgs <- collapse_duplex_groups(gi_clust_adj)

## ----cl4----------------------------------------------------------------------
length(gi_clust_adj)

## ----session-info,cache = F,echo=T,message=T,warning=FALSE--------------------
sessionInfo()

#' This script shows how the the example data used for test and vignettes was generated.
#' It is not intended to be run 'as-is'. To replicate the process, one should
#' start with downloading the raw RNA duplex read read data from i.e SRA and proceed
#' with the downstream steps:

#' For creating this example data, we used single replicate of the SPLASH ES
#' cells libraries from SRA https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943.
#' Further details on SPLASH can be found here
#' https://csb5.github.io/splash/
#' Reads were adidtionally pre-processed with fastp using default parameters.
#' SPLASH libraries were sequenced in paied-end mode,
#' but stored as merged (vie seqprep)to single-end in in SRA
#' Following STAR command was used to map raw reads and produce
#' Chimeric.out.junction file and ReadsPerGene.out.tab
##' -------------------------------
##'   ##### Final effective command line:
##'   STAR   --runThreadN 8   --genomeDir ./hg38_masked_index
##' --genomeLoad NoSharedMemory   --readFilesIn ./SRR3404943.filt.fastq
##' --outFileNamePrefix ./SPLASH_ES_1/   --outSAMtype BAM   Unsorted
##' --outSAMattributes All
##' --outSAMunmapped Within
##' --outSJfilterReads All
##' --alignIntronMin 1
##' --alignIntronMax 10
##' --chimSegmentMin 15
##' --chimScoreMin 1
##' --chimScoreDropMax 30
##' --chimScoreJunctionNonGTAG 0
##' --chimJunctionOverhangMin 10
##' --chimOutType Junctions
##' --chimSegmentReadGapMax 3
##' --chimMultimapNmax 5
##' --chimNonchimScoreDropMin 20
##' --chimOutJunctionFormat 1
##' --quantMode GeneCounts
##' ----------------------------------------
##' After calling STAR, one can follow the steps below, modifying the
##' paths to the user location
##'
##' Use STAR Chimeric.out.junction file as the input.
##' For the test data, we use chr 22 and only whole genes as the features.
##'
library(tidyverse)
library(InteractionSet)
stardf = readr::read_tsv("./SPLASH_ES_1/Chimeric.out.junction")
RNADuplexesRawChimSTAR <- stardf %>% dplyr::filter(chr_donorA==chr_acceptorB,
                                                   chr_donorA == 'chr22')

# Import gene read counts
df_counts = read_tsv('./SPLASH_ES_1/ReadsPerGene.out.tab',skip = 4,col_names = F)
colnames(df_counts) = c('id','s_strand','r_strand','unstr')
RNADuplexesGeneCounts = df_counts[c('id','unstr')]

##' Import gencode human refernce annotation
##' requires downloaded GENCODE or other annotation.
##' https://www.gencodegenes.org/human/release_44.html or via Annotation Hub
##' As thsi test data inlcudes only chr 22, we reduce the annotatoin accordingly.

library(rtracklayer)
gtf_gnc = import.gff('./gencode.v44.annotation.gtf')
SampleGeneAnnoGR = gtf_gnc[seqlevels(gtf_gnc) %in% 'chr22']
SampleGeneAnnoGR = SampleGeneAnnoGR[SampleGeneAnnoGR$type=='gene']

##' To create the example splice junction GRanges object, one can use the reference
##' annotation, as we show below.
##' However, for working with the real-life duplex probing data, we
##' recommend to use the custom file containing splice
##' junctions discovered from mapping the RNA-seq reads of the appropriate
##' cell lines. Novel junctions can be found by STAR run and extracted from the
##' output SJ.out.tab

library(GenomicFeatures)
library(SGSeq)
# create txdb from the same gencode annotation
txdb_v44 <- makeTxDbFromGFF("./gencode.v44.annotation.gtf")
saveDb(txdb_v44,"./txdb_v44.txdb")
tx_gtf<-AnnotationDbi::loadDb("./txdb_v44.txdb")
txf <- convertToTxFeatures(tx_gtf)
SampleSpliceJncGR<-GRanges(txf[txf@type=='J'])

# call the pipeline to produce clustered and unclustered reads samples
library(DuplexDiscovereR)
res <- runDuplexDiscoverer(
  df = RNADuplexesRawChimSTAR,
  junctions_gr = SampleSpliceJncGR,
  anno_gr = SampleGeneAnnoGR,
  df_counts = RNADuplexesGeneCounts,
  sample_name = 'test',
  lib_type = "SE",
  table_type = "STAR"
)
# Two-arm chimeric reads, which can be represented in the GInteraction object
RNADuplexSampleGI = res$gi_reads
mcols(RNADuplexSampleGI) = as_tibble(data.frame(mcols(res$gi_reads))) %>%
  dplyr::select(n_reads,n_reads_dg,dg_id,score,readname,map_type,junction_type,
                cigar_alnA ,cigar_alnB) %>% data.frame()
# Reads clustered into duplex groups
RNADuplexSampleClustReads= res$gi_reads
# Reads clustered and collapsed into duplex groups
RNADuplexSampleDGs = res$gi_clusters
# Reads which can be represented as 2-arm chimeras
# We leave only required BEDPE-specific columns in order defined in a format defintion
# see specification at https://bedtools.readthedocs.io/en/latest/content/general-usage.html#bedpe-format
# We use 'flag field' as the placeholder for the alignment score
RNADuplexesRawBed = DuplexDiscovereR::MakeDfFromGi(RNADuplexSampleGI) %>%
dplyr::select(chromA ,startA,endA,chromB ,startB,endB,readname,score,strandA,strandB)


save(RNADuplexSampleGI,RNADuplexSampleDGs,RNADuplexesRawBed,
     RNADuplexesRawChimSTAR,RNADuplexesGeneCounts,SampleSpliceJncGR,RNADuplexSampleClustReads,
     SampleGeneAnnoGR,file = './data/RNADuplexesSampleData.rda', compress = 'xz')


# save as individual files for tests
write_tsv(RNADuplexesRawBed,'./inst/extdata/test_SPLASH_DuplexesRaw.bedpe',col_names = F)
write_tsv(RNADuplexesRawChimSTAR,'./inst/extdata/test_SPLASH_Chimeric.out.junction',col_names = T)
write_tsv(RNADuplexesGeneCounts,'./inst/extdata/test_ReadsPerGene.out.tab',col_names = T)
export.bed(SampleSpliceJncGR,'./inst/extdata/test_chr22SpliceJunctions.bed')
export.gff(SampleGeneAnnoGR,'./inst/extdata/test_chr22Anno.gtf')



# stardf <- DuplexDiscovereR::run_preproc(stardf,table_type = 'STAR',library_type = 'SE')
# # select
# twoarm <- stardf %>% filter(map_type=='2arm')
# twoarm_gi = MakeGiFromDf(twoarm)
# load('/data/meyer/egor/duplex_workflow/public/DuplexDiscovereR/data/example_data.rda')
# gr_region = GRanges(IRanges(start=17504924,end=17524195),seqnames='chr22',strand = '+')
# add_reads = subsetByOverlaps(twoarm_gi,gr_region+50000)$readname
# add_df = stardf_raw[stardf_raw$read_name %in% add_reads,]
#
# stardf %>% filter(readname %in% example_reads_star$read_name)
# new_removed = example_reads_star[-sample(c(1:5000),size=81),]
# new_test = rbind(new_removed,add_df)
# write_tsv(new_test,'/data/meyer/egor/duplex_workflow/public/DuplexDiscovereR/inst/extdata/sample_Chimeric.out.junction')
# run_duplexdisco()
# setwd("/data/meyer/egor/duplex_workflow/public/DuplexDiscovereR/")
#
#
#

#' DuplexDiscovereR
#'
#' @title Analysis of the data from RNA duplex probing experiments
#' @description
#' DuplexDiscovereR is a package for analysing data from RNA cross-linking and
#' proximity ligation protocols such as SPLASH, PARIS, LIGR-seq and others,
#' which provide information about intra-molecular RNA-RNA interactions through
#' chimeric RNA-seq reads. Chimerically aligned fragments in these experiments
#' correspond to the base-paired stretches (RNA duplexes) of RNA molecules .
#' DuplexDiscovereR takes input in the form of chimericly or split -aligned reads,
#' It implements procedures for alignment classification, filtering and efficient
#' clustering  of  individual chimeric reads into duplex groups (DGs).
#' Once DGs are found, RNA duplex formation and their hybridization energies are
#' predicted. Additional metrics, such as p-values or mean DG alignment scores,
#' can be calculated to rank and analyse the final set of RNA duplexes.
#' Data from multiple experiments or replicates can be processed separately
#' and further compared to check the reproducibility of the experimental method.
#'
#' @name DuplexDiscovereR
#'
#' @import rtracklayer
#' @import InteractionSet
#' @import rlang
#' @importFrom scales rescale
#' @importFrom GenomicAlignments cigarWidthAlongReferenceSpace
#' @import tibble
#' @import dplyr
#' @import stringr
#' @importFrom tidyr unite replace_na
#' @importFrom igraph graph_from_data_frame simplify decompose cluster_louvain cluster_fast_greedy membership edge.attributes V
#' @importFrom grDevices col2rgb  hsv  rgb2hsv
#' @importFrom methods as callNextMethod getClassDef new is getClass slot
#' @importFrom stats median p.adjust pbinom setNames
#' @importFrom utils head setTxtProgressBar txtProgressBar write.table
#' @importFrom purrr set_names
#' @importFrom GenomicRanges GRanges
#' @import Gviz
#' @import grid
#' @importFrom Biostrings readBStringSet RNAStringSet letterFrequency
#' @importFrom ggsci pal_igv
#' @importFrom vctrs vec_cast
#' @importFrom GenomeInfoDb keepSeqlevels
#' @importFrom stats fisher.test
#' @author Egor Semenchenko
#' @seealso [DuplexDiscovereR vignette](`browseVignettes("DuplexDiscovereR")`)
"_PACKAGE"

# Quiets R CMD CHECK NOTES from tidyverse syntax and func from pkg imported with
# @import
varlist <- c(
    "A", "AB", "A_ovl", "A_span", "B", "B_ovl", "B_span",
    "Pa", "Pb", "ambig", "ambig.A", "ambig.B", "ambig_list.A", "ambig_list.B",
    "arc_max_y", "arm", "artCIGAR", "artCIGAR1", "artCIGAR2", "artSEQ",
    "bad_junction", "brkpt_acceptorB", "brkpt_donorA", "chr_acceptorB",
    "chr_donorA", "chromA", "chromB", "cigar_alnA", "cigar_alnB",
    "cigar_str", ".handleComposite", "cmp", "col11", "col1read1",
    "col1read2", "col2", "col5", "col7", "col8", "col9", "dg_id",
    "dg_id_raw", "distance", "duplex_id",
    "end1", "end2", "endA", "endB", "feature_id",
    "findOverlapPairs", "subsetByOverlaps", "GRanges",
    "mcols<-", "punion", "pintersect", "subjectHits", "IRanges",
    "full_in_plot", "gap", "gene_count.A", "gene_count.B",
    "gi_trans", "grp", "grpvar", "has_A", "has_B", "has_p",
    "hg38_seqlengths", "idcol.1",
    "idcol.2", "in_range", "junction_type", "lengapsA", "lengapsB",
    "map_type", "minus_p", "multigap", "multimap", "nA", "nB",
    "n_i", "n_m", "n_n", "n_p", "n_reads", "n_reads_dg", "n_s",
    "new_duplex_id", "ngapsA", "ngapsB", "nr", "num_chim_aln",
    "old_duplex_id", "overlap_w", "p.adj", "pair_arm",
    "queryHits", "rDens", "ratio.A", "ratio.B", "read_id",
    "read_name", "read_type", "readname", "region_id", "rname",
    "seq_anchor1", "seq_anchor2", "seqlengths", "seqlengths",
    "seqnames1", "seqnames2", "sign_p", "splicejnc", "start1", "start2",
    "startA", "startB", "start_alnA", "start_alnB", "strand1", "strand2",
    "strandA", "strandB", "strand_acceptorB", "strand_donorA",
    "tag", "this_chim_aln_score", "seqlengths<-", ".",
    "vert_id", "vert_id1", "vert_id2", "weight", "width1", "width2",
    "x1", "x2", "x3", "t1", "t2", "t3", "t4", "value",
    "RNA", "chim_withB_noA", "pval", "chim_withA_noB",
    "chim_noB_noA", "chim_id", "aln_lenA", "aln_lenB", "jA", "jB", "lA", "lB",
    "len_not_ok", "n_seqnames", "sort_col","C1","C2","C3","C4",
    "nonchimA", "nonchimB","odds_p.adj","odds_ratio",'odds_pval'
)

if (getRversion() >= "2.15.1") utils::globalVariables(varlist, add = FALSE)

#' Run prediciton of RNA hybridization
#'
#' Calls RNAduplex from ViennaRNA to find base-pairs for every entry in the
#' input, throws a message and system warning if it is not installed
#'
#' @param gi `Ginteraction` with pairs of regions
#' @param fafile path to the .fasta file with genome
#'
#' @returns object parallel to input with added energy GC content,
#' dot-format base-pairings and lenghts of RNA hybrids
#' will return the input, if RNAhybrids cannot be run
#'
#' @export
#' @examples
#' sequence <- paste0(
#'     "AGCUAGCGAUAGCUAGCAUCGUAGCAUCGAUCGUAAGCUAGCUAGCUAGCAUCGAUCGUAGCUAGCAUCGAU",
#'     "CGUAGCAUCGUAGCUAGCUAGCUAUGCGAUU"
#' )
#'
#' # Save the sequence to a temp fasta file
#' fasta_file <- tempfile(fileext = ".fa")
#' chrom <- "test_chrA"
#' writeLines(c(">test_chrA", sequence), con = fasta_file)
#'
#' # Create the GInteraction object
#' # Define start and end positions for the base-pairing regions
#' regions <- data.frame(
#'     start1 = c(1, 11, 21, 31, 41),
#'     end1 = c(10, 20, 30, 40, 50),
#'     start2 = c(91, 81, 71, 61, 51),
#'     end2 = c(100, 90, 80, 70, 60)
#' )
#' # GRanges objects for the anchors
#' anchor1 <- GRanges(seqnames = chrom, ranges = IRanges(start = regions$start1, end = regions$end1))
#' anchor2 <- GRanges(seqnames = chrom, ranges = IRanges(start = regions$start2, end = regions$end2))
#' interaction <- GInteractions(anchor1, anchor2)
#' # predict hybrids
#' # In case ViennaRNA is installed
#' \dontrun{
#' gi_with_hybrids <- getRNAHybrids(interaction, fasta_file)
#' }
getRNAHybrids <- function(gi, fafile) {
    # exit if not installed, warn about license if installed
    if (.checkRNAduplexinstalled() != 0) {
        message("RNAduplex from ViennaRNA is not found. Unable to call hybrid predictions")
        return(gi)
    } else {
        message("Calling  RNAduplex from ViennaRNA.\nNote that ViennaRNA is distributed under its own licence.\nPlease visit https://viennarna.readthedocs.io/en/latest/license.html for details ")
    }

    sq <- Biostrings::readBStringSet(fafile)
    sq <- Biostrings::DNAStringSet(sq)
    sq <- Biostrings::RNAStringSet(sq)
    if (!all(as.character(seqnames(regions(refresh_gi(gi)))) %in% names(sq))) {
        stop("Calling hybrids: Seqnames in fasta file do not contain all seqnames of the input gi ")
    }

    gi$GC_content.A <- .getGCContent(sq[get_arm_a(gi)])
    gi$GC_content.B <- .getGCContent(sq[get_arm_b(gi)])

    gr1 <- get_arm_a(gi)
    gr2 <- get_arm_b(gi)
    gr1 <- GenomeInfoDb::keepSeqlevels(gr1, unique(seqnames(gr1)), pruning.mode = "tidy")
    gr2 <- GenomeInfoDb::keepSeqlevels(gr2, unique(seqnames(gr2)), pruning.mode = "tidy")

    seq1 <- sq[gr1]
    seq2 <- sq[gr2]

    rr <- mapply(.runRNAduplex, seq1, seq2)

    dots <- str_split_i(rr, " ", 1)
    energies <- as.numeric(sub(".*\\((-?[0-9]+\\.[0-9]+)\\).*", "\\1", rr))

    matches <- str_match(rr, "\\s(\\d+),(\\d+)\\s+:\\s+(\\d+),(\\d+)\\s")
    len1 <- as.numeric(matches[, 3]) - as.numeric(matches[, 2])
    len2 <- as.numeric(matches[, 5]) - as.numeric(matches[, 4])
    gi$E <- energies
    gi$hyb_len_1 <- len1
    gi$hyb_len_2 <- len2
    gi$hyb_start_A <- as.numeric(matches[, 2])
    gi$hyb_end_A <- as.numeric(matches[, 3])
    gi$hyb_start_B <- as.numeric(matches[, 4])
    gi$hyb_end_B <- as.numeric(matches[, 5])

    gi$dots <- dots
    return(gi)
}

#' Helper function to add ids to the duplex groups missed during global clustering
#'
#' @description
#' Check if there are a temporary duplex records with \code{duplex_id}, which consist of more than one read  \code{n_reads > 1} , but
#' does not have assigned any \code{dg_id} as the duplex group (DG) index. Creates new  \code{dg_id} if \code{n_reads > 1}
#' @details
#' Meant to be used in the situations when previous collapsing steps merged two or more reads to the temporary DG with \code{duplex_id}, but
#' global clustering has not identified any overlap between this temporary group and other duplexes, resulting in undefined \code{dg_id}.
#' This function looks up for these cases and creates new \code{dg_id} for temporary DGs, marking them as the final DGs.
#' New \code{dg_id} values are unique and allocated sequentially after the  maximum value of \code{dg_id}
#' @param gi_input \pkg{GInteractions} with the  \code{dg_id},  \code{duplex_id} and  \code{n_reads} column
#' @return
#' \pkg{GInteractions} object with new \code{dg_id} for rows with  \code{n_reads > 1}
.addDGidsForTmpDGs <- function(gi_input) {
    # add new cluster ids
    col_check <- c("duplex_id", "dg_id", "n_reads")
    if (!all(col_check %in% colnames(mcols(gi_input)))) {
        ermsg <- paste0(col_check, sep = " ")
        stop("One of the ", ermsg, " columns is not in the input gi ")
    }

    n_add <- length(gi_input[is.na(gi_input$dg_id) & gi_input$n_reads > 1])
    if (n_add == 0) {
        message("No collapsed duplexes with n_reads>1 and without dg_id found")
        return(gi_input)
    } else {
        # max_dg_id = max(gi_input$dg_id,na_rm=TRUE) # returns NA for some reason
        max_dg_id <- max(gi_input$dg_id[!is.na(gi_input$dg_id)])

        gi_keep <- gi_input[!(is.na(gi_input$dg_id) & gi_input$n_reads > 1)]

        gi_change <- gi_input[is.na(gi_input$dg_id) & gi_input$n_reads > 1]
        df_add <- makeDfFromGi(gi_change) %>%
            mutate(dg_id = max_dg_id + row_number())
        mcols(gi_change) <- df_add

        # message(" Changing ",n_add, "records from NA dg_id to",paste0(df_add$dg_id,sep=' '))
        gi_res <- c(gi_keep, gi_change)
        return(gi_res)
    }
}


#' Call clustering multiple times to collapse similar reads into duplex groups
#'
#' Function calls clustering algorithm several times and collapses highly similar
#' reads to the temporary duplex groups (DGs).
#'
#' @details
#' Calling this procedure before global read clustering
#' substantially reduces time required for calling DGs.
#' Collapsed duplex groups are aggregated only from the reads which are shifted
#'  by only a few nucleotides from each other.
#' These DGs are temporary until full library clustering is called.
#' To keep track of the mapping of the temprary DGs to the input, dedicated
#' dataframe is returned. The 'duplex_id' column will be added or updated as
#' identifier for the temporary duplex group.
#' The number of reads under single 'duplex_id' is recorded in the 'n_reads' fields
#' @param gi  `GInteractions` object
#' @param read_stats_df `tibble` with the mapping 'read_id' and 'duplex_id' fields
#'  'read_id' refers to the unique read, 'duplex_id' refers to the entry collapsed
#'  identical reads i.e two identical reads will will correspond to two unique read_id and
#'  the single duplex_id with n_reads=2
#' @param maxgap  Maximum relative shift between the overlapping read arms
#' @param niter Number of times clustering will be called
#' @param minoverlap Minimum required overlap between either read arm
#' @param min_nodes Minimum count of nodes to finish the interaction merging
#' @return a list with the following keys
#' \describe{
#'   \item{gi_updated}{ `GInteractions` object with both collapsed duplex groups
#'   and not-collapsed unchanged reads}
#'   \item{stats_df}{ `tibble` With the mapping from the unique read -
#'    with the the infromation about time and memory reaquired for the function
#'    call
#'    }
#' }
collapseSimilarChimeras <- function(
        gi, read_stats_df,
        maxgap = 5,
        niter = 2,
        minoverlap = 10,
        min_nodes = 10) {
    message("--- Collapsing the reads shifted by <= ", maxgap, " nt ---")
    gi_base <- gi
    gi_base$dg_id <- NULL
    stats_df <- read_stats_df
    read_stats_df_upd <- read_stats_df
    for (i in seq_len(niter)) {
        ovl_df <- computeGISelfOverlaps(gi_base, maxgap = maxgap, minoverlap = minoverlap)
        message("----iter ", i, "-----")
        message(
            "Connectivity of the duplex graph: ", .getConnectivity(gi_base, ovl_df),
            " n_edges = ", nrow(ovl_df), " n_nodes = ", length(gi_base)
        )
        if (is_empty(ovl_df) | nrow(ovl_df) < min_nodes) {
            message("Maximum compression level reached on iter", i)
            break
        }

        gi_clustered <- clusterDuplexGroups(gi = gi_base, graphdf = ovl_df, maxgap = maxgap)

        grc <- collapse_duplex_groups(gi_clustered,
            return_unclustered = TRUE,
            return_collapsed = FALSE
        )
        message("N nodes after collapse: ", length(grc))

        df_reads <- makeDfFromGi(grc)

        # update duplex_ids
        orig_mcols <- as_tibble(data.frame(mcols(gi_clustered)[c("duplex_id", "dg_id")]))
        new_mcols <- as_tibble(data.frame(mcols(grc)[c("duplex_id", "dg_id")]))
        fill_mcols <- orig_mcols %>%
            dplyr::filter(!is.na(dg_id)) %>%
            group_by(dg_id) %>%
            dplyr::summarize(
                new_duplex_id = dplyr::first(duplex_id)
            )

        # {
        #   orig_mcols <- as_tibble(data.frame(mcols(gi_clustered)[c("duplex_id", "dg_id")]))
        #   new_mcols <- as_tibble(data.frame(mcols(grc)[c("duplex_id", "dg_id")]))
        #   orig_mcols %>% mutate(was_clustered = if_else())
        #
        # }
        #

        # update in the gi object
        new_collapsed_stats <- left_join(new_mcols, fill_mcols, by = "dg_id") %>%
            mutate(
                new_duplex_id = ifelse(!is.na(duplex_id), duplex_id, new_duplex_id)
            )
        grc$duplex_id <- new_collapsed_stats %>% pull(new_duplex_id)
        if (any(is.na(grc$duplex_id))) {
            stop("After collapsing: Index column 'duplex_id' contains undefined variables")
        }
        # update the stats dataframe parallel to the original gi object
        new_original_stats <- left_join(orig_mcols, fill_mcols, by = c("dg_id")) %>%
            mutate(
                old_duplex_id = duplex_id,
                new_duplex_id = ifelse(is.na(dg_id), old_duplex_id, new_duplex_id)
            ) %>%
            dplyr::select(c(new_duplex_id, old_duplex_id, dg_id))

        read_stats_df_upd <- left_join(stats_df, new_original_stats,
            by = c("duplex_id" = "old_duplex_id")
        ) %>%
            mutate(duplex_id = if_else(is.na(dg_id), duplex_id, new_duplex_id)) %>%
            dplyr::select(-c(new_duplex_id, dg_id)) %>%
            dplyr::distinct(read_id, .keep_all = TRUE)

        # update for next round or exit
        grc$dg_id <- NULL
        gi_base <- grc
        stats_df <- read_stats_df_upd
    }
    res <- list()
    res$gi_updated <- gi_base
    res$stats_df <- read_stats_df_upd

    df_reads <- makeDfFromGi(gi_base)

    return(res)
}


#' Accessor for mapping between temporary and final cluster ids
#' @keywords internal
#' @param gi
#'
#' @return tibble
.DGIdToDuplexId <- function(gi) {
    future_clusters <- as_tibble(data.frame(gi))
    return(future_clusters %>% dplyr::select(duplex_id, dg_id))
}


#' Collapses identical interactions
#' @description
#' Two entries (reads) are considered identical if they share start, end, strand and score vales
#' Identical entries are collapsed into the single one.
#' @details
#'
#' Adds columns to the collapsed object
#' duplex_id (int) unique record id
#' n_reads (int) number of entries collapsed
#'
#' @param gi GInteractions(mode='strict') object with chromA, strandA, startA,
#' endA, chromB, strandB, startB, endB, score columns
#' Optionally cigar_alnA, cigar_alnB columns are also considered for collapsing
#' 'read_id' column used as the index in the initial objects. Created, if not exists
#' @return result_list object with keys
#'' gi_collapsed':   New collapsed GInteraction object
#'' stats_df': tibble with the mapping of the original entries to the new duplex_id
#' @export
#' @examples
#' # load data
#' data("RNADuplexesSmallGI")
#' res_collapse <- collapseIdenticalReads(SampleSmallGI)
#' gi_new <- res_collapse[["gi_collapsed"]]
#' # keeps the mapping of the colapsed object to new
#' read_stats_df <- res_collapse[["stats_df"]]
collapseIdenticalReads <- function(gi) {
    if (is(gi, "StrictGInteractions")) {
        out_gi <- TRUE
        gi_dt <- makeDfFromGi(gi)
    } else {
        out_gi <- FALSE
        message("Input type is not StrictGInteractions, will try to work with the dataframe")
        message("Expected columns: chromA, strandA, startA, endA, chromB, strandB, startB, endB, score")
        message("Optional: cigar_alnA, cigar_alnB")
        gi_dt <- gi
    }

    if (!("read_id" %in% colnames(gi_dt))) {
        gi_dt$read_id <- seq_len(nrow(gi_dt))
        message("Created 'read_id' column as the unique index in  the provided object")
    }

    if ("cigar_alnA" %in% colnames(gi_dt)) {
        dt1 <- gi_dt %>%
            tidyr::unite("grp", c(
                chromA, strandA, startA, endA,
                chromB, strandB, startB, endB, cigar_alnA, cigar_alnB, score
            ), remove = FALSE) %>%
            mutate(num = (grp))
    } else {
        dt1 <- gi_dt %>%
            tidyr::unite("grp", c(
                chromA, strandA, startA, endA,
                chromB, strandB, startB, endB, score
            ), remove = FALSE) %>%
            mutate(num = (grp))
    }

    dt1$n_reads <- NULL
    # to avoid groupby, use duplicated count +1
    readcts <- dt1 %>%
        dplyr::filter(duplicated(num)) %>%
        dplyr::select(num)

    readcts <- table(readcts$num)

    readcts <- tibble::tibble(
        num = names(readcts),
        n_reads = as.vector(readcts) + 1
    )
    if (nrow(readcts) != 0) { #  duplicated reads are present
        dt1 <- dplyr::left_join(dt1, readcts, by = "num") %>%
            dplyr::mutate(n_reads = tidyr::replace_na(n_reads, 1))
    } else { #   no duplicated reads
        dt1 <- dt1 %>% dplyr::mutate(n_reads = 1)
    }


    read2duplex_map <- dt1 %>% dplyr::select(read_id, num)

    dt1 <- dt1 %>% dplyr::distinct(num, .keep_all = TRUE)
    dt1 <- dt1 %>%
        mutate(duplex_id = seq_len(nrow(dt1)))

    read2duplex_map <- left_join(read2duplex_map, dt1[, c("num", "duplex_id", "n_reads")], by = "num") %>%
        dplyr::rename(n_reads_collapsed = n_reads) %>%
        dplyr::select(-c(num))

    dt1 <- dt1 %>%
        dplyr::select(c(
            chromA, strandA, startA, endA,
            chromB, strandB, startB, endB, duplex_id, n_reads, score
        ))


    dt1 <- dt1 %>%
        relocate(duplex_id, .before = score)
    message("Duplicated   :  ", round((1 - nrow(dt1) / nrow(gi_dt)) * 100, 2), "% of initial")
    message("Initial size :  ", nrow(gi_dt))
    message("New size     :  ", nrow(dt1))

    if (out_gi == TRUE) {
        dt1 <- makeGiFromDf(dt1)
    }

    res <- list()
    res$gi_collapsed <- dt1
    res$stats_df <- read2duplex_map
    return(res)
}


.getConnectivity <- function(gi, dt_conn) {
    edg <- nrow(dt_conn)
    vert <- length(gi)
    connectivity <- round(edg / vert, 4)
    return(connectivity)
}

#' Find overlaps between entries in `GInteractions`
#'
#' Utility function to find overlapping reads in the input and calculate
#' overlap scores. Removes self-hits. Computes overlap/span ratios for
#' each interaction arm. Sum of the scores is recorded in 'weight' field
#'
#' @param gi input gi object
#' @param id_column column which use for using as ids for entries
#' @param maxgap parameter for call of [InteractionSet::findOverlaps()]
#' @param minoverlap  parameter for call [InteractionSet::findOverlaps()]
#'
#' @return dataframe with indexes of pairwise overlapsin input and
#' columns for span, overlap, ratios of either read arm
#' @export
#' @examples
#' data("RNADuplexesSmallGI")
#' computeGISelfOverlaps(SampleSmallGI)
computeGISelfOverlaps <- function(gi, id_column = "duplex_id", maxgap = 40, minoverlap = 10) {
    if (id_column %in% colnames(mcols(gi))) {
        # message("Using ",id_column," as id for computing overlaps")
        ids <- mcols(gi)[, id_column]
        mcols(gi) <- NULL
        gi$idcol <- ids
    } else {
        # message("Using index as id for computing overlaps")
        mcols(gi) <- NULL
        gi$idcol <- seq_len(length(gi))
        id_column <- "index"
    }
    id_names <- str_c(id_column, c(1, 2), sep = ".")

    fo <- findOverlaps(gi, ignore.strand = FALSE, type = "equal", maxgap = maxgap, minoverlap = minoverlap)
    fo <- fo[queryHits(fo) < subjectHits(fo)]
    if (length(fo) == 0) {
        dt <- tibble()
        message("No overlap found")
        return(dt)
    }
    dt <- tibble(
        idcol.1 = gi[queryHits(fo)]$idcol,
        idcol.2 = gi[subjectHits(fo)]$idcol,
        A_span = width(punion(
            gi@regions[gi[queryHits(fo)]@anchor1],
            gi@regions[gi[subjectHits(fo)]@anchor1],
            fill.gap = TRUE
        )),
        B_span = width(punion(
            gi@regions[gi[queryHits(fo)]@anchor2],
            gi@regions[gi[subjectHits(fo)]@anchor2],
            fill.gap = TRUE
        )),
        A_ovl = width(pintersect(
            gi@regions[gi[queryHits(fo)]@anchor1],
            gi@regions[gi[subjectHits(fo)]@anchor1]
        )),
        B_ovl = width(pintersect(
            gi@regions[gi[queryHits(fo)]@anchor2],
            gi@regions[gi[subjectHits(fo)]@anchor2]
        ))
    ) %>%
        mutate(
            ratio.A = (A_ovl / A_span),
            ratio.B = (B_ovl) / (B_span),
            weight = ratio.A + ratio.B
        ) %>%
        dplyr::rename(
            !!quo_name(id_names[1]) := idcol.1,
            !!quo_name(id_names[2]) := idcol.2
        )

    return(dt)
}

#' Collapse the reads into the duplex groups after clustering
#'
#' @description
#' Collapse each interaction in the input to the duplex group based on the pre-computed dg_id
#' @param gi `GInteractions` with the 'dg_id' metadata field
#' @param return_unclustered add unclustered reads to output
#' @param return_collapsed add duplex groups, which were created as temporary
#' with n_reads > 1 but was not clustered to the DG golabally. This parameter
#' is used internally and should be kept default in most situations.
#' @param keep_meta whether to keep metadata, which only unclustered reads have, in case of a mixed output
#'
#' @details
#' 'dg_id' is used as the identifier for the duplex group
#' Reads belonging to the same duplex group are collapsed into a single entry
#' with start and end are set as min() and max() coordinate of the reads in within the duplex group.
#' The 'score' column is averaged across the duplex group reads is calculated and
#' put as the 'score' for the collapsed duplex group
#' Behavior in case 'dg_id'  = NA: Option '\code{return_unclustered}' -
#'  whether unclustered reads with should be added to the output gi
#' \describe{
#'   \item{return_unclustered == FALSE}{Interaction is not returned in the output. Default. }
#'   \item{return_unclustered == TRUE}{Interaction is returned in the output,
#'    output is mixed duplex groups and individual reads}
#'   }
#'  Internally used argument
#'  #' \describe{
#'   \item{return_collapsed == FALSE}{In case interaction already collapsed and n_read > 1,
#'   interaction will not be returned as duplex group }
#'   \item{return_collapsed == TRUE}{In case interaction has n_read > 1, interaction will be treated
#'    as duplex group }
#'   }
#' @importFrom dplyr first
#' @export
#' @return
#' `GInteractions` object with collapsed duplex groups
#' @examples
#' # load example of clustered data
#' data("RNADuplexesSampleData")
#' # some reads assigned to DG, some are not
#' table(is.na(RNADuplexSampleGI$dg_id))
#' # Return only DGs
#' gicollapsed <- collapse_duplex_groups(RNADuplexSampleGI, return_unclustered = FALSE)
#' # Return DGs and unclustered reads as well
#' gimixed <- collapse_duplex_groups(RNADuplexSampleGI, return_unclustered = TRUE)
#'
#' # load small sample GInteractions and process it manually
#' data("RNADuplexesSmallGI")
#' # First, collapse duplicated reads. This adds n_reads and duplex ids
#' ginodup <- collapseIdenticalReads(SampleSmallGI)$gi_collapsed
#' # Second, run clustering, get DG ids
#' ginodup <- clusterDuplexGroups(ginodup)
#' # Return all DGs result in n=3 DGS, one of them formed by
#' # identical duplicated alignments
#' collapse_duplex_groups(ginodup, return_collapsed = TRUE)
#' # Return DGs, but drop duplicated returns n=2 DGs
#' collapse_duplex_groups(ginodup, return_collapsed = FALSE)
collapse_duplex_groups <- function(
        gi, return_unclustered = FALSE,
        return_collapsed = TRUE,
        keep_meta = TRUE) {
    # check n_reads field
    if (!is.null(gi$n_reads)) {
        gi$n_reads <- as_tibble(mcols(gi)["n_reads"]) %>%
            mutate(n_reads = if_else(!is.na(n_reads), n_reads, 1)) %>%
            pull(n_reads)
    } else {
        gi$n_reads <- 1
    }

    if (return_collapsed) {
        to_add_n <- length(gi[is.na(gi$dg_id) & gi$n_reads > 1])
        if (to_add_n != 0) {
            message("Found temporary duplex groups without assigned dg_id", to_add_n)
            gi <- .addDGidsForTmpDGs(gi)
        }
    }


    # #DEBUG
    # df_reads =  as_tibble(data.frame(gi))
    # df_bad = df_reads %>% filter(!is.na(dg_id))
    # if (nrow(df_bad)==0){
    #   place = 56
    #   browser()
    # }
    # df_bad = df_bad  %>% group_by(dg_id) %>%
    #   mutate(n_seqnames = length(unique(c(seqnames1,seqnames2)))) %>%
    #   filter(n_seqnames>=3)
    # if (nrow(df_bad)!=0){
    #   place = 55
    #   browser()
    # }
    #

    future_clusters <- as_tibble(data.frame(gi))

    future_clusters <- future_clusters %>%
        dplyr::filter(!is.na(dg_id)) %>%
        group_by(dg_id) %>%
        summarise(
            chromA = dplyr::first(seqnames1),
            startA = min(start1),
            endA = max(end1),
            strandA = dplyr::first(strand1),
            chromB = dplyr::first(seqnames2),
            startB = min(start2),
            endB = max(end2),
            strandB = dplyr::first(strand2),
            n_reads = sum(n_reads),
            score = mean(score, na.rm = TRUE)
        ) %>%
        relocate(dg_id, .after = n_reads)

    res_gi <- makeGiFromDf(future_clusters)
    if (return_unclustered) {
        unclustered <- gi[is.na(gi$dg_id)]
        res_gi <- c(res_gi, unclustered)
    }

    return(res_gi)
}

#' Check the column names and types in read dataframe
#' @description
#'
#' Function to check the correct column names and types in dataframe input.
#' Tries to guess the column names, if colnames are not provided,
#' but the types are correct
#'
#' @details
#'
#' - Expected column names for bedpe file
#' `c("chromA","startA",'endA',"chromB",
#'                     'startB','endB','readname','flag','strandA','strandB')`
#'
#' - Expected colnames for STAR Chimeric junction input:
#'
#'  - For the 'old' chimeric detection scheme: \code{ c("chr_donorA","brkpt_donorA",
#' "strand_donorA","chr_acceptorB",
#' "brkpt_acceptorB","strand_acceptorB",
#' "junction_type","repeat_left_lenA",
#' "repeat_right_lenB","read_name",
#' "start_alnA","cigar_alnA",
#' "start_alnB","cigar_alnB",
#' "num_chim_aln","max_poss_aln_score",
#' "non_chim_aln_score","this_chim_aln_score",
#' "bestall_chim_aln_score") }
#'
#' - For the 'new' chimeric detection scheme
#' \code{c("chr_donorA","brkpt_donorA",
#'                   "strand_donorA","chr_acceptorB",
#'                   "brkpt_acceptorB","strand_acceptorB",
#'                   "junction_type","repeat_left_lenA",
#'                   "repeat_right_lenB","read_name",
#'                   "start_alnA","cigar_alnA",
#'                   "start_alnB","cigar_alnB")}
#'
#' @keywords internal
#' @param df input
#' @param table_type one in c("STAR","bedpe")
#'
#' @return dataframe with the properly formatted columns
col_check_rename <- function(df, table_type = "STAR") {
    ncol <- dim(df)[2]

    # load without storing globally
    STAR_COLNAMES19 <- get_colnames_and_types_for_input("STAR_COLNAMES19")
    STAR_COLNAMES14 <- get_colnames_and_types_for_input("STAR_COLNAMES14")
    STAR_COLTYPES19 <- get_colnames_and_types_for_input("STAR_COLTYPES19")
    STAR_COLTYPES14 <- get_colnames_and_types_for_input("STAR_COLTYPES14")
    BEDPE_COLNAMES <- get_colnames_and_types_for_input("BEDPE_COLNAMES")
    BEDPE_COLTYPES <- get_colnames_and_types_for_input("BEDPE_COLTYPES")


    if (table_type != "STAR") {
        expected_names <- BEDPE_COLNAMES
        expected_types <- BEDPE_COLTYPES
        n_col_check <- 10
        message("Detected input type generic .bedpe dataframe , expecting 10+ columns \n")
    }
    if (table_type == "STAR") {
        if (all(STAR_COLNAMES19 %in% colnames(df))) {
            expected_names <- STAR_COLNAMES19
            expected_types <- STAR_COLTYPES19
            n_col_check <- 19
            message("Detected input type Chimeric.out.Junction with multimaps, expecting first ", n_col_check, "columns to follow STAR naming\n")
        } else {
            if (all(STAR_COLNAMES14 %in% colnames(df))) {
                expected_names <- STAR_COLNAMES14
                expected_types <- STAR_COLTYPES14
                message("Detected input type Chimeric.out.Junction without multimaps, expecting first ", n_col_check, "columns  to follow STAR naming \n")
                n_col_check <- 14
            } else {
                expected_names <- STAR_COLNAMES14
                expected_types <- STAR_COLTYPES14
                n_col_check <- 14
                message("Specified format is STAR Chimeric.out.Junction, but columns are missing or unnamed \n")
                message("Will guess and validate first 14 columns. Consider transformig input to more general .bedpe format or settignt he column names ")
            }
        }
    }

    df <- as_tibble(df)
    testt <- data.frame(head(df))

    actual_names <- names(testt)
    actual_types <- vapply(
        testt %>% dplyr::select(all_of(seq_len(n_col_check))),
        function(col) class(col), ""
    )

    # Check if there are extra columns
    extra_columns <- setdiff(actual_names, expected_names)

    # check types
    if (all(actual_types == expected_types)) {
        message("Verified the correct column types and order \n")
        # if(length(extra_columns) > 0) {
        #   message("Extra columns are found:", paste(extra_columns, collapse = ", "), "\n")
        # } else {
        #   message("No extra columns were found.\n")
        # }
        # check names
        if (!all(expected_names %in% colnames(testt))) {
            message("Guessing columns")
            df <- df %>%
                rename_with(~expected_names, seq_len(n_col_check))
            status <- data.frame(Before = actual_names[seq_len(n_col_check)], After = names(df)[seq_len(n_col_check)])
            message(paste(status$Before, "\t-> ", status$After, collapse = "\n", sep = ""))
            message("\n ")
        } else {
            message("Column names are correct \n")
        }
    } else { # TYPES INCONSISTENCY

        result_guess <- tryCatch(
            {
                df_tmp <- head(df, 5)
                df_tmp[seq_len(n_col_check)] <- lapply(seq_len(n_col_check), function(i) as(df_tmp[[i]], expected_types[i]))
                TRUE # Return TRUE if successful
            },
            error = function(e) {
                FALSE # Return FALSE if conversion of type is not possible
            }
        )
        # if types can be formatted, do that and check names
        if (result_guess) {
            df[seq_len(n_col_check)] <- lapply(seq_len(n_col_check), function(i) as(df[[i]], expected_types[i]))
            df <- df %>%
                rename_with(~expected_names, all_of(actual_names[seq_len(n_col_check)]))
            status <- data.frame(Before = actual_names[seq_len(n_col_check)], After = names(df)[seq_len(n_col_check)])
            message(paste(status$Before, "\t-> ", status$After, collapse = "\n", sep = ""))
            msgtext <- paste0(colnames(df), sep = " ")
            message("\nGuessed column names and converted types \n", msgtext)
        } else {
            # if something was wrong with types guessing
            stats <- ifelse(actual_types[seq_len(n_col_check)] == expected_types, "ok", "<-problem")
            expected <- paste(seq_len(n_col_check),
                expected_names, ":\t",
                expected_types, "\t|| ",
                actual_names[seq_len(n_col_check)], ":\t", actual_types[seq_len(n_col_check)], "\t:",
                stats,
                collapse = "\n"
            )
            message("Problem with the input format found")
            message("Expected name :\t type \t || Found name : type\n", expected, sep = "")
            stop("\nColumn names, types, or order do not match the expected criteria.")
        }
    }
    return(df)
}

#' Processing of of the STAR SE Chimeric.junction.out
#'
#' Calculates alignment coordinates and returns reads with categories
#'
#' #' \describe{
#'   \item{multimap}{ multi-mapped read}
#'   \item{multigap}{ more than one junction (more than two 'N' in CIGAR string)}
#'   \item{bad junction}{ Artifacts. I.e alignments for both arms are continious, but with 'backward' chimeric junction was wrongly put}
#' }
#' @keywords interal
#' @param dt Chimeric.out.junction with the correct column names
#' @param keep_all_columns - TRUE or FALSE. Keep CIGAR strings and junction coordinate columns
#' @seealso [col_check_rename()]
#' @return tibble with annotated reads
preproc_chim_junction_out_se <- function(dt, keep_all_columns = FALSE) {
    message("processing raw Chimeric.junction.out...")

    if (!("this_chim_aln_score" %in% colnames(dt))) {
        dt$this_chim_aln_score <- 1
        dt$num_chim_aln <- 1
    }
    if (!keep_all_columns) {
        dt <- dt %>% dplyr::select(c(
            read_name,
            chr_donorA, chr_acceptorB,
            start_alnA, start_alnB,
            strand_donorA, strand_acceptorB,
            cigar_alnA, cigar_alnB,
            brkpt_donorA, brkpt_acceptorB, this_chim_aln_score,
            num_chim_aln
        ))
    }
    # rename, count gaps
    dt <- dt %>%
        dplyr::rename(
            readname = read_name,
            chromA = chr_donorA,
            strandA = strand_donorA,
            startA = start_alnA,
            chromB = chr_acceptorB,
            strandB = strand_acceptorB,
            startB = start_alnB,
            score = this_chim_aln_score
        ) %>%
        mutate(
            ngapsA = str_count(cigar_alnA, "N"),
            ngapsB = str_count(cigar_alnB, "N")
        ) %>%
        mutate(
            lengapsA = get_char_count_cigar(cigar_alnA, "N"),
            lengapsB = get_char_count_cigar(cigar_alnB, "N")
        )

    message("getting alignment coordinates ")
    dt <- dt %>%
        mutate(
            endA = startA + GenomicAlignments::cigarWidthAlongReferenceSpace(cigar_alnA),
            endB = startB + GenomicAlignments::cigarWidthAlongReferenceSpace(cigar_alnB),
            bad_junction = ifelse(chromA == chromB &
                strandA == strandB &
                abs(brkpt_donorA - brkpt_acceptorB) <= 2, 1, 0),
            bad_junction = ifelse(
                chromA == chromB &
                    strandA == strandB &
                    (abs(startA - endB) <= 3 | abs(endA - startB) <= 3), 1, bad_junction
            ),
            multigap = as.integer((lengapsA >= 2) | (lengapsB >= 2)),
            multimap = as.integer(num_chim_aln > 1)
        ) %>%
        relocate(endB, .after = startB) %>%
        relocate(endA, .after = startA)

    if (!keep_all_columns) {
        dt <- dt %>% dplyr::select(-c(
            brkpt_donorA, brkpt_acceptorB,
            cigar_alnA, cigar_alnB, num_chim_aln
        ))
    }
    return(dt)
}

#' Processing of of the STAR PE Chimeric.junction.out
#'
#' Calculates alignment coordinates and returns reads with categories
#'
#' #' \describe{
#'   \item{multimap}{ multi-mapped read}
#'   \item{multigap}{ more than one junction (more than two 'N' in CIGAR string)}
#'   \item{bad junction}{ Artifacts. I.e alignments for both arms are continious, but with 'backward' chimeric junction was wrongly put}
#' }
#' @keywords internal
#' @param dt Chimeric.out.junction with the correct column names
#' @param keep_all_columns - TRUE or FALSE. Keep CIGAR strings and junction coordinate columns
#'
#' @return tibble with annotated reads
preproc_chim_junction_out_pe <- function(dt, keep_all_columns = FALSE) {
    message("processing raw Chimeric.junction.out... PE")
    bad_junction_tr <- 6
    if (!keep_all_columns) {
        dt <- dt %>% dplyr::select(c(
            read_name,
            chr_donorA, chr_acceptorB,
            start_alnA, start_alnB,
            strand_donorA, strand_acceptorB,
            cigar_alnA, cigar_alnB,
            brkpt_donorA, brkpt_acceptorB, this_chim_aln_score,
            num_chim_aln
        ))
    }

    # rename, count gaps
    message("getting alignment coordinates ")
    dt <- dt %>%
        dplyr::rename(
            readname = read_name,
            chromA = chr_donorA,
            strandA = strand_donorA,
            startA = start_alnA,
            chromB = chr_acceptorB,
            strandB = strand_acceptorB,
            startB = start_alnB,
            score = this_chim_aln_score
        ) %>%
        mutate(
            ngapsA = str_count(cigar_alnA, "N"),
            ngapsB = str_count(cigar_alnB, "N")
        ) %>%
        add_column(
            lengapsA = 0,
            lengapsB = 0
        ) %>%
        mutate(
            cigar_str = cigar_alnA,
            has_p = str_count(cigar_str, "p"),
            minus_p = str_count(cigar_str, "-"),
            sign_p = if_else(minus_p == 1, -1, 1 * has_p),
            n_p = get_char_count_cigar(cigar_str, "p"),
            n_s = get_char_count_cigar(cigar_str, "S"),
            n_m = get_char_count_cigar(cigar_str, "M"),
            n_i = get_char_count_cigar(cigar_str, "I"),
            n_n = get_char_count_cigar(cigar_str, "N"),
        ) %>%
        mutate(
            lengapsA = n_n,
            endA_along = startA + cigarWidthAlongReferenceSpace(cigar_str),
            lA = n_m + n_p * sign_p + n_n,
            endA = startA + lA - 1
        ) %>%
        mutate(
            cigar_str = cigar_alnB,
            has_p = str_count(cigar_str, "p"),
            minus_p = str_count(cigar_str, "-"),
            sign_p = if_else(minus_p == 1, -1, 1 * has_p),
            n_p = get_char_count_cigar(cigar_str, "p"),
            n_s = get_char_count_cigar(cigar_str, "S"),
            n_m = get_char_count_cigar(cigar_str, "M"),
            n_i = get_char_count_cigar(cigar_str, "I"),
            n_n = get_char_count_cigar(cigar_str, "N")
        ) %>%
        mutate(
            lengapsB = n_n,
            lB = n_m + n_p * sign_p + n_n,
            endB_along = startB + cigarWidthAlongReferenceSpace(cigar_str),
            endB = startB + lB - 1
        ) %>%
        mutate(
            bad_junction = ifelse(chromA == chromB &
                strandA == strandB &
                (abs(startA - endB) <= 3) |
                (abs(endA - startB) <= 3), 1, 0),
            multigap = as.integer((lengapsA >= 2) | (lengapsB >= 2)),
            multimap = as.integer(num_chim_aln > 1)
        ) %>%
        relocate(endB, .after = startB) %>%
        relocate(endA, .after = startA) %>%
        dplyr::select(-c(
            cigar_str, has_p, minus_p, sign_p,
            n_p, n_s, n_m, n_i, n_n,
            ngapsA, ngapsB, lengapsA, lengapsB, lA, lB
        ))

    if (!keep_all_columns) {
        dt <- dt %>% dplyr::select(-c(
            brkpt_donorA, brkpt_acceptorB,
            cigar_alnA, cigar_alnB, num_chim_aln
        ))
    }

    return(dt)
}

#' Preprocess .bedpe input
#'
#' Searches for the multi-mapped and bad reads (overlapping arms)
#' Adds 'multimap', 'bad_junction' columns filled with 0 or 1 and 'multigap' = 0
#' for consistency with other pre-processing methods
#'
#' @keywords internal
#' @param dt dataframe with reads aligned to strictly two loci
#' @param keep_all_columns keeep columns apart form the required from .bedpe format
#'
#' @return pre-processed dataframe
preproc_generic <- function(dt, keep_all_columns = TRUE) {
    message("processing generic bedpe input...")
    BEDPE_COLNAMES <- get_colnames_and_types_for_input("BEDPE_COLNAMES")
    if (!keep_all_columns) {
        dt <- dt %>% dplyr::select(all_of(BEDPE_COLNAMES))
    }
    dt <- dt %>% mutate(
        bad_junction = 0,
        multimap = 0,
        multigap = 0
    )
    dt <- dt %>%
        mutate(
            bad_junction = ifelse(chromA == chromB & strandA == strandB &
                (abs(startA - endB) <= 3 | abs(endA - startB) <= 3), 1, bad_junction)
        ) %>%
        group_by(readname) %>%
        mutate(
            nr = n(),
            multimap = ifelse(nr != 1, 1, multimap)
        ) %>%
        ungroup()
    dt$nr <- NULL
    if (!("score" %in% colnames(dt))) {
        message("no alignment score column found. Will create one for consistency")
        dt$score <- 1
    }


    return(dt)
}

#' Preprocess GInteractions input
#'
#' Searches for the multi-mapped reads (overlapping arms)
#' Adds 'multimap', 'bad_junction' columns filled with 0/1 and 'multigap' = 0
#' for consistency with other pre-processing methods.
#'
#' @keywords internal
#' @param gi_raw `GInteractions` with inpit RNA interactions
#' @param keep_all_columns keep columns apart from those required by .bedpe format
#'
#' @return pre-processed dataframe
preproc_generic_gi <- function(gi_raw, keep_all_columns = TRUE) {
    message("processing generic GInteractions input...")
    BEDPE_COLNAMES <- get_colnames_and_types_for_input("BEDPE_COLNAMES")
    cnames <- colnames(mcols(gi_raw))
    dt <- makeDfFromGi(gi_raw)
    if (!("readname" %in% cnames)) {
        message("no radname column found. Will create one for consistency")
        dt$readname <- seq_along(gi_raw)
    }
    if (!("score" %in% cnames)) {
        message("no alignment score column found. Will create one for consistency")
        dt$score <- 1
    }
    if (!keep_all_columns) {
        dt <- dt %>% dplyr::select(any_of(BEDPE_COLNAMES), score)
    }
    dt <- dt %>% mutate(
        bad_junction = 0,
        multimap = 0,
        multigap = 0
    )
    dt <- dt %>%
        mutate(
            bad_junction = ifelse(chromA == chromB & strandA == strandB &
                (abs(startA - endB) <= 3 | abs(endA - startB) <= 3), 1, bad_junction)
        ) %>%
        group_by(readname) %>%
        mutate(
            nr = n(),
            multimap = ifelse(nr != 1, 1, multimap)
        ) %>%
        ungroup()
    dt$nr <- NULL

    return(dt)
}


#' Run pre-processing of chimeric reads input
#' @description
#' Imports dataframe with reads (*.bedpe* or *Chimeric.out.junction* ) or `GInteractions`
#' object.
#' Checks column names or tries to quess them if not provided.
#' Adds necessary annotation depending on the input type,
#' For *STAR* input, calculates length of the alignments
#' and marks unique 2-arm alignments.
#' For the *.bedpe* or `GInteractions` input, all entries are already represented as reads
#' with two different aligned parts (2-arm), so only check for unique readname
#' is performed.
#'
#' @details
#' If not existed, adds fields required for the downstream steps:
#' 'readname', 'map_type', 'score', 'n_reads'.
#' 'map_type' field determines the type of the chimeric read:
#' \describe{
#'   \item{multimap}{ multi-mapped read}
#'   \item{multigap}{ more than one junction (more than two 'N' in CIGAR string)}
#'   \item{bad junction}{ Artifacts or possibly unaccounted types.
#'   I.e alignments for both arms are continuous, but with 'backward' chimeric
#'   junction was wrongly introduced in the mapping}
#' }
#' @param data Either dataframe-like object: *Chimeric.out.junction* from *STAR* or
#' *.bedpe* - formatted or `GInteractions` object from \pkg{InteractionSet} package
#' @param keep_metadata  `c(TRUE,FALSE)` Whether extra fields like
#'  CIGAR strings and junction coordinates should be kept
#' @param table_type in \code{c("STAR","bedpe")} for Chimeric.out.Junction or generic input
#' @param library_type \code{c("SE","PE")} for pair- or single- end input
#' @param return_gi if the return object should be `GInteractions`
#' @param  min_arm_len minimum allowed length of the alignment arm.
#' Read will be dropped if either arm is shorter
#' @return tibble with new metadata fields OR GInteractions if `return_gi` is
#' set to TRUE
#' @export
#' @examples
#' # load data
#' data(RNADuplexesSampleData)
#' # with bedpe input
#' preproc_reads <- runDuplexDiscoPreproc(RNADuplexesRawBed, table_type = "bedpe")
#' # with STAR input
#' preproc_reads_star <- runDuplexDiscoPreproc(RNADuplexesRawChimSTAR,
#'     table_type = "STAR",
#'     keep_metadata = FALSE
#' )
runDuplexDiscoPreproc <- function(data, table_type,
    library_type = "SE",
    keep_metadata = TRUE,
    return_gi = FALSE,
    min_arm_len = 15) {
    # Start with determining the input data type
    if (is(data, "GInteractions")) {
        # GI input
        message("Detected input type is GInteractions")
        df <- preproc_generic_gi(gi_raw = data, keep_all_columns = keep_metadata)
    } else {
        # DF input
        if (is(data, "data.frame")) {
            message("Input type is table")
            if (!table_type %in% c("STAR", "bedpe")) {
                stop("Input type of table is not specified or not accepted. Accepted values are: bedpe, STAR ")
            }
            # Table input preproc
            if (!(table_type %in% c("STAR", "bedpe"))) {
                message("Wrong type of the table provided. Use 'STAR' for
                Chimeric.out.Junction or \n 'bedpe' for generic bedpe format \n")
            }
            df <- data
            df <- col_check_rename(df, table_type)

            if (table_type == "STAR" && (library_type %in% c("SE", "PE"))) {
                if (library_type == "SE") {
                    df <- preproc_chim_junction_out_se(df, keep_all_columns = keep_metadata)
                }
                if (library_type == "PE") {
                    df <- preproc_chim_junction_out_pe(df, keep_all_columns = keep_metadata)
                }
            } else {
                df <- preproc_generic(df, keep_all_columns = keep_metadata)
            }
        } else {
            stop("cannot determine input data type. Provided : ", class(data)[1])
        }
    }

    # Add alignment length
    df <- df %>% mutate(
        aln_lenA = endA - startA,
        aln_lenB = endB - startB
    )

    # Add number of reads per record (should be 1, but check in case of collapsed input)
    if (!("n_reads" %in% colnames(df))) {
        df <- df %>% mutate(
            n_reads = 1,
            read_id = seq_len(nrow(df))
        )
    } else {
        df <- df %>% add_column(read_id = seq_len(nrow(df)))
    }
    df <- df %>% mutate(
        len_too_short = as.integer(!(aln_lenA > min_arm_len &
            aln_lenB > min_arm_len))
    )

    df <- df %>% mutate(map_type = case_when(
        multigap == 0 & multimap == 1 & bad_junction == 0 ~ "multi_map",
        multigap == 1 & multimap == 0 & bad_junction == 0 ~ "multi_split",
        multigap == 1 & multimap == 1 & bad_junction == 0 ~ "multi_split&map",
        bad_junction == 1 ~ "not_chim",
        len_too_short == 1 ~ "too_short",
        multigap == 0 & multimap == 0 & bad_junction == 0 ~ "2arm",
        .default = "notype"
    ))

    df <- df %>%
        dplyr::select(-c(multigap, multimap, bad_junction)) %>%
        dplyr::relocate(c(chromA, startA, endA, strandA, chromB, startB, endB, strandB, readname, map_type, score), .before = everything())
    if (!keep_metadata) {
        df <- df %>% dplyr::select(c(chromA, startA, endA, strandA, chromB, startB, endB, strandB, readname, map_type, score, n_reads))
    }
    if (return_gi) {
        df <- makeGiFromDf(df)
    }
    return(df)
}


.filterAlignmentLength <- function(dt, minlen) {
    dt <- dt %>% dplyr::mutate(len_not_ok = !(aln_lenA > minlen & aln_lenB > minlen))
    nf <- sum(as.integer(dt$len_not_ok))
    nr <- nrow(dt)
    dt <- dt %>% dplyr::filter(!len_not_ok)
    dt$len_not_ok <- NULL
    message(
        "Filtered out : ", nf,
        " too short alignments out of ", nr, " : ", round(nf / nr * 100, 2), " %"
    )
    return(dt)
}

# accepts single gap df
# integrate after preproc, and after SJ ?
#' Extract regions around chimeric junction
#'
#' Trim alignements to contain only 'extract len' nucleotides adajcent
#' to the chimeric junction
#'
#' @details
#' In case of the long alignemtns, it may be necessary trim chimeric alignments
#' to identify RNA duplex. If 'extract_len' is longer than the read
#' alignemnt length, then no trimming is performed
#'
#' @param dt table with the
#' @param extract_len the length of the seqeunce to trim
#'
#' @return dataframe with the trimmed alignments
#' @export
#' @examples
#' data("RNADuplexesSampleData")
#' dt_preproc <- runDuplexDiscoPreproc(RNADuplexesRawChimSTAR,
#'     table_type = "STAR", library_type = "SE"
#' )
#' trimAroundJunction(dt_preproc, 40)
#'
trimAroundJunction <- function(
        dt,
        extract_len = 30) {
    cnames <- c("brkpt_donorA", "brkpt_acceptorB")
    dift <- 2
    extract_len <- extract_len - 1

    if (!all(cnames %in% colnames(dt))) {
        message("cannot extract region around junction
            because junction fields are not present")
        return(dt)
    }

    # Types of junction arrangement (point of ligation)
    # t1: normal ==* *==
    # t2: backwards *== ==*
    # t3: A normal B backwards  ==* ==*
    # t2: B normal A backwards *== *==
    if (all(c("jA", "jB") %in% colnames(dt))) {
        # pass
    } else {
        if (all(cnames %in% colnames(dt))) {
            dt <- dt %>% rename(jA = brkpt_donorA, jB = brkpt_acceptorB)
        } else {
            message("Junction fields not found: Possible filed names are jA and JB or
                brkpt_donorA and brkpt_acceptorB ")
        }
    }

    dt1 <- dt %>%
        mutate(
            t1 = if_else((abs(endA - jA) <= dift) & (abs(startB - jB) <= dift), 1, 0),
            t2 = if_else((abs(startA - jA) <= dift) & (abs(endB - jB) <= dift), 1, 0),
            t3 = if_else((abs(endA - jA) <= dift) & (abs(endB - jB) <= dift), 1, 0),
            t4 = if_else((abs(startA - jA) <= dift) & (abs(startB - jB) <= dift), 1, 0),
            any1 = t1 + t2 + t3 + t4
        )


    if (!any(dt1$any1 > 1) & !any(dt1$any1 == 0)) {
        message("type detected")
    } else {
        message("some problem occured in detection of junction type.
            Check alignment lengths")
    }

    dt2 <- dt1 %>%
        tidyr::pivot_longer(
            cols = all_of(c("t1", "t2", "t3", "t4")),
            names_to = "junctype",
            values_to = "value"
        ) %>%
        filter(value == 1) %>%
        select(-value)


    dt_modified <- dt2 %>%
        mutate(
            aln_lenA = endA - startA,
            aln_lenB = endB - startB,
            extrA = pmin(aln_lenA, extract_len),
            extrB = pmin(aln_lenB, extract_len)
        ) %>%
        mutate(
            startA = case_when(
                junctype == "t1" ~ endA - extrA,
                junctype == "t2" ~ startA,
                junctype == "t3" ~ endA - extrA,
                junctype == "t4" ~ startA
            ),
            endA = case_when(
                junctype == "t1" ~ endA,
                junctype == "t2" ~ startA + extrA,
                junctype == "t3" ~ endA,
                junctype == "t4" ~ startA + extrA
            ),
            startB = case_when(
                junctype == "t1" ~ startB,
                junctype == "t2" ~ endB - extrB,
                junctype == "t3" ~ endB - extrB,
                junctype == "t4" ~ startB
            ),
            endB = case_when(
                junctype == "t1" ~ startB + extrB,
                junctype == "t2" ~ endB,
                junctype == "t3" ~ endB,
                junctype == "t4" ~ startB + extrB
            )
        )
    # dt_modified$junctype = NULL
    return(dt_modified)
}

#' Compare multiple RNA-RNA interactions sets
#'
#' Combines all interaction into single superset by clustering & collapsing.
#' Then compares every input entry with the superset. Overlaps between superset and
#' inputs are recorded in a table as 0/1
#'
#' @param gi_samples_list anmes list with the `GInteractions` entries list('sample1'=gi1,'sample2'='gi2)
#' @param min_ratio If the overlap-to-span ratio for either arm (A or B) for pair
#' of chimeric reads is less than \code{min_arm_ratio}, then the total overlap for this pair is set to zero.
#' Relevant to comparison of superset vs individual samples
#' @param minoverlap Parameter for read clustering to create a superset. Minimum required overlap to for either arm (A or B) for pair of entries.
#' @param maxgap Parameter for read clustering. Minimum required shift between start and end coordinates of arms for pair of overlapping entries..
#' If the shift is longer than \code{max_gap} for either arm, then total read overlap between those reads is zero.
#' @param gi_superset Optional. Superset defining the space (all) of the interactions, against which inputs from the list will be compared.
#' @param niter Internal parameter for debugging. Number of cluster& collapse iterations to find superset
#' @param anno_gr Optional. `Granges` to annotate superset.
#' @return dataframe recodding the overlaps between samples and supeset
#' @export
#' @examples
#' # Create test set of RNA interactions
#' chrom <- "chr1"
#' start1 <- c(1, 11, 21, 31, 41, 51, 61, 71, 81, 91)
#' end1 <- start1 + 9
#' start2 <- c(101, 111, 121, 131, 141, 151, 161, 171, 181, 191)
#' end2 <- start2 + 9
#'
#' anchor1 <- GRanges(seqnames = chrom, ranges = IRanges(start = start1, end = end1))
#' anchor2 <- GRanges(seqnames = chrom, ranges = IRanges(start = start2, end = end2))
#'
#' interaction <- GInteractions(anchor1, anchor2)
#'
#' # Ensure some overlaps
#' n <- length(interaction)
#' group_size <- ceiling(n / 2)
#' group_indices1 <- sort(sample(seq_len(n), group_size))
#' group_indices2 <- sort(sample(seq_len(n), group_size))
#' group_indices3 <- sort(sample(seq_len(n), group_size))
#'
#' # Create separate GInteractions objects for each group
#' group1 <- interaction[group_indices1]
#' group2 <- interaction[group_indices2]
#' group3 <- interaction[group_indices3]
#'
#' # format input and call comparison
#' a <- list("sample1" = group1, "sample2" = group2, "sample3" = group3)
#' res <- compareMultipleInteractions(a)
#' # comparison result
#' head(res$dt_upset)
#' # superset
#' res$gi_all
#' # dataframe for the Upset plot
#' res$dt_upset
compareMultipleInteractions <- function(gi_samples_list,
    min_ratio = 0.3,
    minoverlap = 5,
    maxgap = 50,
    niter = 3,
    gi_superset = NULL,
    anno_gr = NULL) {
    if (!is.null(gi_superset)) {
        message("Using provided superset of interactions")
        gi_all <- gi_superset
        gi_all$n_reads <- 1

        gi_all <- refresh_gi(gi_all)
        gi_all$duplex_id <- seq_len(length(gi_all))

        statdf <- tibble(
            "read_id" = seq_len(length(gi_all)),
            "duplex_id" = seq_len(length(gi_all))
        )
        gi_all_inter <- gi_all
        gi_all_inter$id <- seq_len(length(gi_all_inter))
    } else {
        message("Gathering superset of interactions")
        gi_all <- GInteractions()
        for (i in seq_len(length(gi_samples_list))) {
            samplename <- names(gi_samples_list[i])
            gi_all <- c(gi_all, gi_samples_list[[i]])
        }
        gi_all <- refresh_gi(gi_all)
        gi_all$duplex_id <- seq_len(length(gi_all))
        gi_all$score <- 1
        statdf <- tibble(
            "read_id" = seq_len(length(gi_all)),
            "duplex_id" = seq_len(length(gi_all))
        )

        gi_all_inter <- collapseSimilarChimeras(
            gi = gi_all, niter = niter, minoverlap = minoverlap,
            maxgap = maxgap, read_stats_df = statdf
        )$gi_updated
        gi_all_inter$id <- seq_len(length(gi_all_inter))
    }


    if (!is.null(anno_gr)) {
        gi_all_inter <- annotateGI(gi_all_inter, anno_gr)
        gi_all_inter <- .annotateCisTrans(gi_all_inter)
    }


    dt_upset <- (data.frame(matrix(
        ncol = length((gi_samples_list)) + 1,
        nrow = length(gi_all_inter),
        data = 0
    )))
    colnames(dt_upset) <- c("id", names(gi_samples_list))
    dt_upset$id <- gi_all_inter$id
    if (!is.null(gi_all_inter$cis)) {
        dt_upset$cis <- gi_all_inter$cis
    }
    dt_upset$n_reads <- gi_all_inter$n_reads
    for (i in seq_len(length(gi_samples_list))) {
        samplename2 <- names(gi_samples_list[i])
        message("Superset vs ", samplename2)
        first <- gi_all_inter
        second <- assign_name_to_gi(gi_samples_list[[i]], sample_name = samplename2)
        # unset maxgap because we compare against less-defined superset
        ovldt <- computeGIPairOverlaps(first, second,
            minoverlap = minoverlap,
            maxgap = 100
        )

        if (nrow(ovldt) == 0) {
            message("no overlap")
            next
        }
        ovl_interaction_ids <- unique(ovldt$id.1)
        dt_upset[dt_upset$id %in% ovl_interaction_ids, ][samplename2] <- 1
    }
    res <- list()
    res$gi_all <- gi_all_inter
    res$dt_upset <- dt_upset
    return(res)
}

computeGIPairOverlaps <- function(gi1, gi2, minoverlap = 10,
    maxgap = 10) {
    # this is to unify levels
    # otherwise A-B B-A matches would be messed between query an dsubject
    mcols(gi1) <- NULL
    mcols(gi2) <- NULL
    runi <- unify_gi_levels(gi1, gi2)
    gi1 <- runi$gi1
    gi2 <- runi$gi2
    fo <- findOverlaps(gi1, gi2,
        ignore.strand = FALSE,
        type = "equal", use.region = "both", maxgap = maxgap,
        minoverlap = minoverlap, select = "all"
    )

    dt <- tibble(
        id.1 = queryHits(fo),
        id.2 = subjectHits(fo),
        A_span = width(punion(gi1@regions[gi1[queryHits(fo)]@anchor1],
            gi2@regions[gi2[subjectHits(fo)]@anchor1],
            fill.gap = TRUE
        )),
        B_span = width(punion(gi1@regions[gi1[queryHits(fo)]@anchor2],
            gi2@regions[gi2[subjectHits(fo)]@anchor2],
            fill.gap = TRUE
        )),
        A_ovl = width(pintersect(
            gi1@regions[gi1[queryHits(fo)]@anchor1],
            gi2@regions[gi2[subjectHits(fo)]@anchor1]
        )),
        B_ovl = width(pintersect(
            gi1@regions[gi1[queryHits(fo)]@anchor2],
            gi2@regions[gi2[subjectHits(fo)]@anchor2]
        ))
    ) %>% mutate(ratio.A = (A_ovl / A_span), ratio.B = (B_ovl) / (B_span))
    gc()
    return(dt)
}

# Use artificial interactions
chrom <- "chr1"
start1 <- c(1, 5, 6, 5, 45, 45, 30, 30, 90, 90, 110, 10, 45, 48, 47)
end1 <- start1 + 10
start2 <- c(100, 100, 95, 100, 130, 130, 70, 70, 120, 120, 200, 200, 70, 68, 67)
end2 <- start2 + 10
expected_ovl_1stgroup <- 3
anchor1 <- GRanges(seqnames = chrom, ranges = IRanges(start = start1, end = end1))
anchor2 <- GRanges(seqnames = chrom, ranges = IRanges(start = start2, end = end2))

interaction <- GInteractions(anchor1, anchor2, mode = "strict")

n <- length(interaction)
group_size <- ceiling(n / 3) # should be 5
set.seed(123)
interaction$groups <- sample(rep(1:3, length.out = n))
# Split the dataframe into three groups
group1 <- interaction[interaction$groups == 1]
group2 <- interaction[interaction$groups == 2]
group3 <- interaction[interaction$groups == 3]
test_that("comparing samples works", {
    suppressMessages({
        a <- list("sample1" = group1, "sample2" = group2, "sample3" = group3)
        res <- compareMultipleInteractions(a,
            min_ratio = 0.0,
            minoverlap = 1
        )
        n_reads_real <- sum(res$gi_all$n_reads)
        len_superset <- length(res$gi_all)
        df_res <- res$dt_upset

        real_olv_1st <- sum(df_res[df_res$id == 1, c("sample1", "sample2", "sample3")])
        real_olv_six <- sum(df_res[df_res$id == 6, c("sample1", "sample2", "sample3")])
    })

    expect_equal(n_reads_real, 15, label = "superset size counted reads")
    expect_equal(len_superset, 7, label = "superset size is correct")
    expect_equal(real_olv_1st, expected_ovl_1stgroup,
        label = "test overlap correct:1"
    )
    expect_equal(real_olv_six, 1,
        label = "test overlap correct:6"
    )
})

#' Chimeric reads of SPLASH
#'
#' A Chimeric.out.Junction file with a subset of chr 22 Chimeric reads
#' detected by SPLASH protocol in Human embryonic stem cells.
#' @source [SequenceReadArcive](https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943)
#' Reads were aligned with STAR
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details on the pre-processing and sub-setting the
#' data
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns `tibble` with columns of Chimeric.junction.out
"RNADuplexesRawChimSTAR"

#' Chimeric reads of SPLASH converted to .bedpe fromat
#'
#' A Chimeric.out.Junction file with a subset of chr 22 Chimeric reads
#' detected by SPLASH protocol in Human embryonic stem cells.
#' @source [SequenceReadArcive](https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943)
#' Reads were aligned with STAR and filtered to contain only reads
#' which could be represented as 2-arm chimeric alignments. Converted to the
#' [bedpe](https://bedtools.readthedocs.io/en/latest/content/general-usage.html#bedpe-format)
#' format
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details on the pre-processing and sub-setting the
#' data
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns `tibble` with columns of bedpe format
"RNADuplexesRawBed"

#' RNA duplex reads of SPLASH, clustered and assigned to duplex groups
#'
#' `GInteractions` read-level object containing processed reads,annotated with duplex group
#'  ids, read types gene names and p-values
#' @source [SequenceReadArcive](https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943)
#' Reads were aligned with STAR and duplex groups were identified
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details on the data generation proccedure.
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns `GInteractions` with
#' - `n_reads_dg ` : number of reads in the duplex group (DG)
#' - `duplex_id  ` : temporary id for RNA duplexes which could be found before
#' clustering (duplicated or shifted by couple of nt )
#' - `dg_id` :id of the duplex group
#' - `score` : median alignment score in duplex group
#' - other columns inherited from the STAR Chimeric.out.Junction
"RNADuplexSampleClustReads"


#' RNA duplex reads of SPLASH derived from chimeric alignments
#'
#' `GInteractions` read-level object containing two-arm chimeric reads extracted from
#' mapping output and which can be represented in the `GInteraction` object
#' @source [SequenceReadArcive](https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943)
#' @details
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details on the data generation proccedure.
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns `GInteractions` with
#' - `readname` : read name
#' - `map_type ` : type of the mapped read (2arm by design of pre-filtering)
#' - `junction_type` : if read jucntion is too short, or it not a 'true' ligated
#' reads because of the jucntoin coincides with splice junction
#' - cigar_aln* columns inherited from the STAR Chimeric.out.Junction output
"RNADuplexSampleGI"



#' RNA duplex reads of SPLASH, clustered and collapsed to duplex groups
#'
#' `GInteractions` duplex group -level object containing detected duplex groups,
#' annotated with duplex group ids, gene_names and p-values
#' @source [SequenceReadArcive](https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943)
#' Reads were aligned with STAR and duplex groups were identified
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details on the data generation procedure.
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns `GInteractions` with
#' - `n_reads` : number of reads in the duplex group (DG)
#' - `dg_id` :id of the duplex group
#' - `p_val` : BH adjusted p-value of testing to reject hypothesis of DG arising
#'  from random ligation
#' - `score` : median alignment score in duplex group
#' - other columns with `.A` and `.B` annotating to which genes either arm of the
#' DG maps
"RNADuplexSampleDGs"


#' Gene coordinates on human chromosome 22
#'
#' `Granges` containing gene coordinates of human chromosome 22 obtained from
#' GENCODEv44 annotaion
#' @source [GENCODEv44](https://www.gencodegenes.org/human/release_44.html)
#' @details
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns statdatd GENCODE gtf fields
"SampleGeneAnnoGR"


#' Gene coordinates on human chromosome 22
#'
#' `Granges` containing coordinates of splice junctions human chromosome 22 obtained from
#' GENCODEv44 annotaion
#' @source [GENCODEv44](https://www.gencodegenes.org/human/release_44.html)
#' @details
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns statdatd GENCODE gtf fields
"SampleSpliceJncGR"


#'  Gene counts on human chromosome 22, embryonic stem cells
#'
#' File generated by mapping with STAR using `--quantMode GeneCounts`
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details on the pre-processing and sub-setting the
#' @source [SequenceReadArcive](https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943)
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSampleData)
#' @returns `tibble` with columns of Chimeric.junction.out
"RNADuplexesGeneCounts"

#' RNA duplex reads of SPLASH derived from chimeric alignments
#'
#' `GInteractions` object containing two-arm chimeric reads extracted from
#' mapping output and which can be represented in the `GInteraction` object
#' and subset to chr22: 23877144-45562960 '*'
#' @source [SequenceReadArcive](https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943)
#' @details
#' see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
#' "DuplexDiscovereR")} for details on the data generation procedure.
#' @docType data
#' @keywords datasets
#' @usage data(RNADuplexesSmallGI)
#' @name SampleSmallGI
"SampleSmallGI"

#' Wrapper for classification of the 2arm chimeric reads
#'
#' @description
#' Wraps two procedures for different types of classification for read alignment:
#'  \describe{
#'   \item{overlap type}{test if chimeric junction map to two non-overlapped regions or shorter than defined minimum distance}
#'   \item{splice junction}{test if chimeric junction is also a splice junction}
#' }
#' @details
#' Calls detection of the chimeric junction type, annotates short junctions
#' on same chromosome an strand as 'short'. Compares chimeric
#' junctions with splice junctions. Adds results as the new metadata fields
#' parallel to the input.
#'
#' @param gi `GInteractions` object
#' @param min_junction_len minimum allowed distance between two chimeric arms
#' @param junctions_gr `Granges` object with the splice junctions coordinates
#' @param max_sj_shift maximum shift between either donor and acceptor splice
#' sites and corresponding chimreic junction coordinates to count chimeric
#' junction as splice junction
#' @returns `GInteractions` object object of the same size with new columns:
#' \describe{
#' \item{splicejnc}{filled with 0 or 1}
#' \item{junction_type}{factor for the junction types}
#' }
#' @export
#' @seealso [DuplexDiscovereR::getChimericJunctionTypes(),
#' DuplexDiscovereR::getSpliceJunctionChimeras()]
#' @examples
#' data("RNADuplexesSampleData")
#' head(RNADuplexSampleGI)
#' # remove all metadata
#' mcols(RNADuplexSampleGI) <- NULL
#' gi <- classifyTwoArmChimeras(RNADuplexSampleGI,
#'     min_junction_len = 5,
#'     junctions_gr = SampleSpliceJncGR, max_sj_shift = 10
#' )
#' table(gi$splicejnc)
#' table(gi$junction_type)
classifyTwoArmChimeras <- function(
        gi, min_junction_len = 4,
        junctions_gr, max_sj_shift = 4) {
    gi <- getChimericJunctionTypes(gi, normal_gap_threshold = min_junction_len)
    gi <- getSpliceJunctionChimeras(gi,
        sj_gr = junctions_gr,
        sj_tolerance = max_sj_shift, sj_tolerance_strict = max_sj_shift
    )

    return(gi)
}


#' Classify chimeric junctions of two-arm reads into types
#' @description
#' Chimeric reads which can be represented ans two-arm interactions can be
#' divided into several categories based on the distance between the chimeric
#' fragments and existence of the overlap between these fragments.
#' @details
#' Takes `GInteractions` object and classifies junctions into following categories
#' \describe{
#'   \item{2arm}{ normal chimeric read }
#'   \item{2arm_short}{ normal chimeric read with junction < \emph{normal_gap_threshold}}
#'   \item{self_ovl}{ arms overlap}
#'   \item{antisense_ovl}{ arms overlap on the opposite strand}
#' }
#' @param gi \code{GInteractions} object
#' @param normal_gap_threshold minimum allowed distance between chimeric arms
#
#' @return gi object of the same size with the 'junction_type' field added
#' @export
#' @examples
#' data("RNADuplexesSampleData")
#' preproc_df <- runDuplexDiscoPreproc(RNADuplexesRawBed, table_type = "bedpe")
#' preproc_gi <- makeGiFromDf(preproc_df)
#' preproc_gi <- getChimericJunctionTypes(preproc_gi)
#' table(preproc_gi$junction_type)
getChimericJunctionTypes <- function(gi, normal_gap_threshold = 10) {
    message("\n--- filtering 2-arm duplexes by junction type  ---")
    gi$junction_type <- factor(x = "2arm", levels = c(
        "2arm",
        "2arm_short",
        "self_ovl",
        "antisense_ovl"
    ))
    gi$idx <- seq_len(length(gi))
    gi$gap <- pairdist(gi, type = "gap")
    gi$intra <- pairdist(gi, type = "intra")
    gi$same_strand <- as.integer(gi@regions[gi@anchor1]@strand == gi@regions[gi@anchor2]@strand)

    expr <- ((gi$intra == TRUE) &
        (gi$same_strand == 1) &
        !is.na(gi$gap) &
        (gi$gap < normal_gap_threshold))
    if (any(expr)) {
        gi[expr]$junction_type <- "2arm_short"
    }

    expr <- ((gi$intra == TRUE) &
        (gi$same_strand == 0) &
        (gi$gap < 0))
    if (any(expr)) {
        gi[expr]$junction_type <- "antisense_ovl"
    }


    expr <- ((gi$intra == TRUE) &
        (gi$same_strand == 1) &
        (gi$gap < 0))
    if (any(expr)) {
        gi[expr]$junction_type <- "self_ovl"
    }

    cts <- table(mcols(gi)$junction_type)

    gi$idx <- NULL
    gi$gap <- NULL
    gi$same_strand <- NULL
    gi$intra <- NULL

    nshort <- ifelse(!is.na(cts["2arm_short"]), cts["2arm_short"], 0)
    nantisense <- ifelse(!is.na(cts["antisense_ovl"]), cts["antisense_ovl"], 0)
    nself_ovl <- ifelse(!is.na(cts["self_ovl"]), cts["self_ovl"], 0)

    message("Duplexes categorized by chimeric junction span: ", length(gi))
    message(
        "Duplexes with normal junctions: ", cts["2arm"], ": ",
        round(cts["2arm"] / length(gi) * 100, 2), " %"
    )
    message(
        "Duplexes with self-overlap: ", nself_ovl, ": ",
        round(nself_ovl / length(gi) * 100, 2), " %"
    )
    message(
        "Duplexes with antisense self-overlap: ", nantisense,
        ": ", round(nantisense / length(gi) * 100, 2), " %"
    )
    message(
        "Duplexes with the junction shorter than ", normal_gap_threshold,
        "nt): ", nshort, ": ",
        round(nshort / length(gi) * 100, 2), " %"
    )

    return(gi)
}

#' Identify chimeric junctions coinciding with the splice junctions
#'
#' @description
#' Marks interactions which starts/ends within specified shift from the
#' known splice junctions.
#' @param gi \pkg{GInteractions} object
#' @param sj_gr \pkg{Granges} object with the splice junctions data
#' @param sj_tolerance maximum shift between either donor and acceptor splice
#' sites and corresponding chimreic junction coordinates to count chimeric
#' junction as splice junction
#' @param sj_tolerance_strict maximum shift between either donor and acceptor splice
#' sites irrespective of the particular splice junction. If both chimeric junction start
#' and end correspond to donor or acceptor of any known junction, it is marked
#' as splice junction. Used to catch novel combinations of known 3' and 5' sites
#' @return gi object with added 'splicejnc' and  field
#' Additionally 'splicejnc_donor' 'splicejnc_acceptor' fields are added
#' @export
#' @examples
#' data("RNADuplexesSampleData")
#' gi <- getSpliceJunctionChimeras(RNADuplexSampleGI, SampleSpliceJncGR)
#' table(gi$splicejnc)
#' table(gi$splicejnc_acceptor, gi$splicejnc_donor)
getSpliceJunctionChimeras <- function(gi, sj_gr,
    sj_tolerance = 20,
    sj_tolerance_strict = 10) {
    message("\n--- searching for the exon-exon junctions  ---")
    gi$idx <- seq_len(length(gi))
    gi$gap <- pairdist(gi, type = "gap")
    gi$intra <- pairdist(gi, type = "intra")
    gi$same_strand <- as.integer(gi@regions[gi@anchor1]@strand == gi@regions[gi@anchor2]@strand)
    slct <- tibble(x1 = gi$intra == TRUE, x2 = gi$same_strand == 1, x3 = gi$gap >= 1) %>%
        replace(is.na(.), 0) %>%
        mutate(slct = x1 & x2 & x3) %>%
        pull(slct)

    intragi <- gi[slct]
    intergi <- gi[!slct]

    gr_chim <- get_chimeric_junctions_onestrand(intragi)
    gr_chim$idx <- intragi$idx


    SjHits <- findOverlaps(gr_chim, sj_gr, type = "equal", maxgap = sj_tolerance)
    duplexes_coincide_sj <- unique(gr_chim[queryHits(SjHits)]$idx)

    gi$splicejnc <- left_join(tibble("idx" = gi$idx),
        tibble("idx" = duplexes_coincide_sj, "splicejnc" = 1),
        by = "idx"
    ) %>%
        replace(is.na(.), 0) %>%
        pull("splicejnc")

    # strict mode which uses only single coordinate
    tol <- sj_tolerance_strict
    sj_gr_left <- GRanges(IRanges(start = start(sj_gr), end = start(sj_gr) + 1),
        seqnames = seqnames(sj_gr), strand = strand(sj_gr)
    ) +
        sj_tolerance_strict
    sj_gr_right <- GRanges(IRanges(start = end(sj_gr), end = end(sj_gr) + 1),
        seqnames = seqnames(sj_gr), strand = strand(sj_gr)
    ) +
        sj_tolerance_strict

    gr_chim_start <- GRanges(IRanges(start = start(gr_chim), end = start(gr_chim) + 1),
        seqnames = seqnames(gr_chim), strand = strand(gr_chim)
    )
    gr_chim_end <- GRanges(IRanges(start = end(gr_chim), end = end(gr_chim) + 1),
        seqnames = seqnames(gr_chim), strand = strand(gr_chim)
    )
    mcols(gr_chim_start) <- mcols(gr_chim)
    mcols(gr_chim_end) <- mcols(gr_chim)

    gi$splicejnc_donor <- .getStartEndOvl(gi, gr_chim_start, sj_gr_left)
    gi$splicejnc_acceptor <- .getStartEndOvl(gi, gr_chim_end, sj_gr_right)
    gi$both <- (as.integer(gi$splicejnc_donor & gi$splicejnc_acceptor))
    gi$splicejnc <- (as.integer(gi$both | gi$splicejnc))
    gi$both <- NULL

    nsj <- sum(gi$splicejnc)

    gi$idx <- NULL
    gi$intra <- NULL
    gi$gap <- NULL
    gi$same_strand <- NULL


    message("Chimeras to test against splice junctions: ", length(gi))
    message(
        "SJ entries  : ", nsj, ": ", round(nsj / length(gi) * 100, 3),
        " % of all"
    )
    message(
        "SJ entries / single chr entries: ", nsj, "/",
        length(intragi), " : ", round(nsj / length(intragi) * 100, 2),
        " % of single chromosome chimeras"
    )

    return(gi)
}
#' Helper func for calculating one-side overlap of SJ and junctions of gi
#'
#' @param gi GInteractions object
#' @param gr_chim_c GRanges object with chimeric junctions of gi
#' @param gr_sj_c GRanges with chimeric junctions
#' @param tol overlap tolerance
#' @keywords internal
#' @return Granges object with the left (A) region
.getStartEndOvl <- function(gi, gr_chim_c, gr_sj_c, tol = 3) {
    SjHits <- findOverlaps(gr_chim_c, gr_sj_c, type = "any", minoverlap = 1)
    duplexes_coincide_sj <- unique(gr_chim_c[queryHits(SjHits)]$idx)

    splicejnc_vec <- left_join(tibble("idx" = gi$idx),
        tibble("idx" = duplexes_coincide_sj, "splicejnc" = 1),
        by = "idx"
    ) %>%
        replace(is.na(.), 0) %>%
        pull("splicejnc")
    return(splicejnc_vec)
}

#' Get colnames for expected data types
#'
#' @param nameset name of the table
#' @keywords internal
#' @return character vector
get_colnames_and_types_for_input <- function(nameset) {
    switch(nameset,
        "STAR_COLNAMES19" = c(
            "chr_donorA", "brkpt_donorA", "strand_donorA", "chr_acceptorB",
            "brkpt_acceptorB", "strand_acceptorB", "junction_type", "repeat_left_lenA",
            "repeat_right_lenB", "read_name", "start_alnA", "cigar_alnA",
            "start_alnB", "cigar_alnB", "num_chim_aln", "max_poss_aln_score",
            "non_chim_aln_score", "this_chim_aln_score", "bestall_chim_aln_score"
        ),
        "STAR_COLNAMES14" = c(
            "chr_donorA", "brkpt_donorA", "strand_donorA", "chr_acceptorB",
            "brkpt_acceptorB", "strand_acceptorB", "junction_type", "repeat_left_lenA",
            "repeat_right_lenB", "read_name", "start_alnA", "cigar_alnA",
            "start_alnB", "cigar_alnB"
        ),
        "STAR_COLTYPES19" = c(
            "character", "numeric", "character", "character", "numeric",
            "character", "numeric", "numeric", "numeric", "character",
            "numeric", "character", "numeric", "character", "numeric",
            "numeric", "numeric", "numeric", "numeric"
        ),
        "STAR_COLTYPES14" = c(
            "character", "numeric", "character", "character", "numeric",
            "character", "numeric", "numeric", "numeric", "character",
            "numeric", "character", "numeric", "character"
        ),
        "BEDPE_COLNAMES" = c(
            "chromA", "startA", "endA", "chromB",
            "startB", "endB", "readname", "flag", "strandA", "strandB"
        ),
        "BEDPE_COLTYPES" = c(
            "character", "numeric", "numeric", "character",
            "numeric", "numeric", "character", "numeric", "character", "character"
        ),
        stop("Invalid variable name")
    )
}


#' Count the length of the key type in CIGAR string
#' @description
#'
#' Takes CIGAR operands i.e M,N,S and sums the associated blocks length
#' It is vectorized. i.e supports vector with CIGAR strings
#' @param strings CIGAR string vector
#' @param s CIGAR operands
#'
#' @return vector with length values
#' @export
#'
#' @examples
#' # From a vector
#' get_char_count_cigar(c("4S18M22S", "25S26M"), "S")
#' get_char_count_cigar(c("18M22S", "20M20S"), "M")
get_char_count_cigar <- function(strings, s) {
    reg_tmp <- "\\d+(?=V)"
    reg <- str_replace(reg_tmp, "V", s)
    charcount <- rowSums(as_tibble(data.frame(
        str_extract_all(strings, reg,
            simplify = TRUE
        ),
        fix.empty.names = TRUE
    )) %>%
        mutate_all((as.numeric)) %>%
        replace(is.na(.), 0) %>% as.matrix())

    return(charcount)
}



#' Convert Dataframe to GInteractions
#'
#' @description
#' Converts dataframe-like object to the `GInteractions`.
#' @details
#' arms will be consistent between different objects of same reference
#' Following columns are looked up in input dataframe  to parse region coordinates:
#' c("chromA','startA','endA','strandA',"chromB",'startB','endB','strandB')
#' `GInteractions(mode='strict')` is enforced, to ensure that the order of the regions
#' Extra columns are stored as metadata fields
#' @param df dataframe-like object. Should be convertable to tibble::tibble()
#' @return GInteractions(mode='strict')
#' @seealso [DuplexDiscovereR::makeDfFromGi()]
#' @export
#' @examples
#' # load example GInteractions
#' data(RNADuplexesSmallGI)
#'
#' converted_to_df <- makeDfFromGi(SampleSmallGI)
#' converted_to_gi <- makeGiFromDf(converted_to_df)
makeGiFromDf <- function(df) {
    dtt <- tibble(df)
    # requires "chromA","startA","endA","strandA",
    # "chromB","startB","endB","strandB" columns
    df_a <- dtt %>%
        dplyr::select(chromA, startA, endA, strandA) %>%
        purrr::set_names(c("chrom", "start", "end", "strand"))

    df_b <- dtt %>%
        dplyr::select(chromB, startB, endB, strandB) %>%
        purrr::set_names(c("chrom", "start", "end", "strand"))
    meta <- dtt %>%
        dplyr::select(!dplyr::one_of(c(
            "chromA", "startA", "endA", "strandA",
            "chromB", "startB", "endB", "strandB"
        )))
    a_gr <- GRanges(df_a)
    b_gr <- GRanges(df_b)
    gi <- GInteractions(a_gr, b_gr, mode = "strict")
    mcols(gi) <- data.frame(meta)
    return(gi)
}

#' Convert GInteractions to tibble
#'
#' @description
#' Converts `GInteractions` to tibble, preserves metadata
#' @details
#' Following naming conventions is used  for region coordinates:
#' `c('chromA','startA','endA','strandA',
#' 'chromB','startB','endB','strandB')`
#' @param gi GInteracttions
#' @return tibble preserving metadata columns
#' @seealso [DuplexDiscovereR::makeGiFromDf()]
#' @export
#' @examples
#' data(RNADuplexesSmallGI)
#' converted_to_df <- makeDfFromGi(SampleSmallGI)
#' converted_to_gi <- makeGiFromDf(converted_to_df)
makeDfFromGi <- function(gi) {
    meta <- as_tibble(mcols(gi))
    mcols(gi) <- NULL
    gidf <- as_tibble(data.frame(gi)) %>%
        dplyr::rename(
            chromA = seqnames1,
            startA = start1,
            endA = end1,
            strandA = strand1,
            chromB = seqnames2,
            startB = start2,
            endB = end2,
            strandB = strand2,
        ) %>%
        dplyr::select(-c(width1, width2))
    gidf <- bind_cols(gidf, meta)
    return(gidf)
}

#' Get chimeric junctions
#'
#' Returns chimeric junction defined as range distance between the end and the
#' start of the first and second range respectively
#'
#' @details
#' If the pair of interacting ranges is not on the same strand and chromosome,
#' returns error
#' @keywords internal
#' @returns Granges object with the
get_chimeric_junctions_onestrand <- function(gi_intra) {
    gi_intra$intra <- pairdist(gi_intra, type = "intra")
    if (!all(gi_intra$intra == TRUE)) {
        stop("Cannot extract chimeric junction. Not all ranges are on the same chromosome/strand")
    }
    gr_chim <- GRanges(data.frame(
        "chrom" = seqnames(gi_intra@regions[gi_intra@anchor2]),
        "start" = end(gi_intra@regions[gi_intra@anchor1]),
        "end" = start(gi_intra@regions[gi_intra@anchor2]),
        "strand" = strand(gi_intra@regions[gi_intra@anchor2])
    ))
    return(gr_chim)
}


#' Refresh the `GInteractions` object
#'
#' Sub-setting the `GInteractions` object does not reduce its `ranges` container
#' For some applications, to save memory, we can safely reduce the size of the
#' object by re-creating it.
#' Also, it can be used to ensure the 'strict' mode of the regions in `ranges`
#'
#' @param gi `GInteractions` object
#' @keywords internal
#' @return `GInteractions` object with new ranges attribute
refresh_gi <- function(gi) {
    a1 <- gi@anchor1
    a2 <- gi@anchor2
    gi_collapsed <- GInteractions(gi@regions[a1], gi@regions[a2], mode = "strict")
    mcols(gi_collapsed) <- mcols(gi)
    return(gi_collapsed)
}
#' Subset the `GInteractions` object to single interaction
#'
#' Sub-setting the `GInteractions` object does not reduce its `ranges` container
#' This function selects `Ginteraction` by index and reduces the `ranges`
#' @param gi `GInteractions` object
#' @keywords internal
#' @return `GInteractions` with the range atribure reduced to single interaction

subset_gi <- function(gi, k) {
    gi <- gi_trans$SPLASH_RA_1
    a1 <- gi[k]@anchor1
    a2 <- gi[k]@anchor2
    gi_short <- GInteractions(1, 2, gi@regions[c(a1, a2)], mode = "strict")
    mcols(gi_short) <- mcols(gi[k])
    return(gi_short)
}

#' Convert `GInteractions` object to `Granges`
#'
#' Creates the 'long' `GRanges` by stacking the A and B arms one 'on top' of the other.
#' Adds `id` and  `group` fields as indicators of original index and interaction
#' arm (A- left arm, B- right arm)
#' @param gi `GInteractions`
#' @export
#' @return `GRanges` twice the length of the input
#' @examples
#' data("RNADuplexesSmallGI")
#' convert_gi_to_ranges(SampleSmallGI)
convert_gi_to_ranges <- function(gi) {
    boxes <- c(anchors(gi)$first, anchors(gi)$second)
    groupp <- rep(seq_len(length(gi)), times = 2)
    ids <- str_c(groupp, ".", rep(c("A", "B"), each = length(gi)))
    mcols(boxes) <- rep(mcols(gi), 2)
    boxes$group <- groupp
    boxes$id <- ids
    return(boxes)
}

#' Get left arm of GInteraction
#'
#' @param gi GInteractions object
#' @keywords internal
#' @return Granges object with the left (A) region
get_arm_a <- function(gi) {
    a1 <- gi@anchor1
    a2 <- gi@anchor2
    gi_collapsed <- GInteractions(gi@regions[a1], gi@regions[a2], mode = "strict")
    return(anchors(gi_collapsed)$first)
}

#' Get right arm of GInteraction

#' @param gi GInteractions object
#' @keywords internal
#' @return Granges object with the right (B) region
get_arm_b <- function(gi) {
    a1 <- gi@anchor1
    a2 <- gi@anchor2
    gi_collapsed <- GInteractions(gi@regions[a1], gi@regions[a2], mode = "strict")
    return(anchors(gi_collapsed)$second)
}


assign_name_to_gi <- function(gi, sample_name) {
    df <- mcols(gi) %>%
        as_tibble() %>%
        mutate("sample_name" = sample_name)
    mcols(gi) <- df
    return(gi)
}

unify_gi_levels <- function(gi1, gi2) {
    gi1 <- refresh_gi(gi1)
    gi2 <- refresh_gi(gi2)
    gi1$tmpid <- 1
    gi2$tmpid <- 2
    gi3 <- c(gi1, gi2)
    gi3 <- refresh_gi(gi3)
    gi1 <- gi3[gi3$tmpid == 1]
    gi2 <- gi3[gi3$tmpid == 2]
    return(list("gi1" = gi1, "gi2" = gi2))
}

test_that("Load test data object run small clustering works", {
    suppressMessages({
        data("RNADuplexesSmallGI")
        SampleSmallGI$n_reads <- 1
        gi <- clusterDuplexGroups(SampleSmallGI, maxgap = 40, minoverlap = 10, min_arm_ratio = 0.3)
    })
    expect_equal(length(collapse_duplex_groups(gi)), 3)
    expect_equal(length(collapse_duplex_groups(gi, return_unclustered = T)), 6)
})
test_that("Load STAR format, classification and deduplication works", {
    suppressMessages({
        set.seed(123)
        test_reads_star <- read.table(system.file("extdata", "test_SPLASH_Chimeric.out.junction", package = "DuplexDiscovereR"), header = T)
        df_chim <- runDuplexDiscoPreproc(data = test_reads_star, library_type = "SE", table_type = "STAR", keep_metadata = F)
        typetable <- table(df_chim$map_type)
        two_arm_N <- typetable[["2arm"]]
        mm_N <- typetable[["multi_map"]]
        ms_N <- typetable[["multi_split"]]
        # deduplication
        gi <- makeGiFromDf(df_chim)
        gi_collapse_res <- collapseIdenticalReads(gi)
        n_after <- length(gi_collapse_res$gi_collapsed)
        collapse_counts <- table(gi_collapse_res$stats_df$n_reads_collapsed)
        n_after_in_stats <- sum(unname(collapse_counts) / as.integer(names(collapse_counts)))
    })
    expect_equal(two_arm_N, "expected" = 2090, label = "mapping type classification works 1 ")
    expect_equal(mm_N, "expected" = 218, label = "mapping type classification works 2")
    expect_equal(ms_N, "expected" = 285, label = "mapping type classification works 3")
    expect_equal(n_after, "expected" = 4908, label = "deduplication works")
    expect_equal(n_after_in_stats, "expected" = 4908, label = "deduplication stats works")
})

test_that("Load bedpe format and crude clustering works", {
    suppressMessages({
        df_bedpe <- read.table(system.file("extdata", "test_SPLASH_DuplexesRaw.bedpe", package = "DuplexDiscovereR"))
        colnames(df_bedpe) <- get_colnames_and_types_for_input("BEDPE_COLNAMES")
        set.seed(123)
        df_chim <- runDuplexDiscoPreproc(data = df_bedpe, library_type = "SE", table_type = "bedpe", keep_metadata = F)
        df_chim <- df_chim %>% dplyr::filter(map_type == "2arm")
        gi_clusters <- collapse_duplex_groups(clusterDuplexGroups(makeGiFromDf(df_chim)))
        n_reads_clustered <- sum(gi_clusters$n_reads)

        gi_clusters_decomp <- collapse_duplex_groups(clusterDuplexGroups(makeGiFromDf(df_chim), decompose = TRUE))
        n_reads_clustered_decomp <- sum(gi_clusters_decomp$n_reads)
    })
    expect_equal(n_reads_clustered, 1162, label = "clustering bedpe produced expected results")
    expect_equal(n_reads_clustered_decomp, 1162, label = "clustering+decomposition produced expected results")
})
test_that("Load GInteractions format and crude clustering works", {
    suppressMessages({
        data("RNADuplexesSampleData")
        set.seed(123)
        df_chim <- runDuplexDiscoPreproc(data = RNADuplexSampleGI, keep_metadata = F)
        df_chim <- df_chim %>% dplyr::filter(map_type == "2arm")
        gi_clusters <- collapse_duplex_groups(clusterDuplexGroups(makeGiFromDf(df_chim)))
        n_reads_clustered <- sum(gi_clusters$n_reads)
    })
    expect_equal(n_reads_clustered, 1162, label = "clustering bedpe produced expected results")
})

#' Executes all steps of DuplexDiscovereR pipeline
#' @description
#' Generates GInteractions object with duplex groups from the STAR
#' Chimeric.out.junction or bedpe file.
#' Classifies reads, annotates reads by overlap with the gene or transcript
#' features, calculates p-values and hybridization energies.
#' Additionally, returns mappings from duplex groupd back to genes.
#' @details
#' This is a main function to do the initial discovery of the RNA duplexes after
#' the chimeric read mapping. It wraps following procedures:
#' - Classifies the input reads by the mapping type.
#' Keeps 2-arm chimeric reads for downstream analysis
#' - Compares 2arm duplex reads against provided splice junctions
#' - Classifies 2arm duplexes into spurious self-overlapping, splice junction categoris
#' - Performs clustering of the remaining reads into duplex groups
#'    - Collapses identically mapped reads
#'    - Collapses closely located reads, almost identical reads
#'    - Finds duplex groups throughout whole  data set
#' - Annotates duplex groups with genomic features if annotation is provided
#' - Calculates p-values if gene counts and annotation are provided
#' - Calculates hybridization energies if path to the .fasta file is provided
#' @param data dataframe-like object with the split reads. Output of Chimeric.out.junction or dataframe with fileds defined by bedpe format:
#' c("chromA","startA",'endA',"chromB",'startB','endB','readname','flag','strandA','strandB', ... )
#' Alternatively, `GInteractions` object
#' @param table_type one in c("STAR","bedpe") Defines the type of the input dataframe.
#' ignored if input data is `GInteractions`
#' @param junctions_gr \pkg{GRanges} object with the splice junction coordinates
#' @param anno_gr \pkg{GRanges} object to use for the annotation of the interactions. Optional
#' @param anno_gr_keys c() vector with names of metadata fields in anno_gr which will be used for the annotation. Argument passed to
#' \code{annotateGI()} function. The c('gene_id','gene_name','gene_type') columns in anno_gr are used by default.
#' @param fafile path to the genome .fasta file. Used to calculate hybridization energy with *RNADuplex*. Sequence names should correspond to the sequences from which the mapping index was created. Optional
#' @param df_counts A two- column dataframe with counts. Counts are used for p-value calculation. The first column should match the 'gene_id' feature in anno_gr. The second column is the respective count. Optional
#' @param sample_name A name of the sample, used for assembling the analysis statistics dataframe
#' @param lib_type one in c('SE','PE'). Type of the seqeuncing library. Default is 'SE'
#' @param max_sj_shift Maximum shift between either donor and acceptor splice sites and chimeric junction coordinates to count chimeric junction as splice junction
#' @param min_junction_len a minimum allowed distance between chimeric arms for the read input.
#' Reads with the junction closer than \code{min_junction_len} are annotated as '2arm_shot' and not clustered to duplex groups
#' @param max_gap Parameter for read clustering. Minimum required shift between start and end coordinates of arms for pair of overlapping chimeric reads.
#' If the shift is longer than \code{max_gap} for either arm, then total read overlap between those reads is zero.
#' @param min_overlap Parameter for read clustering. Minimum required overlap to for either arm (A or B) for pair of chimeric reads.
#' @param min_arm_ratio Parameter for read clustering.
#' If the overlap-to-span ratio for either arm (A or B) for pair of chimeric reads is less than \code{min_arm_ratio}, then the total overlap for this pair is set to zero.
#' @param collapse_n_inter  Parameter for read clustering  (iterative step). Number of iterations to repeat step of collapsing of the highly similar chimeric reads.
#' Increasing this from i.e 0 to 5 reduces clustering time and memory for the libraries with many overlapping reads.
#' @param gap_collapse_similar  Parameter for read clustering (iterative step). Analogous to the max_gap, but applied \code{collapse_n_inter} times during the iterative merging step.
#' Reduce this to 1 or 2 to lower RAM usage for clustering the library with many similar reads.
#' @param trim_alignments TRUE or FALSE. Whether to trim arms alignments to
#' 'trim_length' nucleotide around chimeric junction
#' @param trim_length target size of trimmed alignment
#' @param min_arm_len minimum allowed length of the alignment arm.
#' Read will be dropped if either arm is shorter
#' @param compute_p_values TRUE or FALSE. whether to calcualte random ligation test with binominal ligation model
#' @param compute_odds_test_p_values  TRUE or FALSE. Whether to calcualte odds-ratio random ligation test.
#' @return a list with the  following keys
#' \describe{
#'   \item{`duplex_groups`}{ `GInteractions` object with chimeric reads clustered duplex groups }
#'   \item{`chimeric_reads`}{ `GInteractions` object with non-collapsed chimeric reads }
#'   \item{`reads_classes`}{ `tbl_df` dataframe parallel to the the input dataframe, annotated with read categories and duplex groups }
#'   \item{`chimeric_reads_stats`}{ `tbl_df` dataframe containing read type classification statistics }
#'   \item{`run_stats`}{ `tbl_df` dataframe with the time and memory info about the run }
#' }
#' @export
#' @seealso [DuplexDiscovereR::DuplexDiscovererResults()]
#'
#' @examples
#'
#' library(DuplexDiscovereR)
#' # load data
#' data("RNADuplexesSampleData")
#' result <- runDuplexDiscoverer(
#'     data = RNADuplexesRawChimSTAR,
#'     junctions_gr = SampleSpliceJncGR,
#'     anno_gr = SampleGeneAnnoGR,
#'     df_counts = RNADuplexesGeneCounts,
#'     sample_name = "test clustering",
#'     fafile = NULL,
#'     collapse_n_inter = 3,
#'     lib_type = "SE",
#'     table_type = "STAR"
#' )
#' # see results object
#' print(result)
#' # duplex groups
#' dd_get_duplex_groups(result)
#' # individual chimeric reads
#' dd_get_chimeric_reads(result)
#' # counts of detected read tyoes
#' dd_get_chimeric_reads_stats(result)
runDuplexDiscoverer <- function(data,
    table_type = "",
    junctions_gr = NULL,
    anno_gr = NULL,
    anno_gr_keys = c("gene_id", "gene_name", "gene_type"),
    fafile = NULL,
    df_counts = NULL,
    sample_name = "sample",
    lib_type = "SE",
    min_junction_len = 5,
    max_gap = 50,
    min_arm_ratio = 0.1,
    min_overlap = 10,
    max_sj_shift = 10,
    gap_collapse_similar = 2,
    collapse_n_inter = 5,
    trim_alignments = FALSE,
    trim_length = 40,
    min_arm_len = 9,
    compute_p_values = TRUE,
    compute_odds_test_p_values = FALSE) {
    memstart <- sum(data.frame(gc(reset = TRUE))[, 6])
    start_time <- Sys.time()
    # STEP 1 pre-process------
    time1 <- Sys.time()
    df <- runDuplexDiscoPreproc(data,
        table_type = table_type,
        library_type = lib_type,
        keep_metadata = TRUE,
        min_arm_len = min_arm_len
    )

    n_reads_initial <- sum(df$n_reads)
    message("Number of input alignments: ", n_reads_initial)
    time2 <- Sys.time()
    time_preproc <- round(as.numeric(difftime(time2, time1,
        units = "secs"
    )), 3)


    # STEP 2 select only 2-arm type chimeras, find SJ, find "too short" chimeras
    time1 <- Sys.time()
    # 2.1 filter out multi spli and multimap aln
    single_gap_df <- df %>%
        dplyr::filter(map_type == "2arm")
    # 2.1.1 create stats
    read_stats_df <- df %>%
        dplyr::select(readname, n_reads, map_type) %>%
        mutate(read_id = c(seq_len(nrow(df)))) %>%
        relocate(read_id, .after = n_reads)

    # 2.1.2 trim alignments
    if (trim_alignments) {
        single_gap_df <- trimAroundJunction(single_gap_df,
            extract_len = trim_length
        )
    }

    # 2.2 convert to GInteractions and mark short/overlapping/splice junction reads
    big_gi <- makeGiFromDf(single_gap_df)
    if (!is.null(junctions_gr)) {
        big_gi <- classifyTwoArmChimeras(
            gi = big_gi,
            min_junction_len = min_junction_len,
            junctions_gr = junctions_gr,
            max_sj_shift = max_sj_shift
        )
    } else {
        message(" Splice junction refrence is not provided")
        big_gi <- getChimericJunctionTypes(big_gi, normal_gap_threshold = min_junction_len)
        big_gi$splicejnc <- 0
    }

    # 2.2a save stats
    read_stats_df <- left_join(read_stats_df,
        as_tibble(mcols(big_gi)[c(
            "read_id", "junction_type",
            "splicejnc"
        )]),
        by = "read_id"
    ) %>%
        mutate(
            read_type = ifelse(is.na(junction_type),
                map_type, as.character(junction_type)
            ),
            read_type = ifelse(splicejnc == 0 | is.na(junction_type),
                read_type, "2arm_sj"
            )
        )
    time2 <- Sys.time()
    time_classify <- round(as.numeric(difftime(time2, time1,
        units = "secs"
    )), 3)

    # 2.3 keep only chimeric reads with 2 arms and not too short and not splice jnc
    big_gi$keep <- (big_gi$junction_type == "2arm") & (big_gi$splicejnc == 0)
    gi_2arm <- big_gi[big_gi$keep == TRUE]

    # STEP 3 clustering---------
    message("--- clustering ---")
    time1 <- Sys.time()

    # 3.1 prepare clustering: reduce complexity -----------
    message("--- collapsing identical reads ---")
    res_collapse_ident <- collapseIdenticalReads(gi_2arm)
    # DEBUG
    # 1  6724    330184 chr1   chr7   1820921
    # 2  6724    330184 chr2   chrX   4888364
    # 3  6724    330184 chr2   chrX   6233530
    # browser()

    # 3.1a get results of collapse: get new gi object, update read stats
    gi <- res_collapse_ident$gi_collapsed
    read_stats_df <- left_join(read_stats_df,
        res_collapse_ident$stats_df,
        by = "read_id"
    ) %>%
        mutate(collapse_identical = 1)

    # 3.2 collapse (cluster) similar (less than 1..5 nt shift) reads
    # because thye will nodes be higly connected nodes"
    if (collapse_n_inter != 0) {
        message("--- iteratively collapse similar reads  --- ")
        message("minimum shift is  :", gap_collapse_similar, " nt")
        res <- collapseSimilarChimeras(gi, read_stats_df,
            maxgap = gap_collapse_similar,
            niter = collapse_n_inter,
            minoverlap = 20
        )
        # get results
        gi <- res$gi_updated
        read_stats_df <- res$stats_df
    }
    # 3.3 Clustering on the whole-genome -----
    message("--- calculating total read overlaps ---")

    graphdf_fast <- computeGISelfOverlaps(gi,
        maxgap = max_gap,
        id_column = "duplex_id",
        minoverlap = min_overlap
    )

    if ((nrow(graphdf_fast) == 0)) {
        message("Consider re-running pipeline with disabled extra collapse by
            setting collapse_n_inter = 0 ")
        stop("Global clustering cannot be called not run because there no
    further DG merge is possible")
    }

    # scale weights from (min:max): (0,2) to (0,1)
    graphdf_fast$weight <- scales::rescale(graphdf_fast$weight, to = c(0, 1))
    # prune weak edges
    graphdf_fast <- graphdf_fast %>% dplyr::filter(
        ratio.A >= min_arm_ratio,
        ratio.B >= min_arm_ratio
    )
    message("--- finding duplex groups  ---")

    # clustering will add one column "dg_id"
    gi_fast <- clusterDuplexGroups(gi, graphdf = graphdf_fast, decompose = FALSE)

    # add dg_id for duplexes which are aggregated locally, but not clustered globally
    gi_fast <- .addDGidsForTmpDGs(gi_fast)

    # use it to collapse duplexes into suplex groups
    gi_final <- collapse_duplex_groups(gi_fast,
        return_unclustered = FALSE,
        return_collapsed = TRUE,
        keep_meta = FALSE
    )

    # DEBUG
    read_stats_df_old <- read_stats_df


    # update read stats
    read_stats_df <- left_join(read_stats_df, .DGIdToDuplexId(gi_fast),
        by = "duplex_id"
    )



    dt_2arm <- left_join(tibble("read_id" = gi_2arm$read_id), read_stats_df,
        by = "read_id"
    ) %>%
        group_by(dg_id) %>%
        mutate("n_reads_dg" = n()) %>%
        ungroup() %>%
        mutate(n_reads_dg = ifelse(!is.na(dg_id), n_reads_dg, 0)) %>%
        dplyr::select(dg_id, duplex_id, n_reads_dg)

    gi_2arm$dg_id <- dt_2arm$dg_id
    gi_2arm$duplex_id <- dt_2arm$duplex_id
    gi_2arm$n_reads_dg <- dt_2arm$n_reads_dg
    gi_2arm$was_clustered <- 1

    # add the reads, which were dropped before clustering
    gi_2arm_full <- c(gi_2arm, big_gi[big_gi$keep == FALSE])
    gi_2arm_full$was_clustered <- ifelse(!is.na(gi_2arm_full$was_clustered), 1, 0)

    time2 <- Sys.time()
    time_clust <- round(as.numeric(difftime(time2, time1,
        units = "secs"
    )), 3)

    # STEP 4 Annotation...-------
    time1 <- Sys.time()
    if (!is.null(anno_gr)) {
        message("--- annotation --- ")
        gi_final <- annotateGI(gi_final, anno_gr, keys = anno_gr_keys)
        gi_final <- .annotateCisTrans(gi_final)
        not_annotated <- sum(as.integer(is.na(gi_final$gene_id.A) | is.na(gi_final$gene_id.B)))
        not_annotated_full <- sum(as.integer(is.na(gi_final$gene_id.A) & is.na(gi_final$gene_id.B)))
        annotated <- length(gi_final) - not_annotated
        message("N annotated duplex groups: ", annotated)
        message("N duplex groups with at least one arm missing annotaton: ", not_annotated)
        message("N duplex groups with at both arms missing annotaton: ", not_annotated_full)
        if (((compute_odds_test_p_values) || (compute_p_values)) && !is.null(df_counts)) {
            message("--- computing random ligation p-values ---")
            gi_final <- calculateLigationPvalues(gi_final, df_counts,
                compute_binom = compute_p_values,
                compute_odds = compute_odds_test_p_values
            )
        } else {
            if (!is.null(df_counts)) {
                message("--- adding counts from df_counts without p-value calculation  ---")
                gi_final <- .addGeneCounts(gi_final, df_counts)
            }
        }
    } else {
        message("No annotation provided")
    }
    time2 <- Sys.time()
    time_anno <- round(as.numeric(difftime(time2, time1,
        units = "secs"
    )), 3)

    sumreads <- as_tibble(data.frame(mcols(gi_fast))) %>%
        dplyr::filter(!is.na(dg_id)) %>%
        group_by(dg_id) %>%
        summarise(nreads = sum(n_reads))

    widthA <- width(get_arm_a(gi_final))
    widthB <- width(get_arm_b(gi_final))
    meanA <- mean(widthA)
    medA <- median(widthA)
    meanB <- mean(widthB)
    medB <- median(widthB)
    n_dgs <- length(gi_final)

    typetable <- table(read_stats_df$read_type)
    count_stats <- (t(data.frame(unname(typetable))[, 2]))
    colnames(count_stats) <- names(typetable)

    end_time <- Sys.time()
    time_diff <- round(as.numeric(difftime(end_time, start_time,
        units = "secs"
    )), 3)
    memend <- sum(data.frame(gc())[, 6])
    memdiff <- round((memend - memstart) / 1024, 3)
    maxmem <- round((memend) / 1024, 3)
    mem_initial <- round((memstart) / 1024, 3)

    # STEP 5 Calling hybrids if fafile is provided----
    time1 <- Sys.time()
    if (!is.null(fafile)) {
        message("--- calculating hybridization ---")
        gi_final <- getRNAHybrids(gi_final, fafile)
    }
    time2 <- Sys.time()
    time_hyb <- round(as.numeric(difftime(time2, time1,
        units = "secs"
    )), 3)
    # Finally, aggregate stats on exec memory and time
    summary_stats_run <- tibble(
        "sample_name" = sample_name,
        "exec_time" = time_diff,
        "memstart" = memstart,
        "memend" = memend,
        "memdiff" = round((memend - memstart) / 1024, 3),
        "maxmem" = round((memend) / 1024, 3),
        "mem_initial" = round((memstart) / 1024, 3),
        "time_preproc" = time_preproc,
        "time_classify" = time_classify,
        "time_clust" = time_clust,
        "time_hybrids" = time_hyb,
        "time_anno" = time_anno
    )
    # Aggregate stats on duplexes and reads
    summary_stats_reads <- tibble(
        "sample_name" = sample_name,
        "n_reads_input" = n_reads_initial,
        "n_dgs" = n_dgs
    )

    summary_stats_lens <- tibble(
        "mean_len_A" = meanA,
        "median_len_A" = medA,
        "mean_len_B" = meanB,
        "median_len_B" = medB
    )

    summary_stats_reads <- bind_cols(
        summary_stats_reads,
        as_tibble(count_stats),
        summary_stats_lens
    )

    gc(reset = TRUE)

    results <- DuplexDiscovererResults(
        duplex_groups = gi_final,
        chimeric_reads = gi_2arm_full,
        reads_classes = read_stats_df,
        chimeric_reads_stats = summary_stats_reads,
        run_stats = summary_stats_run
    )

    message("finished")
    return(results)
}

test_that("Executing all steps in single-call works", {
    suppressMessages({
        data(RNADuplexesSampleData)
        set.seed(123)
        res <- runDuplexDiscoverer(
            data = RNADuplexesRawChimSTAR,
            junctions_gr = SampleSpliceJncGR,
            anno_gr = SampleGeneAnnoGR,
            df_counts = RNADuplexesGeneCounts,
            sample_name = "test clustering",
            fafile = NULL,
            collapse_n_inter = 3,
            lib_type = "SE",
            table_type = "STAR"
        )

        gi_clusters <- dd_get_duplex_groups(res)
        gi_reads <- dd_get_chimeric_reads(res)
        df_reads <- dd_get_reads_classes(res)

        n_reads_clustered <- sum(gi_clusters$n_reads)
        n_reads_clustered_stats <- df_reads %>%
            dplyr::filter(!is.na(dg_id)) %>%
            pull(n_reads) %>%
            sum()

        n_reads_unclustered <- df_reads %>%
            dplyr::filter(is.na(dg_id)) %>%
            distinct(readname) %>%
            nrow()
        n_reads_passed_to_clustering <- df_reads %>%
            dplyr::filter(map_type == "2arm") %>%
            nrow()
        n_reads_passed_to_clustering_nodg <- df_reads %>%
            dplyr::filter(is.na(dg_id), map_type == "2arm") %>%
            nrow()
        n_clusters <- length(gi_clusters)
    })
    expect_equal(n_reads_clustered, n_reads_clustered_stats, label = "read numbers match between stats and returned GI")
    expect_equal(n_reads_clustered + n_reads_passed_to_clustering_nodg, n_reads_passed_to_clustering, label = "no reads lost before clustering")
    expect_equal(n_reads_clustered, 244, label = "correct number of reads clustered")
    expect_equal(n_clusters, 70, label = "correct number of clusters found")
})

#' Cluster RNA duplexes in `GInteractions` object
#'
#' @description
#' Main method to find duplex groups from the individual interactions
#'
#' @details
#' Accepts or creates the connections graphdf dataframe, creates graph with
#' `igraph` package, uses community detection algoritm to call clusters.
#' New field `dg_id` is added to label the clusters (duplex groups).
#' If no community is found for the read, `dg_id` is `NA`
#'
#' @param gi `GInteractions` object
#' @param min_arm_ratio For graph creation only. Span-to-overlap ratio threshold. If smaller than this value, then edge is not drawn
#' @param maxgap For graph creation only. Max shift between arms starts and ends for pair of overlapping reads
#' @param minoverlap For graph creation only. Minimum required overlap between either arm for pair of overlapping reads
#' Other optional arguments, which are not relevant, unless user want to modify clustering
#' weights or modify clustering in some other way
#' @param graphdf Optional. Dataframe representing connection edges between entries in gi
#' If not provided, graphdf is created inside the function
#' @param id_column Optional. Column name in the  `GInteractions` metadata,
#'  which was used to index temporary duplex groups, if they are present
#' @param weight_column  Optional. If graphdf is provided,  field to use for weight overlaps
#' @param fast_greedy  Optional. Run the fast_greedy algorithm instead of Louvain.
#' Can speed up calcualtion for the large graphs.
#' @param decompose Decompose graph into separate sub-graphs before clustering.
#  Increases runtime, sometimes results in higher number of overlapping DG.
#' @param id_columns_grapdf Column in the graph dataframe, which was used for index
#' @param dump_graph For debug. Export the graph elements. not used
#' @param dump_path For debug. PArt to export the graph elements. not used
#'
#' @return `GInteractions` object with new \code{dg_id} column
#' @export
#' @examples
#' data("RNADuplexesSampleData")
#' # run preprocessing and filtering
#' preproc_df <- runDuplexDiscoPreproc(RNADuplexesRawBed, table_type = "bedpe")
#' preproc_gi <- makeGiFromDf(preproc_df)
#' preproc_gi <- classifyTwoArmChimeras(preproc_gi,
#'     min_junction_len = 5,
#'     junctions_gr = SampleSpliceJncGR, max_sj_shift = 10
#' )
#' # collapse duplicates
#' gi <- collapseIdenticalReads(preproc_gi)$gi
#' # run global clustering
#' gi <- clusterDuplexGroups(gi)
#' # check dg_ids
#' table(is.na(gi$dg_id))
clusterDuplexGroups <- function(gi, graphdf = NULL, maxgap = 40,
    minoverlap = 10,
    id_column = "duplex_id",
    weight_column = "weight",
    fast_greedy = FALSE,
    decompose = FALSE,
    id_columns_grapdf = paste(id_column, c(1, 2), sep = "."),
    min_arm_ratio = 0.3,
    dump_graph = FALSE,
    dump_path = "") {
    gi$dg_id <- NULL

    if (is.null(graphdf)) {
        message("Computing overlaps on-the-fly")
        gi$idx_tmp <- seq_len(length(gi))
        id_column <- "idx_tmp"
        graphdf <- computeGISelfOverlaps(gi,
            id_column = id_column, maxgap = maxgap,
            minoverlap = minoverlap
        )
        if (nrow(graphdf) == 0) {
            message("No overlap found. No DG found in input object")
            gi$dg_id <- NA
            return(gi)
        }
        id_columns_grapdf <- paste(id_column, c(1, 2), sep = ".")
        weight_column <- "weight"
        graphdf <- graphdf %>% dplyr::filter(
            ratio.A >= min_arm_ratio,
            ratio.B >= min_arm_ratio
        )
    } else {
        if (!(id_column %in% colnames(mcols(gi)))) {
            stop(id_column, "not found in provided GenomicInteractions onject")
        } else {
            if (any(is.na(mcols(gi)[id_column]))) {
                stop(paste0("Undefined values in index field found before duplex clustering. Index column used: ", id_column, collapse = ""))
            }
        }
    }
    graphdf$weight <- graphdf[weight_column] %>% pull()
    graphdf$vert_id1 <- graphdf[id_columns_grapdf[1]] %>% pull()
    graphdf$vert_id2 <- graphdf[id_columns_grapdf[2]] %>% pull()
    graphdf <- graphdf %>% dplyr::select(vert_id1, vert_id2, weight)
    graph_vertices <- unique(c(graphdf$vert_id1, graphdf$vert_id2))

    g <- igraph::graph_from_data_frame(graphdf, directed = FALSE, vertices = graph_vertices)
    igraph::edge.attributes(g)$weight <- round(graphdf$weight, 3)
    g <- igraph::simplify(g, edge.attr.comb = list(weight = "sum"), remove.multiple = TRUE)

    # do clustering by subgraphs
    if (decompose) {
        # here we split into subgraphs and cluster each
        comps <- igraph::decompose(g)
        message("Whole-transciptome graph was split into ", length(comps), " components")

        # call clustering over the list of subgraphs
        comps <- lapply(seq_along(comps), function(i) {
            .compute_clusters_comp(comps[[i]], i)
        })
        df_clustered <-
            tibble(
                dg_id_raw = unlist(lapply(comps, function(x) {
                    igraph::V(x)$cluster_group
                })),
                vert_id = as.integer(unlist(lapply(comps, function(x) {
                    names(igraph::V(x))
                })))
            ) %>%
            mutate(idx = as.integer(vert_id)) %>%
            group_by(dg_id_raw) %>%
            mutate(
                dg_id = cur_group_id(),
                n_el = n()
            ) %>%
            ungroup() %>%
            mutate(cmp = str_split_i(dg_id_raw, "_", 1)) %>%
            group_by(cmp) %>%
            mutate(n_clust = n_distinct(dg_id)) %>%
            ungroup()
        if (dump_graph) {
            grres <- list()
            grres$comps <- comps
            grres$df_clustered <- df_clustered
            save(grres, file = dump_path)
        }
    } else {
        # do clustering genome-wide
        if (!fast_greedy) {
            cluster_lou <- igraph::cluster_louvain(g)
        } else {
            cluster_lou <- igraph::cluster_fast_greedy(g)
        }
        igraph::V(g)$cluster_group <- igraph::membership(cluster_lou)

        # create output df for clustered duplexes
        df_clustered <- tibble(
            dg_id = as.integer(igraph::V(g)$cluster_group),
            vert_id = as.integer(names(igraph::V(g)))
        ) %>%
            arrange(dg_id)
        remove(g)
        remove(graphdf)
        gc()
    }

    if ("idx_tmp" %in% colnames(mcols(gi))) {
        gi$idx <- NULL
    }

    gi$dg_id <- left_join(
        tibble(vert_id = as.integer(mcols(gi)[, id_column])),
        df_clustered[, c("vert_id", "dg_id")],
        by = "vert_id"
    ) %>%
        pull(dg_id)

    return(gi)
}

#' Helper function to call clustering on each component
#'
#' @details
#' Call clustering on each independent graph component.
#' Used when decompose==TRUE
#' @keywords internal
#' @return sub-graph with clusters labelled with
#' sample-wise unique ids (cluster_group)
.compute_clusters_comp <- function(graph, index) {
    wt <- cluster_louvain(graph, weights = igraph::E(graph)$weight)
    igraph::V(graph)$cluster_group <- str_c(index, "_", igraph::membership(wt))
    return(graph)
}

#' DuplexDiscovererResults
#'
#' A helper S4 class to store the results of the full
#' DuplexDiscovereR analysis.
#' This class contains the following output:
#' - `duplex_groups`:  clustered duplex groups.
#' - `chimeric_reads`: individual two-regions chimeric reads.
#'    Contains both clustered and unclustered reads.
#'    Clustered reads are linked to the duplex groups though 'dg_id' field in metadata
#' - `reads_classes`: dataframe parallel to the the input containing classification result and detected mapping type for each entry in the input
#' - `chimeric_reads_stats`: dataframe containing read type classification statistics
#' - `run_stats`: data frame containing statistics about the time and memory used by the pipeline
#' @details
#' Each output type has a corresponding accessor:
#'  - [DuplexDiscovereR::dd_get_duplex_groups()]
#'  - [DuplexDiscovereR::dd_get_chimeric_reads()]
#'  - [DuplexDiscovereR::dd_get_reads_classes()]
#'  - [DuplexDiscovereR::dd_get_chimeric_reads_stats()]
#'  - [DuplexDiscovereR::dd_get_run_stats()]
#' @seealso
#'  [DuplexDiscovereR::dd_get_duplex_groups()] ,
#'  [DuplexDiscovereR::dd_get_chimeric_reads()] ,
#'  [DuplexDiscovereR::dd_get_reads_classes()] ,
#'  [DuplexDiscovereR::dd_get_chimeric_reads_stats()] ,
#'  [DuplexDiscovereR::dd_get_run_stats()]
#'
#' @slot duplex_groups \pkg{GInteractions} object with duplex groups
#' @slot chimeric_reads \pkg{GInteractions} object with chimeric reads
#' @slot reads_classes `tibble` (tbl_df) with read classification data.
#' @slot chimeric_reads_stats `tibble` (tbl_df)  read type statistics.
#' @slot run_stats `tibble` (tbl_df)  runtime and memory info
#' @rdname DuplexDiscovererResults
#' @return A `DuplexDiscovererResults` object.
setClass("DuplexDiscovererResults",
    slots = c(
        duplex_groups = "GInteractions",
        chimeric_reads = "GInteractions",
        reads_classes = "tbl_df",
        chimeric_reads_stats = "tbl_df",
        run_stats = "tbl_df"
    )
)
#' Constructor for DuplexDiscovererResults class
#'
#' @param duplex_groups \pkg{GInteractions} object with duplex groups
#' @param chimeric_reads \pkg{GInteractions} object with chimeric reads
#' @param reads_classes `tibble` (tbl_df) with read classification data.
#' @param chimeric_reads_stats `tibble` (tbl_df)  read type statistics.
#' @param run_stats `tibble` (tbl_df)  runtime and memory info
#' @rdname DuplexDiscovererResults
#' @inherit dd_get_duplex_groups examples
#' @export
DuplexDiscovererResults <- function(duplex_groups,
    chimeric_reads,
    reads_classes,
    chimeric_reads_stats,
    run_stats) {
    new("DuplexDiscovererResults",
        duplex_groups = duplex_groups,
        chimeric_reads = chimeric_reads,
        reads_classes = reads_classes,
        chimeric_reads_stats = chimeric_reads_stats,
        run_stats = run_stats
    )
}

#' Show Method for DuplexDiscovererResults class
#'
#' This method provides a summary of the DuplexDiscovererResults object.
#' It prints `chimeric_reads_stats` followed by the `run_stats`.
#'
#' @param object A `DuplexDiscovererResults` object.
#' @return None. Prints a formatted summary.
#' @export
setMethod("show", "DuplexDiscovererResults", function(object) {
    df <- object@chimeric_reads_stats
    sname <- paste0("Sample name: ", df$sample_name[1])
    df$sample_name <- NULL
    transposed <- as.data.frame(t(df))
    transposed$feature <- rownames(transposed)
    colnames(transposed) <- c("count", "feature")
    transposed$count <- round(transposed$count, 2)
    transposed <- transposed[, c("feature", "count")]

    # Pretty print the transposed tibble
    message("DuplexDiscovereR Results")
    message(sname)
    message("Chimeric reads statistics:")
    print(transposed, row.names = FALSE)
    message("Use class accessors to retrieve output. I.e duplex_groups(resultsObject) ")
})

#' Accessor for `duplex_groups` slot
#'
#' Retrieves the value of the `duplex_groups` slot in a `DuplexDiscovererResults` object.
#' @param object A `DuplexDiscovererResults` object.
#' @return GInteractions object from the `duplex_groups` slot.
#' @docType methods
#' @rdname dd_get_duplex_groups
#' @export
#' @examples
#'
#' # load example input
#' data("RNADuplexesSmallGI")
#' data("RNADuplexesSampleData")
#' # run whole pipeline
#' result <- runDuplexDiscoverer(
#'     data = SampleSmallGI,
#'     junctions_gr = SampleSpliceJncGR,
#'     anno_gr = SampleGeneAnnoGR,
#'     sample_name = "run_example",
#'     lib_type = "SE",
#'     table_type = "STAR"
#' )
#' # access results
#' show(result)
#' gi_clusters <- dd_get_duplex_groups(result)
#' gi_reads <- dd_get_chimeric_reads(result)
#' df_reads <- dd_get_reads_classes(result)
#' dd_get_reads_classes(result)
#' dd_get_run_stats(result)
setGeneric("dd_get_duplex_groups", function(object) standardGeneric("dd_get_duplex_groups"))

#' @rdname dd_get_duplex_groups
#' @docType methods
setMethod("dd_get_duplex_groups", "DuplexDiscovererResults", function(object) object@duplex_groups)


#' Accessor for `chimeric_reads` Slot
#'
#' Retrieves the value of the `chimeric_reads` slot in a `DuplexDiscovererResults` object.
#' @param object A `DuplexDiscovererResults` object.
#' @return GInteractions object from the `chimeric_reads` slot.
#' @docType methods
#' @rdname dd_get_chimeric_reads
#' @inherit dd_get_duplex_groups examples
#' @export
setGeneric("dd_get_chimeric_reads", function(object) standardGeneric("dd_get_chimeric_reads"))
#' @docType methods
#' @rdname dd_get_chimeric_reads
setMethod("dd_get_chimeric_reads", "DuplexDiscovererResults", function(object) object@chimeric_reads)

#' Accessor for `reads_classes` Slot
#'
#' Retrieves the value of the `reads_classes` slot in a `DuplexDiscovererResults` object.
#' @param object A `DuplexDiscovererResults` object.
#' @return `tibble` from the `reads_classes` slot.
#' @docType methods
#' @rdname dd_get_reads_classes
#' @inherit dd_get_duplex_groups examples
#' @export
setGeneric("dd_get_reads_classes", function(object) standardGeneric("dd_get_reads_classes"))

#' @docType methods
#' @rdname dd_get_reads_classes
setMethod("dd_get_reads_classes", "DuplexDiscovererResults", function(object) object@reads_classes)

#' Accessor for `chimeric_reads_stats` Slot
#'
#' Retrieves the value of the `chimeric_reads_stats` slot in a `DuplexDiscovererResults` object.
#' @param object A `DuplexDiscovererResults` object.
#' @return `tibble` from the `chimeric_reads_stats` slot.
#' @docType methods
#' @rdname dd_get_chimeric_reads_stats
#' @inherit dd_get_duplex_groups examples
#' @export
setGeneric("dd_get_chimeric_reads_stats", function(object) standardGeneric("dd_get_chimeric_reads_stats"))

#' @docType methods
#' @rdname dd_get_chimeric_reads_stats
setMethod("dd_get_chimeric_reads_stats", "DuplexDiscovererResults", function(object) object@chimeric_reads_stats)


#' Accessor for `run_stats` Slot
#'
#' Retrieves the value of the `run_stats` slot in a `DuplexDiscovererResults` object.
#' @param object A `DuplexDiscovererResults` object.
#' @return `tibble` from the `run_stats` slot.
#' @docType methods
#' @rdname dd_get_run_stats
#' @inherit dd_get_duplex_groups examples
#' @export
setGeneric("dd_get_run_stats", function(object) standardGeneric("dd_get_run_stats"))

#' @docType methods
#' @rdname dd_get_run_stats
setMethod("dd_get_run_stats", "DuplexDiscovererResults", function(object) object@run_stats)

#' Annotate RNA duplexes with features
#'
#' Overlays RNA duplexes with `GRanges` annotation object.
#'
#' @param gi `GInteraction` object to annotate
#' @param anno_gr `GRanges` object with the `keys` columns in the metadata
#' @param keys vectors with the names of the features to use for annotation.
#' @param save_ambig When RNA duplex overlaps multiple features, ambiguous annotation for a single key
#' can be stored in `ambig_list.A` and `ambig_list.B`. `ambig.A` and `ambig.B' fields will be added as the 0/1 label
#' @param ambig_key a which feature to use for recording the annotation ambiguity. Determines the values in `ambig_list.A` and `ambig_list.B`.
#' @param order_key Experimental. In case RNA duplex overlaps multiple features, this key will be used to sort the overlapping features.
#' @param order_vec Experimental. An ordered vector of values in `order_key` annotation feature, which sets the priority in case of feature overlap.
#' @export
#' @return `GInteractions` object with new fields
#' @details
#' For each annotation feature in `keys`, i.e if keys=c(keyname1),
#' then `<keyname1>.A`, `<keyname1>.B` annotation fields will be created, containing the
#' names of overlapping features
#' If no overlap is found for the feature, then filed will have NA
#' @examples
#' data("RNADuplexesSampleData")
#' annotateGI(gi = RNADuplexSampleDGs, anno_gr = SampleGeneAnnoGR)
#'
#' # Prioritisation of the snRNA and lncRNA before mRNA if genes overlap
#' annotateGI(
#'     gi = RNADuplexSampleDGs, anno_gr = SampleGeneAnnoGR,
#'     keys = c("gene_id", "gene_name", "gene_type"),
#'     order_key = "gene_type",
#'     order_vec = c("snRNA", "lncRNA", "protein_coding"),
#'     save_ambig = TRUE
#' )
annotateGI <- function(gi, anno_gr,
    keys = c("gene_name", "gene_type", "gene_id"),
    ambig_key = keys[1],
    save_ambig = TRUE,
    order_key = NULL,
    order_vec = NULL) {
    orininal_columns <- colnames(mcols(gi))

    if (!all(keys %in% colnames(mcols(anno_gr)))) {
        ms <- str_c(setdiff(keys, colnames(mcols(anno_gr))), collapse = ", ")
        message()
        stop("Not all provided keys are found in the anotation GRanges \nmissing keys: ", ms)
    }
    anno_gr <- anno_gr[, keys]
    anno_gr$feature_id <- seq_len(length(anno_gr))
    gi@regions$region_id <- seq_len(length(gi@regions))
    po <- findOverlapPairs(gi@regions, anno_gr)

    # df will be grouped by region from below
    df <- tibble(
        "region_id" = po@first$region_id,
        "overlap_w" = width(pintersect(po))
    ) %>%
        bind_cols(data.frame(mcols(po@second))) %>%
        group_by(region_id) %>%
        mutate(ambig = vctrs::vec_cast(n_distinct(.data[[ambig_key]], na.rm = TRUE) > 1, integer()))


    # save ambig mapping regions (use pre-defined key to save ids)
    if (save_ambig) {
        message("Will save all redundant values in: ", ambig_key)
        ambig_save <- df %>% dplyr::filter(ambig == 1)
        ambig_save <- ambig_save %>%
            dplyr::filter(!is.na(.data[[ambig_key]])) %>%
            group_by(region_id) %>%
            dplyr::summarise(ambig_list = paste(unique(.data[[ambig_key]]), collapse = ",")) %>%
            ungroup()
    }

    # if we use some column to sort the annotation for resolution of ambiguities
    if (!is.null(order_key) & !is.null(order_vec)) {
        # Order by the overlap amount first, then by the order key (gene_type)
        message("Sorting the annotation layers by: ", order_key)
        if ((order_key %in% colnames(df) && (order_key %in% keys))) {
            df <- df %>%
                mutate(sort_col = factor(.data[[order_key]], levels = order_vec, ordered = TRUE)) %>%
                arrange(sort_col) %>%
                select(-sort_col) # sort by provided order
        } else {
            warning(
                "Provided annotaion order key", order_key,
                "does not exist in annotation or not part of the keys, skipping"
            )
        }
    }

    df <- df %>%
        tidyr::fill(all_of(keys), .direction = "up") %>%
        slice(1) %>%
        ungroup()

    dt <- left_join(as_tibble(mcols(gi@regions)), df, by = "region_id")

    if (save_ambig) {
        dt <- left_join(dt, ambig_save, by = "region_id")
    }

    dt <- dt %>%
        select(-c(feature_id, overlap_w, region_id))

    cnames <- c(str_c(colnames(dt), ".A"), str_c(colnames(dt), ".B"))

    dt <- cbind(dt[gi@anchor1, ], dt[gi@anchor2, ])
    colnames(dt) <- cnames
    dt <- dt %>% dplyr::select(order(colnames(dt)))

    if (save_ambig) {
        dt <- dt %>%
            relocate(ambig.A, ambig.B, ambig_list.A, ambig_list.B, .after = last_col())
    } else {
        dt <- dt %>% dplyr::select(-c(ambig.A, ambig.B, ambig_list.A, ambig_list.B))
    }

    dt <- dt %>% data.frame()
    mcols(gi) <- cbind(mcols(gi)[orininal_columns], dt)
    gi@regions$region_id <- NULL
    return(gi)
}



#' Annotate RNA-RNA interactions as cis- and trans-
#'
#' @description
#' Annotated each entry gi object as cis, if the .A and .B arms correspond to
#' the same feature (i.e transcript_id or gene_id)
#' If the values are are equal, then annotated with value: `cis` = 1, If not equal
#' or `NA`: `cis` = 0
#'
#' @param gi `GInteractions` object containing two metadata columns as feature annotation
#' @param id_col_base base name of the feature id columns to use. Function will
#' look for <id_col_base>.A and <id_col_base>.B columns and compare them
#' @keywords internal
#' @return gi `GInteractions` object containing `cis` field with 0/1 values
.annotateCisTrans <- function(gi, id_col_base = "gene_id") {
    gi$cis <- 0

    name_col1 <- paste0(id_col_base, ".A")
    name_col2 <- paste0(id_col_base, ".B")

    gi$c1 <- mcols(gi)[, name_col1]
    gi$c2 <- mcols(gi)[, name_col2]


    if (length(gi[!is.na(gi$c1) & !is.na(gi$c2) &
        (gi$c1 == gi$c2)]) != 0) {
        gi[!is.na(gi$c1) & !is.na(gi$c2) &
            (gi$c1 == gi$c2)]$cis <- 1
        gi$c1 <- NULL
        gi$c2 <- NULL
        return(gi)
    } else {
        message("Could not do cis/trans annotation because columns content is missing ")
        return(gi)
    }
}

#' Helper function to add count data to metadata of `GInteractions`
#'
#' Merges the count dataframe and interactions metadata by `id_col`
#' If key is not found, in metadata throws error

#' @param gi `GInteractions`
#' @param df_counts dataframe with read counts
#' @param id_col key to use in merge
#' @return `GInteractions` with added counts
.addGeneCounts <- function(gi, df_counts, id_col = "gene_id") {
    df_counts <- df_counts %>% as.data.frame()
    df_all_cts <- as_tibble(data.frame("RNA" = unname(df_counts[1]), "n" = unname(df_counts[2])))

    id_columns <- str_c(id_col, c(".A", ".B"))

    if (!all(id_columns %in% colnames(mcols(gi)))) {
        stop(id_columns, " are not found in the input gi")
    }
    colnames(df_all_cts) <- c(id_columns[1], "gene_count.A")
    gi$gene_count.A <- left_join(
        as_tibble(mcols(gi)[id_columns[1]]),
        df_all_cts
    ) %>% pull(gene_count.A)
    colnames(df_all_cts) <- c(id_columns[2], "gene_count.B")
    gi$gene_count.B <- left_join(
        as_tibble(mcols(gi)[id_columns[2]]),
        df_all_cts
    ) %>% pull(gene_count.B)
    return(gi)
}
#' Calculate ligation-based significance statistics for RNA duplexes
#'
#' Calculates significance statistics for each gene/transcript pair represented
#' in a `GInteractions` object. Two optional models are supported:
#'
#' 1. A binomial random-ligation model, where pair probability is derived from
#'    per-gene/transcript counts and tested using the right-tail binomial test.
#' 2. An odds-ratio / Fisher exact test based on a 2x2 contingency table using
#'    chimeric counts and singleton counts. This test is used in RBP-dependent and
#'    RNA probing methods like RIL-seq
#'
#' The function also outputs abundance fractions for each duplex pair and counts
#' of other RNA duplexes formed by either or neither gene/transcript in the pair.
#'
#' @param gi `GInteractions` object annotated with gene/transcript names.
#' @param df_counts `data.frame`. A two-column data frame with gene/transcript
#'   counts. The first column should match the `<id_col>.A` / `<id_col>.B`
#'   identifiers in `gi`. The second column is the corresponding count.
#' @param id_col `character(1)`. Prefix for gene/transcript metadata fields in
#'   `gi`. Two fields, `<id_col>.A` and `<id_col>.B`, are expected.
#' @param compute_binom `logical(1)`. Whether to compute the binomial
#'   random-ligation test. Default is `TRUE`.
#' @param compute_odds `logical(1)`. Whether to compute the odds-ratio /
#'   Fisher exact test. Default is `TRUE`.
#'
#' @return `GInteractions` object with added metadata columns.
#'
#' @details
#' **Binomial model**
#'
#' Null hypothesis: an observed duplex is generated by random ligation.
#'
#' Alternative hypothesis: the duplex is enriched above the random-ligation
#' expectation.
#'
#' The random-ligation probability \eqn{P(a, b)} is defined as:
#'
#' \deqn{
#' P(a, b) \propto
#' \begin{cases}
#'     2 \cdot P(a) \cdot P(b) & \text{if } a:b \text{ is observed and } a \neq b \\
#'     P(a)^2                  & \text{if } a:b \text{ is observed and } a = b \\
#'     0                       & \text{otherwise}
#' \end{cases}
#' }
#'
#' where:
#'
#' \deqn{
#' P(a) = \frac{\text{N reads}(a)}{\text{total N reads}}
#' }
#'
#' The observed-pair probabilities are renormalized to sum to one and the
#' p-value is computed from the right tail of the binomial distribution.
#'
#' **Odds-ratio / Fisher model**
#'
#' For a pair `(A, B)`, the contingency table is:
#'
#' \tabular{lll}{
#'                 \tab B present \tab B absent\cr
#'   A present     \tab C1        \tab C2\cr
#'   A absent      \tab C3        \tab C4\cr
#' }
#'
#' with:
#' \itemize{
#'   \item `C1`: chimeric fragments containing both `A` and `B`
#'   \item `C2`: fragments containing `A` but not `B`
#'   \item `C3`: fragments containing `B` but not `A`
#'   \item `C4`: fragments containing neither `A` nor `B`
#' }
#'
#' A one-sided Fisher exact test (`alternative = "greater"`) is used.
#'
#' @examples
#' data("RNADuplexesSampleData")
#' gi <- calculateLigationPvalues(
#'     RNADuplexSampleDGs,
#'     df_counts = RNADuplexesGeneCounts
#' )
#' hist(gi$p.adj, breaks = 20)
#'
#' @export
calculateLigationPvalues <- function(gi,
    df_counts,
    id_col = "gene_id",
    compute_binom = TRUE,
    compute_odds = FALSE) {
    df_counts <- df_counts %>% as.data.frame()
    if (!(compute_binom || compute_odds)) {
        stop("Error in calculateLigationPvalues at least one compute_binom or compute_odds must be TRUE")
    }


    df_all_cts <- as_tibble(data.frame(
        RNA = unname(df_counts[[1]]),
        n = unname(df_counts[[2]])
    ))

    id_columns <- str_c(id_col, c(".A", ".B"))

    if (!all(id_columns %in% colnames(mcols(gi)))) {
        stop(id_columns, " are not found in the input gi")
    }

    chim_df <- as_tibble(mcols(gi)) %>%
        dplyr::select(c(n_reads, !!!id_columns))
    colnames(chim_df) <- c("n", "Araw", "Braw")
    chim_df$chim_id <- seq_len(nrow(chim_df))
    # filter not annotated chimeras and those for which we don't have record in counts df
    unique_ids <- unique(df_all_cts$RNA)


    # keep original row-to-pair mapping so we can join back later
    chim_df$AB <- apply(chim_df[c("Araw", "Braw")], 1, function(row) {
        paste(sort(row), collapse = "<>")
    })
    chimdf_save <- chim_df[c("chim_id", "AB")]
    chim_df <- chim_df %>%
        group_by(AB) %>%
        summarise(n_reads_chim_total = sum(n), .groups = "drop")

    ssplit <- str_split_fixed(chim_df$AB, "<>", n = 2)
    chim_df$A <- ssplit[, 1]
    chim_df$B <- ssplit[, 2]

    chim_df <- chim_df %>%
        left_join(
            tibble(idA = df_all_cts$RNA, nA = df_all_cts$n),
            by = c("A" = "idA")
        ) %>%
        left_join(
            tibble(idB = df_all_cts$RNA, nB = df_all_cts$n),
            by = c("B" = "idB")
        ) %>%
        dplyr::filter((A %in% unique_ids) & (B %in% unique_ids))

    N_chim_total <- sum(chim_df$n_reads_chim_total)
    N_total <- sum(df_all_cts$n)
    N_frag_total <- N_total + N_chim_total

    # pair-level abundance
    chim_df <- chim_df %>%
        mutate(
            Pa = nA / N_total,
            Pb = nB / N_total,
            Pab = ifelse(A != B, 2 * Pa * Pb, Pa * Pa)
        )
    df <- data.frame(chim_df[c("A", "B", "n_reads_chim_total")]) %>%
        dplyr::rename(n = "n_reads_chim_total")

    # For odds- test count chimeras with A but not B, with B but not A, and with neither
    chim_df$chim_withA_noB <- apply(df, 1, function(row) {
        df %>%
            dplyr::filter(
                (A == row["A"] | B == row["A"]) &
                    (A != row["B"] & B != row["B"])
            ) %>%
            pull(n) %>%
            sum()
    })

    chim_df$chim_withB_noA <- apply(df, 1, function(row) {
        df %>%
            dplyr::filter(
                (A == row["B"] | B == row["B"]) &
                    (A != row["A"] & B != row["A"])
            ) %>%
            pull(n) %>%
            sum()
    })

    chim_df$chim_noB_noA <- apply(df, 1, function(row) {
        df %>%
            dplyr::filter(
                (A != row["B"] & B != row["B"]) &
                    (A != row["A"] & B != row["A"])
            ) %>%
            pull(n) %>%
            sum()
    })

    # singleton fpor odds
    chim_df$nonchimA <- vapply(
        vctrs::vec_cast(chim_df$A, character()),
        function(x) {
            df_all_cts %>%
                dplyr::filter(RNA == x) %>%
                pull(n)
        },
        numeric(1)
    )

    chim_df$nonchimB <- vapply(
        vctrs::vec_cast(chim_df$B, character()),
        function(x) {
            df_all_cts %>%
                dplyr::filter(RNA == x) %>%
                pull(n)
        },
        numeric(1)
    )

    # unify output
    chim_df$pval <- NA_real_
    chim_df$p.adj <- NA_real_
    chim_df$odds_ratio <- NA_real_
    chim_df$odds_pval <- NA_real_
    chim_df$odds_p.adj <- NA_real_

    if (compute_binom) {
        chim_df$Pab_norm <- chim_df$Pab / sum(chim_df$Pab)

        chim_df$pval <- pbinom(
            chim_df$n_reads_chim_total - 1,
            size = N_chim_total,
            prob = chim_df$Pab_norm,
            lower.tail = FALSE
        )
        chim_df$p.adj <- p.adjust(chim_df$pval, method = "BH")
    } else {
        chim_df$Pab_norm <- NA_real_
    }

    if (compute_odds) {
        # 2x2 table - odds test similar to the one in bacterial methods :
        #            B present                  B absent
        # A present  C1 = AB chimera           C2 = A only
        # A absent   C3 = B only               C4 = neither A nor B
        chim_df$C1 <- chim_df$n_reads_chim_total
        chim_df$C2 <- chim_df$chim_withA_noB + chim_df$nonchimA
        chim_df$C3 <- chim_df$chim_withB_noA + chim_df$nonchimB
        chim_df$C4 <- N_frag_total - chim_df$C1 - chim_df$C2 - chim_df$C3

        odds_res <- lapply(seq_len(nrow(chim_df)), function(i) {
            mat <- matrix(
                c(
                    chim_df$C1[i], chim_df$C2[i],
                    chim_df$C3[i], chim_df$C4[i]
                ),
                nrow = 2,
                byrow = TRUE
            )

            ft <- stats::fisher.test(mat, alternative = "greater")

            data.frame(
                odds_ratio = unname(ft$estimate),
                odds_pval = ft$p.value
            )
        })

        odds_res <- dplyr::bind_rows(odds_res)
        chim_df$odds_ratio <- odds_res$odds_ratio
        chim_df$odds_pval <- odds_res$odds_pval
        chim_df$odds_p.adj <- p.adjust(chim_df$odds_pval, method = "BH")
    } else {
        chim_df$C1 <- NA_real_
        chim_df$C2 <- NA_real_
        chim_df$C3 <- NA_real_
        chim_df$C4 <- NA_real_
    }

    counts_stats <- chim_df %>%
        dplyr::select(
            AB,
            pval, p.adj, Pa, Pb,
            chim_withA_noB, chim_withB_noA, chim_noB_noA,
            nonchimA, nonchimB,
            C1, C2, C3, C4,
            odds_ratio, odds_pval, odds_p.adj
        ) %>%
        dplyr::rename(
            p_val = pval,
            count.chim.A.notB = chim_withA_noB,
            count.chim.notA.B = chim_withB_noA,
            count.chim.notA.notB = chim_noB_noA
        )

    chimdf_save <- chimdf_save %>%
        left_join(counts_stats, by = "AB")

    statcounts_final <- tibble(chim_id = seq_len(length(gi))) %>%
        left_join(chimdf_save, by = "chim_id") %>%
        dplyr::select(-c(chim_id, AB)) %>%
        data.frame()

    mcols_old <- mcols(gi) %>%
        data.frame() %>%
        as_tibble() %>%
        dplyr::select(-any_of(colnames(statcounts_final))) %>%
        data.frame()

    mcols(gi) <- cbind(mcols_old, statcounts_final)

    gi <- .addGeneCounts(gi, df_counts)

    return(gi)
}

.checkRNAduplexinstalled <- function() {
    returncode <- system2("RNAduplex", args = c("-h"), stdout = NULL, stderr = NULL)
    if (returncode != 0) {
        return(1)
    } else {
        return(0)
    }
}

.runRNAduplex <- function(RNA1, RNA2) {
    input <- paste(RNA1, RNA2, sep = "\n")
    result <- system2("RNAduplex", input = input, stdout = TRUE)
    return(result)
}

.getDuplexString <- function(gi, fafile) {
    s <- Biostrings::readBStringSet(fafile)
    s <- Biostrings::RNAStringSet(s)
    newnames <- str_squish(str_sub(names(s), 1, 5))
    names(s) <- newnames
    gi$seq1 <- s[get_arm_a(gi)]
    gi$seq2 <- s[get_arm_b(gi)]
    return(gi)
}


.getGCContent <- function(seqrna) {
    # seqrna is a RNAstringset based on the arm
    gc_freq <- Biostrings::letterFrequency(seqrna, c("G", "C"))
    gc_content <- rowSums(gc_freq) / width(seqrna) * 100
    return(gc_content)
}

#' @name DuplexTrack
#' @title  class for the visualization of RNA duplexes

#' @description
#' Inherits the \code{Gviz::AnnotationTrack}, plots interaction ranges as boxes.
#' Arguments from \code{Gviz::AnnotationTrack}, as `stacking` which set boxes
#' layout are accepted. Parent aesthetics for labels are overwritten with Display
#' parameters of this class.
#' Accepts `GInteractions` object to plot and `GRanges` to define plot region
#'
#' Duplexes which can be displayed on the plot range are connected with arcs.
#' Duplexes which are partially outside of the range are displayed without arcs.
#' Labeles and appearance can be controlled with display parameters
#'
#' @param gi An `GInteractions` object
#' @param gr_region `GRanges` region for plotting
#' @param from Integer start coordinate of subset region. Used if `gr_region` is not provided
#' @param to Integer end coordinate of subset region. Used if `gr_region` is not provided
#' @param chromosome Chromosome of subset region. Used if `gr_region` is not provided
#' @param strand Used if `gr_region` is not provided
#' @param fill.column used for fill. Default is "" (empty) and triggers IGV color pallete.
#' **Display parameters**
#' \describe{
#'   \item{arcs.color}{Character. Color of the arcs. Default is "black".}
#'   \item{arc.location}{Character in c('inner','outer','midpoint'). Location of the arcs in X axis relative to range.
#'   Default is "inner"}
#'   \item{labels.v.offset.base}{Numeric. Base vertical offset for the labels. Default is 0.2. Other offesets are added to it.
#'   }
#'   \item{labels.v.offset.trans}{Numeric. Vertical offset for trans labels. Applied when one part of the duplex is outside of the plot.
#'    Recommended ranges are in -0.5 to 0.5 Default is 0.0.   }
#'   \item{labels.h.offset.trans}{Numeric. Horizontal offset for trans labels.
#'    Applied when one part of the duplex is outside of the plot
#'   Value is in nucleotide units. Default is 0.0. }
#'   \item{labels.v.offset.cis}{Numeric. Vertical offset for cis labels.  Recommended ranges are in -0.5 to 0.5 Default is 0.0. Default is 0.0. }
#'   \item{labels.h.offset.cis}{Numeric. Horizontal offset for cis labels.  Value is in nucleotide units. Default is 0.0.}
#'   \item{labels.fontsize}{Numeric. Font size of the labels. Default is 18.}
#'   \item{label.cis.above}{Logical. Whether the cis labels should be above. When set to FALSE, labels are plot for each box separately.
#'   Default is TRUE}
#'   \item{annotation.column1}{Character. First annotation column to use for labels. Default is "group" and generated internally.}
#'   \item{annotation.column2}{Character. Second annotation column to use for labels. Default is "" (empty).}
#'   \item{fill.column}{Character. Column used for fill. Default is "" (empty) and triggers IGV color pallete.}
#'   \item{labels.color}{Character. Color of the labels. Default is 'black'.}
#'   \item{labels.align}{Character. Alignment of the labels. Default is 'center'.
#'   Possible values are in c('left','right','center)}
#'   \item{arcConstrain}{Numeric. Minimum gap distance between arms of the
#'    interaction to draw arcs}
#' }
#' @export DuplexTrack
#' @examples
#' library(InteractionSet)
#' library(Gviz)
#' # generate input
#' anchor1 <- GRanges(
#'     seqnames = "chr1",
#'     ranges = IRanges(
#'         start = c(100, 600, 1100, 1600, 2100, 150, 400),
#'         end = c(200, 700, 1200, 1700, 2200, 250, 500)
#'     ),
#'     strand = "+"
#' )
#' anchor2 <- GRanges(
#'     seqnames = "chr1",
#'     ranges = IRanges(
#'         start = c(300, 800, 1300, 1800, 2300, 1500, 1700),
#'         end = c(400, 900, 1400, 1900, 2400, 1600, 1800)
#'     ),
#'     strand = "+"
#' )
#'
#' interactions <- GInteractions(anchor1, anchor2, mode = "strict")
#' # define plotting range
#' gr_region <- range(anchor1, anchor2)
#' interactions$anno_A <- sample(LETTERS, length(interactions))
#' interactions$anno_B <- interactions$anno_A
#' a <- DuplexTrack(interactions, gr_region = gr_region, stacking = "dense")
#' plotTracks(a, stacking = "dense")
#' plotTracks(a, stacking = "squish", annotation.column1 = "anno_A")
#'
#' # add interactions which are not fully in plot range: outside the range or on different chromosome()
#'
#' # one left (A) interaction arm outside of the plot, other on different chromosome
#' new_anchor1 <- GRanges(
#'     seqnames = c("chr1", "chr2"),
#'     ranges = IRanges(
#'         start = c(10, 600),
#'         end = c(90, 700)
#'     ),
#'     strand = "+"
#' )
#' new_anchor2 <- GRanges(
#'     seqnames = c("chr1", "chr1"),
#'     ranges = IRanges(
#'         start = c(1500, 1000),
#'         end = c(1600, 1200)
#'     ),
#'     strand = "+"
#' )
#'
#' new_interactions <- GInteractions(new_anchor1, new_anchor2)
#' new_interactions$anno_A <- c("A.out", "A.out_chr")
#' new_interactions$anno_B <- c("B.in", "B.in")
#' all_interactions <- c(interactions, new_interactions)
#'
#' b <- DuplexDiscovereR::DuplexTrack(all_interactions,
#'     gr_region = gr_region,
#'     annotation.column1 = "anno_A",
#'     annotation.column2 = "anno_B"
#' )
#'
#' plotTracks(b)
#'
#' # to customize plot, one can call, to see options
#' DuplexDiscovereR::availableDisplayPars(b)
#'
setClass(
    "DuplexTrack",
    contains = c("AnnotationTrack"),
    representation = representation(
        giobject = "StrictGInteractions",
        name = "character",
        gr_region = "GenomicRanges"
    ),
    prototype = prototype(
        name = "DuplexTrack",
        dp = DisplayPars(
            arcs.color = "black",
            arc.location = "inner",
            arcConstrain = 4,
            labels.v.offset.base = 0.2,
            labels.v.offset.trans = 0.0,
            labels.h.offset.trans = 0.0,
            labels.v.offset.cis = 0.0,
            labels.h.offset.cis = 0.0,
            labels.fontsize = 18,
            label.cis.above = TRUE,
            annotation.column1 = "group",
            annotation.column2 = "",
            fill.column = "",
            labels.color = "black",
            labels.align = "center"
        )
    )
)

setMethod("initialize", "DuplexTrack", function(.Object, ...) {
    if (is.null(list(...)$range) && is.null(list(...)$genome) && is.null(list(...)$chromosome)) {
        return(.Object)
    }
    # display parameter defaults
    .Object <- .gv_updatepars(.Object, "DuplexTrack")
    range <- list(...)$range
    giobject <- list(...)$giobject
    .Object@giobject <- giobject
    .Object@gr_region <- list(...)$gr_region

    .Object <- callNextMethod()
    message("init")

    return(.Object)
})

DuplexTrack <- function(
        gi, start = NULL, end = NULL, gr_region = NULL,
        group, id, strand, chromosome, fill = NULL, fill.column = "",
        genome, stacking = "squish", name = "DuplexTrack", selectFun, importFunction,
        stream = FALSE, ...) {
    message("constructor")
    ## Some defaults
    if (is.null(gr_region)) {
        if (!any(c(
            is.null(start), is.null(strand),
            is.null(strand), is.null(chromosome)
        ))) {
            gr_region <- GRanges(as.character(chromosome),
                IRanges(start, end),
                strand = strand
            )
        } else {
            stop("start, end, chromosome and strand OR Granges should be provided")
        }
    } else {
        message("Using Granges for plot region")
    }

    # first select gi where at least single region is on the range
    gi_sbs <- subsetByOverlaps(gi, gr_region)
    if (length(gi_sbs) == 0) {
        errorCondition(message("No overlap found for the object ot plot and defined plotting range"))
    }

    # load fill colors or use pallete
    if (!is.null(fill.column) & (fill.column %in% colnames(mcols(gi_sbs)))) {
        # use colum with fill
        fill <- mcols(gi)[, fill.column]
    } else {
        # Use igv pallete
        mypal <- pal_igv("default", alpha = 0.9)(length(gi_sbs))
        message("Using IGV pallete for fill")
        mypal <- ggsci::pal_igv("default", alpha = 0.9)(length(gi_sbs))
        fill <- mypal
    }
    range_meta <- tibble(gap = pairdist(gi_sbs, type = "gap"), ) %>%
        mutate(
            in_range = ifelse(is.na(gap), 0, 1),
            arms_overlap = ifelse(in_range == 1 & gap <= 0, 1, 0)
        ) %>%
        data.frame()
    gi_sbs$gap <- NULL
    gi_sbs$arms_overlap <- NULL
    gi_sbs$in_range <- NULL
    mcols(gi_sbs) <- cbind(data.frame(mcols(gi_sbs)), range_meta)
    # select those boxes which are on the plotting range
    gi_boxes <- convert_gi_to_ranges(gi_sbs)
    range <- subsetByOverlaps(gi_boxes, gr_region)
    if (length(range) == 0) {
        stop("No overlaps found")
    }
    add_meta <- as_tibble(mcols(range)[c("id", "group")]) %>%
        mutate(arm = str_split_i(id, "\\.", 2)) %>%
        group_by(group) %>%
        mutate(n = n()) %>%
        ungroup() %>%
        mutate(
            full_in_plot = if_else(n == 2, 1, 0),
            pair_arm = if_else(arm == "B", "A", "B")
        ) %>%
        dplyr::select(arm, full_in_plot, pair_arm) %>%
        data.frame()

    mcols(range) <- cbind(mcols(range), add_meta)
    range$feature <- "unknown"
    range$density <- 1

    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- if (length(range) > 0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    }
    ## And finally the object instantiation, we have to distinguish between
    # DetailsAnnotationTracks and normal ones
    genome <- rtracklayer::genome(range)
    return(new("DuplexTrack",
        giobject = (gi), start, end,
        chromosome = as.character(chromosome[1]), range = range,
        gr_region = gr_region,
        fill = fill, groupAnnotation = "group",
        name = name, genome = genome, stacking = stacking, ...
    ))
}


#' Show method for DuplexTrack
#'
#' @param object DuplexTrack.
#' @importFrom methods slotNames show
#' @return class representation
#' @export
#'
#' @examples
#' library(InteractionSet)
#' anchor1 <- GRanges(
#'     seqnames = "chr1",
#'     ranges = IRanges(
#'         start = c(100, 600, 1100, 1600, 2100),
#'         end = c(200, 700, 1200, 1700, 2200)
#'     ),
#'     strand = "+"
#' )
#' anchor2 <- GRanges(
#'     seqnames = "chr1",
#'     ranges = IRanges(
#'         start = c(300, 800, 1300, 1800, 2300),
#'         end = c(400, 900, 1400, 1900, 2400)
#'     ),
#'     strand = "+"
#' )
#'
#' interactions <- GInteractions(anchor1, anchor2, mode = "strict")
#' gr_region <- range(anchor1, anchor2)
#' a <- DuplexTrack(interactions, gr_region = gr_region, stacking = "dense")
#' show(a)
setMethod("show", signature(object = "DuplexTrack"), function(object) {
    slots <- methods::slotNames(object)
    cat(names(object), " class")
    selected_slots <- slots[slots != "dp"]
    for (slot_name in selected_slots) {
        cat((slot_name), ": \n")
        show(slot(object, slot_name))
    }
    pars_to_display <- names(availableDisplayPars(class(object)))
    plist <- object@dp@pars[pars_to_display]
    cat("Display parameters: \n")
    cat(paste(names(plist), plist, sep = ": ", collapse = "\n"))
})


#' Draw method for DuplexTrack
#'
#' `Gviz::AnnotationTrack` stacking algorithm is used to calculate vertical
#' distribution of boxes for the interactions.
#' Boxes coordinates are later imported for placing labels and arcs
#' @keywords internal
#' @returns pushes boxes, arcs and labels to viewport
setMethod("drawGD", signature("DuplexTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...) {
    imageMap(GdObject) <- NULL
    if (!length(GdObject)) {
        return(invisible(GdObject))
    }
    # Brought here from AnnotationTrack
    ## In prepare mode we need to make sure that the stacking information is updated from the optional display parameter (by calling
    ## the StackedTrack drawGD method) and also perform the collapsing of track items which could potentially lead to re-stacking.
    if (prepare) {
        message("prepare")
        pushViewport(viewport(xscale = c(minBase, maxBase), yscale = c(0, 1)))
        popViewport(1)
        return(invisible(GdObject))

        message("P. start: ", minBase)
        message("P. end: ", maxBase)
    }
    ## If there are too many stacks for the available device resolution we cast an error, otherwise we set the viewport
    bins <- stacks(GdObject)
    stacks <- max(bins)
    rev <- Gviz:::.dpOrDefault(GdObject, "reverseStrand", FALSE)

    temp_info <- GdObject@range

    # vector defining ids which are present in both reads
    vec_both_present <- as.integer(names(table(temp_info$group)[which(table(temp_info$group) == 2)]))
    # set the row of reads
    mcols(temp_info)$row_plot_id <- as.vector(bins)
    # provide info if gap is sufficient for arcs
    arcConstrain <- Gviz:::.dpOrDefault(GdObject, "arcConstrain")
    mcols(temp_info)$distance_gap <- ifelse(mcols(temp_info)$gap > arcConstrain, 1, 0)
    mcols(temp_info)$distance_gap[is.na(mcols(temp_info)$distance_gap)] <- 0
    # provide info about height required for blocks
    vec_double_height <- c()
    for (internal in seq_len(stacks)) {
        tmp <- temp_info[which(mcols(temp_info)$row_plot_id == internal)]
        unique_dist <- unique(mcols(tmp)$distance_gap)
        vec_double_height <- c(vec_double_height, if (length(unique_dist) == 1) unique_dist else 1)
    }
    vec_double_height_axis <- vec_double_height + 1
    vec_double_text <- 1 - vec_double_height

    # create boxes height
    # for usual box we will use 1 and for for boxes with requirements of arc 2
    # plus we should have text size
    annot_size <- Gviz:::.dpOrDefault(GdObject, "labels.v.offset.base")
    xscale <- if (!rev) c(minBase, maxBase) else c(maxBase, minBase)

    height_formula <- sum(vec_double_height_axis) + sum(vec_double_text * 0.05) + 1
    yscale <- if (!Gviz:::.dpOrDefault(GdObject, "reverseStacking", FALSE)) c(1, height_formula) else c(height_formula, 1)

    pushViewport(dataViewport(xscale = xscale, extension = 0, yscale = yscale, clip = TRUE))
    res <- Gviz:::.pxResolution(coord = "x")
    curVp <- Gviz:::vpLocation()

    if (curVp$size["height"] / stacks < Gviz:::.dpOrDefault(GdObject, "min.height", 3)) {
        stop("Too many stacks to draw. Either increase the device size or limit the drawing to a smaller region.")
    }

    ## We adjust the color saturation to indicate overplotting if necessary
    if (Gviz:::.dpOrDefault(GdObject, "showOverplotting", FALSE)) {
        dens <- as.numeric(values(GdObject)$density)
        if (length(unique(dens)) != 1) {
            minSat <- max(0.25, 1 / max(dens))
            minDens <- min(dens)
            saturation <- minSat + ((dens - minDens) / rDens / (1 / (1 - minSat)))
            bc <- unique(Gviz:::.getBiotypeColor(GdObject))
            baseCol <- rgb2hsv(col2rgb(bc))
            desatCols <- unlist(lapply(saturation, function(x) hsv(baseCol[1, ], x, baseCol[3, ])))
            names(desatCols) <- paste(unique(feature(GdObject)), rep(dens, each = length(bc)), sep = "_")
            feature(GdObject) <- paste(feature(GdObject), dens, sep = "_")
            desatCols <- desatCols[unique(names(desatCols))]
            displayPars(GdObject) <- as.list(desatCols)
        }
    }
    # Starting with boxes coordinates
    # bottom and top positions for every case
    internal <- 1
    vector_fin_bot <- c()
    vector_fin_top <- c()
    for (internal in seq(1, stacks)) {
        val_based_box_height <- seq(1, stacks)[internal]
        val_text_height <- sum((vec_double_text * annot_size)[seq_len(internal)])
        val_arc_height <- sum(vec_double_height[c(seq_len(internal))])
        val_fin_bot <- height_formula - val_based_box_height - val_text_height - val_arc_height
        val_fin_top <- val_fin_bot + 1
        vector_fin_bot <- c(vector_fin_bot, val_fin_bot)
        vector_fin_top <- c(vector_fin_top, val_fin_top)
        # print(paste(paste("Bottom coordinates row:",internal,"   "),height_formula,val_based_box_height,val_text_height,val_arc_height,val_fin_bot,sep="|"))
    }
    df_coord <- data.frame(row_plot_id = seq(1, stacks), top = vector_fin_top, bot = vector_fin_bot)

    ## Now we can pre-compute all the coordinates and settings for the elements to be drawn, ...
    box <- Gviz:::.boxes(GdObject, (stacks - bins) + 1)
    tmp_box <- merge(box, mcols(temp_info)[, c("row_plot_id", "id")], by = "id")
    tmp_box <- merge(tmp_box, df_coord, by = "row_plot_id")
    tmp_box$cy1 <- tmp_box$bot
    tmp_box$cy2 <- tmp_box$top
    tmp_box$textY <- tmp_box$top + annot_size
    row.names(tmp_box) <- tmp_box$id
    box <- tmp_box

    ## all the necessary display parameters from gviz
    barsAndLab <- Gviz:::.barsAndLabels(GdObject)
    bar <- barsAndLab$bars
    bartext <- barsAndLab$labels
    shape <- "box"

    # Taken from AnnotationTrack
    col.line <- Gviz:::.dpOrDefault(GdObject, "col.line")[1]
    border <- Gviz:::.dpOrDefault(GdObject, "col")[1]
    lwd <- Gviz:::.dpOrDefault(GdObject, "lwd", 2)
    lty <- Gviz:::.dpOrDefault(GdObject, "lty", 1)
    alpha <- Gviz:::.dpOrDefault(GdObject, "alpha", 1)
    rotation <- Gviz:::.dpOrDefaultFont(GdObject, "rotation", "item", 0)
    rotation.group <- Gviz:::.dpOrDefaultFont(GdObject, "rotation", "group", 0)
    just <- Gviz:::.dpOrDefault(GdObject, "just.group", "above")

    # Our added dupslaypars
    color.arcs <- Gviz:::.dpOrDefault(GdObject, "arcs.color")
    font_gpar <- Gviz:::.fontGp(GdObject, subtype = "group")
    font_gpar$fontsize <- displayPars(GdObject, "labels.fontsize")
    font_gpar$col <- displayPars(GdObject, "labels.color")
    arc.location <- displayPars(GdObject, "arc.location")
    labels.v.offset.trans <- displayPars(GdObject, "labels.v.offset.trans")
    labels.h.offset.trans <- displayPars(GdObject, "labels.h.offset.trans")
    labels.v.offset.cis <- displayPars(GdObject, "labels.v.offset.cis")
    labels.h.offset.cis <- displayPars(GdObject, "labels.h.offset.cis")
    labels.align <- displayPars(GdObject, "labels.align")
    label.cis.above <- displayPars(GdObject, "label.cis.above")
    box.fill.colors <- displayPars(GdObject, "fill")
    # can be custom, but not tested for now
    height_factor_box <- 0.8
    height_factor_arc <- 0.8

    ## Plotting of the boxes
    message("Plotting Boxes")
    box$col <- if (is.na(border)) box$fill else border
    fill_vector <- box.fill.colors
    # assign correct fill colours
    if (!is.null(fill_vector)) {
        # Get unique indexes for each pair
        pair_indexes <- unique(sub("\\..*", "", rownames(box)))
        # Create a named vector with pairs and their corresponding colors
        pair_colors <- setNames(fill_vector, pair_indexes)
        # Map colors to each pair
        pair_colors_vector <- vapply(sub("\\..*", "", rownames(box)), function(index) pair_colors[[index]], "")
        box$fill <- as.vector(pair_colors_vector)
    }

    # adjust before plotting boxes
    box2 <- box
    hhe <- box2$cy2 - box2$cy1
    height_orig <- hhe[1]
    height_box <- hhe[1] * height_factor_box
    height_red <- height_orig - height_box
    box2$cy2 <- box2$cy1 + height_box
    box2$textY <- box2$textY - height_red

    # plot boxes
    .gv_plotboxes(box2, lwd = lwd, lty = lty, alpha = alpha)
    box <- box2

    message("Plotting Labels")
    # EGORS for compatibility tmp box is
    tmp_box <- box
    # create arcs
    message("Plotting arcs")
    # select unique ids with both arms available
    result <- as_tibble(tmp_box) %>%
        dplyr::filter(gap > arcConstrain) %>%
        group_by(group = sub("\\..*", "", id)) %>%
        summarize(has_A = any(grepl("A", id)), has_B = any(grepl("B", id))) %>%
        dplyr::filter(has_A & has_B)
    # if no ars are to plot - don't try to call graphics to avoid confusion
    if (nrow(result) == 0) {
        stop("Cannot plot provided region. Possible reason: interactions overlap.
        Suggestion: try to convert input to long GRanges and plot with standard Gviz Annotation track")
    }
    pass_arcs <- result$group

    # Egor S added df for max arc height
    arc_h_list <- list()
    df_arc_y <- data.frame()
    if (length(pass_arcs) != 0) {
        for (internal in seq_along(pass_arcs)) {
            grp_var <- paste(pass_arcs[internal], c("A", "B"), sep = ".")
            tmp <- tmp_box[grp_var, ]
            # options for creating arcs start end end coordinates
            # 1) midpoint
            # 2) inner
            # 3) outer
            if (arc.location == "midpoint") {
                points_x <- (abs(tmp$end) + abs(tmp$start)) / 2
                point_y <- max(tmp$cy2)
            } else if (arc.location == "inner") {
                points_x <- c(abs(tmp$end[1]), abs(tmp$start[2]))
                point_y <- max(tmp$cy2)
            } else if (arc.location == "outer") {
                points_x <- c(abs(tmp$start[1]), abs(tmp$end[2]))
                point_y <- max(tmp$cy2) + annot_size
            }
            # code for default
            y_base <- point_y #- height_red
            y_elev <- height_factor_arc * height_box

            r <- (points_x[2] - points_x[1]) / 2
            a <- y_elev / r / r
            x <- seq(-r, r, length.out = 20)
            y <- -a * x * x
            # shift
            x <- x + r + points_x[1]
            y <- y + y_elev + y_base
            arc_h_list[["grpvar"]] <- grp_var
            arc_h_list[["arc_max_y"]] <- max(y)
            arc_h_list[["arc_center_x"]] <- points_x[1] + (points_x[2] - points_x[1]) / 2
            df_arc_y <- rbind(df_arc_y, data.frame(arc_h_list))
            grid.xspline(x, y,
                shape = -1, open = TRUE,
                default.units = "native",
                gp = gpar(lwd = lwd, lty = lty, col = color.arcs)
            )
        }
        df_arc_y <- df_arc_y %>% mutate("grp" = str_split_i(grpvar, "\\.", 1))
    }
    # text exists and asked to plot
    # depending on the annotation annotation.column2, set labels for cis

    if (nrow(box) > 0) {
        tmp_box2 <- as_tibble(tmp_box) %>% mutate(coordinate = (start + end) / 2)
        if (Gviz:::.dpOrDefault(GdObject, "annotation.column2") == "") {
            annotation.column1 <- Gviz:::.dpOrDefault(GdObject, "annotation.column1")
            annotation.column2 <- Gviz:::.dpOrDefault(GdObject, "annotation.column1")
        } else {
            annotation.column1 <- Gviz:::.dpOrDefault(GdObject, "annotation.column1")
            annotation.column2 <- Gviz:::.dpOrDefault(GdObject, "annotation.column2")
        }
        tmp_box2 <- tmp_box2[c("id", "textY", "coordinate", annotation.column1, annotation.column2)]
        colnames(tmp_box2) <- c("id", "y", "x", "txt1", "txt2")
        # apply offest
        tmp_box2$yOffset.A <- tmp_box2$y
        tmp_box2$yOffset.B <- tmp_box2$y
        tmp_box2$arm <- str_split_i(tmp_box$id, "\\.", 2)
        tmp.A <- tmp_box2[str_detect(tmp_box2$id, ".A"), ]
        tmp.B <- tmp_box2[str_detect(tmp_box2$id, ".B"), ]
        # for per-box
        yOffset.A <- tmp.A$y
        yOffset.B <- tmp.B$y
        if (label.cis.above == TRUE) {
            # cis labels
            if (nrow(df_arc_y) != 0) {
                cislabels <- left_join(tmp_box2, df_arc_y, by = c("id" = "grpvar"))
                translabels <- cislabels %>% dplyr::filter(is.na(arc_max_y))
                cislabels <- cislabels %>%
                    dplyr::filter(!is.na(arc_max_y)) %>%
                    distinct(grp, .keep_all = TRUE)
                grid.text(cislabels$txt2, cislabels$arc_center_x + labels.h.offset.cis, cislabels$arc_max_y +
                    annot_size + labels.v.offset.cis,
                rot = rotation.group, gp = font_gpar,
                default.units = "native", just = labels.align
                )
            } else {
                translabels <- tmp_box2
            }
            # trans labels
            if (nrow(translabels) != 0) {
                translabels$joined_lab <- str_c(translabels$txt1, "_", translabels$txt2)
                grid.text(translabels$joined_lab,
                    translabels$x + labels.h.offset.trans,
                    translabels$yOffset.A + labels.v.offset.trans,
                    rot = rotation.group, gp = font_gpar,
                    default.units = "native", just = labels.align
                )
            }
        } else {
            # plot per-box labels
            grid.text(tmp.A$txt1, tmp.A$x, yOffset.A + annot_size,
                rot = rotation.group, gp = font_gpar,
                default.units = "native", just = labels.align
            )

            grid.text(tmp.B$txt2, tmp.B$x, yOffset.B + annot_size,
                rot = rotation.group, gp = font_gpar,
                default.units = "native", just = labels.align
            )
        }
    }
    popViewport(1)
    # Brought here from AnnotationTrack # EgorS
    ## Finally we set up the image map
    im <- if (!is.null(box)) {
        coords <- as.matrix(box[, c("x1", "y1", "x2", "y2"), drop = FALSE])
        restCols <- setdiff(colnames(box), c("x1", "x2", "y1", "y2", "cx1", "cx2", "cy1", "cy2", "textX", "textY"))
        tags <- lapply(restCols, function(x) {
            tmp <- as.character(box[, x])
            names(tmp) <- rownames(coords)
            tmp
        })
        names(tags) <- restCols
        tags$title <- identifier(GdObject)
        Gviz:::ImageMap(coords = coords, tags = tags)
    } else {
        NULL
    }
    imageMap(GdObject) <- im
    return(invisible(GdObject))
})

#' The default display parameters for a  `DuplexTrack` object
#'
#' `DuplexTrack` inherits from `[Gviz::Annotaiontrack()]` and its Gviz parents.
#'  Most likely, user doesn't need all dioplay pars for the parents, so only
#'  parameters relevant to the `DuplexTrack` are returned by default.
#' @param class  `DuplexTrack` track object
#'   This function allows user to display the
#'   default display parameters for the \code{DuplexTrack} class.
#' @returns list of the default display parameters.
#' @importFrom Gviz availableDisplayPars
#' @export
#' @examples
#' library(InteractionSet)
#' anchor1 <- GRanges(
#'     seqnames = "chr1",
#'     ranges = IRanges(
#'         start = c(100, 600, 1100, 1600, 2100),
#'         end = c(200, 700, 1200, 1700, 2200)
#'     ),
#'     strand = "+"
#' )
#' anchor2 <- GRanges(
#'     seqnames = "chr1",
#'     ranges = IRanges(
#'         start = c(300, 800, 1300, 1800, 2300),
#'         end = c(400, 900, 1400, 1900, 2400)
#'     ),
#'     strand = "+"
#' )
#'
#' interactions <- GInteractions(anchor1, anchor2, mode = "strict")
#' gr_region <- range(anchor1, anchor2)
#' a <- DuplexTrack(interactions, gr_region = gr_region, stacking = "dense")
#' availableDisplayPars("DuplexTrack")
#' DuplexDiscovereR::availableDisplayPars(a)
availableDisplayPars <- function(class) {
    if (!is.character(class)) {
        class <- class(class)
    }
    # trick from Ginteraction Interaction to not mask Gviz generic class
    if (class == "DuplexTrack") {
        displaypars <- as(getClassDef(class)@prototype@dp, "list")
        return(displaypars)
    } else {
        return(Gviz::availableDisplayPars(class))
    }
}

#' Plots distributed boxes
#'
#' Non-exported from Gviz, contains logic for calcualting boxes alignment on plot
#'
#' @keywords internal
#' @returns pushes boxes to the viewport
.gv_plotboxes <- function(box, lwd, lty, alpha) {
    if ("transcript" %in% colnames(box)) {
        box <- .handleComposite(box, "box")
        if (nrow(box$box)) {
            grid.rect(box$box$cx2, box$box$cy1,
                width = box$box$cx2 -
                    box$box$cx1, height = box$box$cy2 - box$box$cy1,
                gp = gpar(
                    col = as.character(box$box$col), fill = as.character(box$box$fill),
                    lwd = lwd, lty = lty, alpha = alpha
                ), default.units = "native",
                just = c("right", "bottom")
            )
        }
        if (nrow(box$pols)) {
            grid.polygon(
                x = box$pols$x, y = box$pols$y, id = factor(box$pols$id),
                gp = gpar(
                    col = as.character(box$polpars$col),
                    fill = as.character(box$polpars$fill), lwd = lwd,
                    lty = lty, alpha = alpha
                ), default.units = "native"
            )
        }
    } else {
        grid.rect(box$cx2, box$cy1,
            width = box$cx2 - box$cx1,
            height = box$cy2 - box$cy1, gp = gpar(
                col = as.character(box$col),
                fill = as.character(box$fill), lwd = lwd, lty = lty,
                alpha = alpha
            ), default.units = "native", just = c(
                "right",
                "bottom"
            )
        )
    }
}

#' Set Gviz graphical parameters
#'
#' @details
#' Non-exported from Gviz. Uses the same procedure as in AnnotationTrack
#' to set defaults.
#'
#' @importFrom Gviz getScheme
#' @keywords internal
#' @returns set default values inside class upon calling withn class constructor
.gv_updatepars <- function(x, class) {
    current <- getPar(x, hideInternal = FALSE)
    defaults <- getPar(getClass(class)@prototype@dp, hideInternal = FALSE)
    sid <- getOption("Gviz.scheme")
    if (is.null(sid)) {
        scheme <- list()
    } else {
        # use scheme compatibility
        scheme <- Gviz::getScheme(sid)
    }
    # set this to calculate labels - we use groups as interactions pars
    scheme[["AnnotationTrack"]]["groupAnnotation"] <- "group"
    schemePars <- if (is.null(scheme) || is.null(scheme[[class]])) {
        list()
    } else {
        scheme[[class]]
    }
    if (!is.list(schemePars)) {
        stop(sprintf(
            "Corrupted parameter definition for class '%s' in scheme '%s'",
            class, sid
        ))
    }
    defaults[names(schemePars)] <- schemePars
    missing <- setdiff(names(defaults), names(current))
    if (is.null(getPar(x, ".__appliedScheme")) && length(schemePars)) {
        defaults[[".__appliedScheme"]] <- if (is.null(sid)) {
            NA
        } else {
            sid
        }
        defaults[names(schemePars)] <- schemePars
        missing <- c(union(missing, names(schemePars)), ".__appliedScheme")
    }
    x <- setPar(x, defaults[missing], interactive = FALSE)
    return(x)
}

# generate example duplexes to plot
anchor1 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(100, 600, 1100, 1600, 2100, 150, 400),
        end = c(200, 700, 1200, 1700, 2200, 250, 500)
    ),
    strand = "+"
)
anchor2 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(300, 800, 1300, 1800, 2300, 1500, 1700),
        end = c(400, 900, 1400, 1900, 2400, 1600, 1800)
    ),
    strand = "+"
)

interactions <- GInteractions(anchor1, anchor2, mode = "strict")

# define plotting range, assigns some annotaiton
gr_region <- range(anchor1, anchor2)
interactions$anno_A <- sample(LETTERS, length(interactions))
interactions$anno_B <- interactions$anno_A

# create track

# add interactions which are not fully in plot range: outside the range
# or on different chromosome()
# one left (A) interaction arm outside of the plot,
# other on different chromosome
new_anchor1 <- GRanges(
    seqnames = c("chr1", "chr2"),
    ranges = IRanges(
        start = c(10, 600),
        end = c(90, 700)
    ),
    strand = "+"
)
new_anchor2 <- GRanges(
    seqnames = c("chr1", "chr1"),
    ranges = IRanges(
        start = c(1500, 1000),
        end = c(1600, 1200)
    ),
    strand = "+"
)

new_interactions <- GInteractions(new_anchor1, new_anchor2)
new_interactions$anno_A <- c("A.out", "A.out_chr")
new_interactions$anno_B <- c("B.in", "B.in")
all_interactions <- c(interactions, new_interactions)



test_that("Creating visualization track workds", {
    suppressMessages({
        track_a <- DuplexTrack(interactions, gr_region = gr_region, stacking = "dense")
        track_b <- DuplexDiscovereR::DuplexTrack(all_interactions,
            gr_region = gr_region
        )
    })
    expect_equal(length(range(track_a)), 14,
        label =
            "number or ranges on plot is correct:1"
    )
    expect_equal(length(range(track_b)), 16,
        label =
            "number or ranges on plot is correct:2"
    )
})
# for trackB check in-out-range calculation
expacted_in_names <- c(
    "1.A", "1.B", "2.A", "2.B", "5.A", "5.B", "8.B", "9.A", "6.A",
    "6.B", "7.A", "7.B", "3.A", "3.B", "4.A", "4.B"
)
expected_in_labels <- c(
    "1", "1", "1", "1", "1", "1", "1", "0", "1", "1", "1",
    "1", "1", "1", "1", "1"
)
names(expected_in_labels) <- expacted_in_names

# for trackA check plot in 'dense' to indirectly test AnotationTrack option worked
expected_rowids <- rep("1", 14)
names(expected_rowids) <- paste0(rep(seq(1:7), each = 2), ".", rep(c("A", "B"), 7))


test_that("Drawing visualization track workds", {
    set.seed(123)
    suppressMessages({
        track_a <- DuplexTrack(interactions,
            gr_region = gr_region,
            stacking = "dense"
        )
        track_b <- DuplexDiscovereR::DuplexTrack(all_interactions,
            gr_region = gr_region
        )
        r_a <- plotTracks(track_a, stacking = "dense")
        r_b <- plotTracks(track_b, stacking = "squish")
    })
    in_range <- tags(imageMap(r_b$DuplexTrack))$in_range
    rowids <- tags(imageMap(r_a$DuplexTrack))$row_plot_id


    expect_equal(in_range, expected_in_labels)
    expect_equal(rowids, expected_rowids)
})

# Function to generate strings based on the 'length' column value
set_hg38_seqlengths <- function(gr) {
    # Define the sequence lengths for the hg38 assembly
    hg38_seqlengths <- c(
        chr1 = 248956422, chr2 = 242193529, chr3 = 198295559,
        chr4 = 190214555, chr5 = 181538259, chr6 = 170805979,
        chr7 = 159345973, chr8 = 145138636, chr9 = 138394717,
        chr10 = 133797422, chr11 = 135086622, chr12 = 133275309,
        chr13 = 114364328, chr14 = 107043718, chr15 = 101991189,
        chr16 = 90338345, chr17 = 83257441, chr18 = 80373285,
        chr19 = 58617616, chr20 = 64444167, chr21 = 46709983,
        chr22 = 50818468, chrX = 156040895, chrY = 57227415,
        chrM = 16569
    )

    # Intersect the provided GRanges object seqlevels with the hg38 seqlevels
    common_seqlevels <- intersect(seqlevels(gr), names(hg38_seqlengths))

    # Set the seqlengths for the GRanges object
    seqlengths(gr) <- hg38_seqlengths[common_seqlevels]

    return(gr)
}

set_hg19_seqlengths <- function(gr) {
    # Define the sequence lengths for the hg19 assembly
    hg19_seqlengths <- c(
        chr1 = 249250621, chr2 = 243199373, chr3 = 198022430,
        chr4 = 191154276, chr5 = 180915260, chr6 = 171115067,
        chr7 = 159138663, chr8 = 146364022, chr9 = 141213431,
        chr10 = 135534747, chr11 = 135006516, chr12 = 133851895,
        chr13 = 115169878, chr14 = 107349540, chr15 = 102531392,
        chr16 = 90354753, chr17 = 81195210, chr18 = 78077248,
        chr19 = 59128983, chr20 = 63025520, chr21 = 48129895,
        chr22 = 51304566, chrX = 155270560, chrY = 59373566,
        chrM = 16571
    )
    common_seqlevels <- intersect(seqlevels(gr), names(hg38_seqlengths))

    # Set the seqlengths for the GRanges object
    seqlengths(gr) <- hg38_seqlengths[common_seqlevels]

    return(gr)
}

#' Write reads to sam file
#' @description
#' Writes interactions to the sam file for visualization in extrnal browsers.
#' Takes input as GInteractions object containing reads or duplex groups.
#' @param gi_coords input Ginteraction object
#' @param genome DNAStringSet object with the reference genome
#' @param distance_chim_junction maximum distance between input
#' duplex groups/reads, which will be represented as the single-line in .sam file.
#' Junction will be output as N- gap. For the interactions with longer distances,
#' chimeric junction will be represented as MR:Z:i tag
#' @param read_name_column character field, pointing out to read names.
#' Read names are generated automatically if not provided.
#' @param id_column character name of the field containing integer duplex group ids.
#'  NA are replaced with zeros
#' @param genome character. Genome version. Required for the retrieval of sequence lengths
#' for sam file header-  SQ and SN tags.
#' For convenience, hg38 and hg19 chromosome lengths will be assigned
#'  automatically.
#' If the value is not in c('hg38','hg19'), seqlengths will be looked for be in
#' attribute in seqlengths() of regions(gi_coords)
#' @param file_out path to write output file
#' @param sample_name  name to use in RG SAM tag in header
#' @returns no object is returned
#' @export
#' @examples
#' # Load test data
#' data("RNADuplexesSampleData")
#' # if the input is read-based, it should have integer duplex group ids
#' # here, we have 2090 reads
#' length(RNADuplexSampleGI)
#' # among them 300 reads does not belong to any DG
#' # missing ids will be converted to 0
#' table(is.na(RNADuplexSampleGI$dg_id))
#' tmpf <- tempfile(".sam")
#' writeGiToSAMfile(
#'     gi_coords = RNADuplexSampleGI,
#'     id_column = "dg_id",
#'     file_out = tmpf,
#'     distance_chim_junction = 1e5,
#'     genome = "hg38"
#' )
writeGiToSAMfile <- function(gi_coords, file_out,
    distance_chim_junction = 10000,
    read_name_column = "readname",
    id_column = "dg_id",
    genome = "",
    sample_name = "noname_sample") {
    # read names from metadata or generate
    if (read_name_column %in% colnames(mcols(gi_coords))) {
        gi_coords$rname <- mcols(gi_coords)[, read_name_column]
    } else {
        gi_coords$rname <- paste0("read_", seq_len(length(gi_coords)))
    }
    # id column - fill with 0 if there is a NA DG ids i
    if (id_column %in% colnames(mcols(gi_coords))) {
        gi_coords$GRP_id <- mcols(gi_coords)[, id_column]
        gi_coords$GRP_id <- ifelse(is.na(gi_coords$GRP_id), 0, gi_coords$GRP_id)
    } else {
        stop('id column: "', id_column, '" was not found in the gi object metadata')
    }

    # Use known seqlen for assembly hg38 or hg19, or require it to be provided
    if (genome == "hg38") {
        seql <- seqlengths(set_hg38_seqlengths(regions(gi_coords)))
    } else {
        if ((genome == "hg19")) {
            seql <- seqlengths(set_hg19_seqlengths(regions(gi_coords)))
        } else {
            # take from the regions
            seql <- seqlengths(regions(gi_coords))
        }
    }

    if (any(is.na(seql))) {
        stop("To output in sam file, seqlens are required. Please assign seqlengths
        to the acnhor regions of input gi object. Human hg38 and hg19 can be
        provided as 'genome' argument ")
    }

    temp_gi_coords <- gi_coords
    mcols(temp_gi_coords)$seq_anchor1 <- "*"
    mcols(temp_gi_coords)$seq_anchor2 <- "*"

    mcols(temp_gi_coords)$width_anchor1 <- width(temp_gi_coords)$first
    mcols(temp_gi_coords)$width_anchor2 <- width(temp_gi_coords)$second
    mcols(temp_gi_coords)$artQUAL1 <- "*"
    mcols(temp_gi_coords)$artQUAL2 <- "*"
    # create art CIGAR of perfect matches
    mcols(temp_gi_coords)$artCIGAR1 <- paste(mcols(temp_gi_coords)$width_anchor1, "M", sep = "")
    mcols(temp_gi_coords)$artCIGAR2 <- paste(mcols(temp_gi_coords)$width_anchor2, "M", sep = "")

    # there are 2 types of joints.
    # 1. chimeric junctions, relatively close and on SAME chromosome
    # 2. interchromosomal
    # get separated by cis and trans

    grange1 <- temp_gi_coords@regions[temp_gi_coords@anchor1]
    grange2 <- temp_gi_coords@regions[temp_gi_coords@anchor2]
    partner1_dist <- distance(grange1, grange2)
    mcols(temp_gi_coords)$partner1_dist <- partner1_dist

    message("Processing single-line sam records")
    cis_inter <- temp_gi_coords[!is.na(partner1_dist)]
    cis_inter_large_dist <- cis_inter[mcols(cis_inter)$partner1_dist >= distance_chim_junction]
    cis_inter <- cis_inter[mcols(cis_inter)$partner1_dist < distance_chim_junction]
    if (length(cis_inter) > 0) {
        cis_inter_tib <- as_tibble(cis_inter)
        # important to have same gene to have them correct
        mcols(cis_inter)$artCIGAR <- paste(mcols(cis_inter)$width_anchor1, "M", mcols(cis_inter)$partner1_dist, "N", mcols(cis_inter)$width_anchor2, "M", sep = "")
        mcols(cis_inter)$artSEQ <- "*"
        mcols(cis_inter)$artQUAL <- "*"
        mcols(cis_inter)$col2 <- 0
        mcols(cis_inter)$col5 <- 0
        mcols(cis_inter)$col7 <- "*"
        mcols(cis_inter)$col8 <- 0
        mcols(cis_inter)$col9 <- 0
        mcols(cis_inter)$col11 <- "*"
        mcols(cis_inter)$tag <- paste(paste("RG:Z:", cis_inter$GRP_id, sep = ""),
            paste("DG:Z:", cis_inter$GRP_id, sep = ""),
            paste("MR:Z:", paste(cis_inter_tib$seqnames2, cis_inter_tib$start2, cis_inter_tib$end2, cis_inter_tib$strand2, sep = ","), sep = ""),
            sep = "\t"
        )

        cis_inter_tibble <- as_tibble(cis_inter) %>%
            dplyr::select(
                rname, col2, seqnames1, start1,
                col5, artCIGAR, col7, col8, col9, artSEQ, col11, tag
            )
    } else {
        cis_inter_tibble <- tibble()
    }

    message("Processing double-line sam records")
    trans_inter <- temp_gi_coords[is.na(partner1_dist)]
    trans_inter <- c(trans_inter, cis_inter_large_dist)

    trans_inter_tib <- as_tibble(trans_inter)

    mcols(trans_inter)$col1read1 <- paste(mcols(trans_inter)[, "rname"], "", sep = "")
    mcols(trans_inter)$col1read2 <- paste(mcols(trans_inter)[, "rname"], "", sep = "")
    mcols(trans_inter)$col2 <- 0
    mcols(trans_inter)$col5 <- 0
    mcols(trans_inter)$col7 <- "*"
    mcols(trans_inter)$col8 <- 0
    mcols(trans_inter)$col9 <- 0
    mcols(trans_inter)$col11 <- "*"
    mcols(trans_inter)$tag <- paste(paste("RG:Z:", trans_inter$GRP_id, sep = ""),
        paste("DG:Z:", trans_inter$GRP_id, sep = ""),
        paste("MR:Z:", paste(trans_inter_tib$seqnames2,
            trans_inter_tib$start2,
            trans_inter_tib$end2,
            trans_inter_tib$strand2,
            sep = ","
        ), sep = ""),
        sep = "\t"
    )

    #  pairs of gi_coords
    tibble_read1 <- as_tibble(trans_inter) %>% dplyr::select(
        col1read1, col2,
        seqnames1, start1, col5,
        artCIGAR1, col7, col8,
        col9, seq_anchor1,
        col11, tag
    )
    tibble_read2 <- as_tibble(trans_inter) %>% dplyr::select(
        col1read2, col2,
        seqnames2, start2, col5,
        artCIGAR2, col7, col8,
        col9, seq_anchor2,
        col11, tag
    )

    ####
    # starting creating SAM fata.frame
    # header
    header_chr_sizes <- c("@HD\tVN:1.4\tSO:coordinate")
    # sizes from genome fasta
    header_chr_sizes <- c(header_chr_sizes, paste("@SQ\tSN:", names(seql), "\tLN:", unname(seql), sep = ""))
    rg_tags <- paste0("@RG\tID:", sample_name, "\tSM:", sample_name, collapse = "")
    header_chr_sizes <- c(header_chr_sizes, rg_tags)
    pkg_ver <- utils::packageVersion("DuplexDiscovereR")

    header_chr_sizes <- c(
        header_chr_sizes,
        paste0("@PG\tID:DuplexDiscovereR\tPN:DuplexDiscovereR\tVN:",
            pkg_ver,
            collapse = ""
        )
    )

    tibble_read1$seq_anchor1 <- unlist(tibble_read1$seq_anchor1)
    tibble_read2$seq_anchor2 <- unlist(tibble_read2$seq_anchor2)

    writeLines(text = header_chr_sizes, con = file_out)
    write.table(cis_inter_tibble, file_out, sep = "\t", row.names = FALSE, col.names = FALSE, quote = FALSE, append = TRUE)
    write.table(tibble_read1, file_out, sep = "\t", row.names = FALSE, col.names = FALSE, quote = FALSE, append = TRUE)
    write.table(tibble_read2, file_out, sep = "\t", row.names = FALSE, col.names = FALSE, quote = FALSE, append = TRUE)

    message("SAM is written into: ", file_out)
}

n_dgs_expected <- 1830
chrlen_ln <- 50818468
test_that("write to sam file works", {
    suppressMessages({
        data("RNADuplexesSampleData")
        length(RNADuplexSampleGI)
        table(is.na(RNADuplexSampleGI$dg_id))
        tmpf <- tempfile(".sam")
        writeGiToSAMfile(
            gi_coords = RNADuplexSampleGI,
            id_column = "dg_id",
            file_out = tmpf,
            distance_chim_junction = 1e5,
            genome = "hg38"
        )
        sam_body <- read.delim(tmpf, header = FALSE, comment.char = "@")
        dgs_assigned <- as.integer(stringr::str_split_i(sam_body$V13, "DG:Z:", 2))
        dgs_count_tab <- table(dgs_assigned)


        sam_header <- read.delim(tmpf, header = FALSE, nrows = 4)
        chrlen_real <- gsub(".*LN:(\\d+).*", "\\1", paste0(sam_header$V3, collapse = ""))
        chrlen_real <- as.integer(chrlen_real)
    })
    expect_equal(dgs_count_tab[["0"]], n_dgs_expected, label = "NA dgs assigned to zero")
    expect_equal(chrlen_real, chrlen_ln, label = "Chrlen is written")
})

---
title: "DuplexDiscovereR tutorial"
author:
  - name: "Egor Semenchenko"
    affiliation: Max Delbrück Center for molecular medicine; Berlin Institute for Medical Systems Biology; Freie Universität Berlin  
    email: egorsbs01@gmail.com
package: DuplexDiscovereR
abstract: |
    DuplexDiscovereR implements methods for analysing data from RNA cross-linking 
    and proximity ligation protocols such as SPLASH, PARIS, LIGR-seq 
    and other methods which yeild information on RNA-RNA interactions as 
    chimeric read fragments after high-throughput sequencing. 
bibliography: '`r system.file("references.bib", package = "DuplexDiscovereR")`'
output:
  BiocStyle::html_document:
    toc: true
    toc_depth: 5
    toc_float: true
    number_sections: true
vignette: |
  %\VignetteIndexEntry{DuplexDiscovereR tutorial}
  %\VignetteEncoding{UTF-8} 
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---


```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown(css.files = c("custom.css"))
```

```{r setup, include=FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##",
    crop = FALSE
)
```

# Installation

`DuplexDiscovereR` can be installed from Bioconductor:

```{r install_chunk, eval=FALSE}
if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")

BiocManager::install("DuplexDiscovereR")

library(DuplexDiscovereR)
?DuplexDiscovereR
```

or from Github page:

```{r install_chunk2, eval=FALSE}
if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")

library(devtools)
devtools::install_github("Egors01/DuplexDiscovereR")
```

## Installing RNAduplex

For calculating hybridization energies `DuplexDiscovereR` uses
[RNAduplex](https://www.tbi.univie.ac.at/RNA/RNAduplex.1.html) program
from the [ViennaRNA](https://www.tbi.univie.ac.at/RNA/) software suite.

Although this step is optional and is not required for using most of the
package methods, we strongly recommend installing ViennaRNA from its
[web-page](https://www.tbi.univie.ac.at/RNA/), as predicting RNA hybrids
is one of the central aims of the analysis of RNA-RNA interaction data.
Please note that ViennaRNA is distributed under its own licence.

# Introduction

RNA cross-linking and proximity ligation methods as SPLASH, PARIS,
LIGR-seq, RIC-seq and others provide infromation on the RNA-RNA
interactions on a transcriptome-wide scale. These experiments generate
high-throughput sequencing data containing a fraction of chimeric reads,
where each chimeric part corresponds to the base-paired stretches of RNA
(RNA duplexes)

`DuplexDiscovereR` is designed for the bioinformatic analysis of RNA data.
It employs a workflow that allows users to identify RNA duplexes and
and provides several options for filtering and ranking the results.

Starting from the set of aligned chimeric reads, it implements
Classification and filtering of non-RNA duplex alignments and their
into duplex groups (DGs). Identified DGs are then annotated
with transcripts or other genomic features. P-values are calculated to test
the hypothesis that DGs are generated by random ligation. Finally, RNA
duplex base pairing and hybridization energies are predicted.

The optimal procedures for identifying RNA duplexes may vary between
methods, as RNA-RNA interaction probing protocols are known to have
biases for certain types of RNA and differ in experimental steps.
`DuplexDiscovereR` allows user to build customized analysis by utilizing
its methods for efficient chimeric read clustering, classification and
convenience functions for flexible data filtering, annotation and
visualization.

RNA duplex data is known to be sparse, with little overlap between the results of
between the results of different probing protocols. To facilitate
comparisons between different experiments and to allow straightforward
cross-checking between replicates, `DuplexDiscovereR` includes
functionality to intersect multiple RNA interaction datasets.

`DuplexDiscovereR` relies on the `GInteractions` container from
`r Biocpkg("InteractionSet")` package for the storing of the RNA-RNA
interaction data and uses tidyverse `tibble` and `dplyr` for internal
the `r Biocpkg("Gviz")` - based `DuplexTrack` track and
supports the exportof the results to the .sam file.

# Quick start

##  Analysis steps

Key steps of RNA duplex data analysis with `DuplexDiscovereR` can be run
through the single function call. It is expected that user already
aligned the raw sequencing reads which produced some chimeric or
split-read alignments. The input can be provided in several formats:

-   Tabular file with the split-read alignments
    -   *Chimeric.out.junction* file generated by STAR
    -   *.bedpe* format file (defined in
        [bedtools](https://bedtools.readthedocs.io/en/latest/content/general-usage.html#bedpe-format))
-   `GInteractions` object with chimeric alignments. 

Full `DuplexDiscovereR` workflow consists of the following steps:

-   Pre-processing of the input. This step calls the classification of the reads 
    into following classes: 
    -   2arm reads (only one chimeric junction per aligned read)
    -   multi-split (two or more one chimeric junctions per aligned
        read)
    -   multi-mapped reads
-   Comparing the 2arm reads mapped reads with the splice
    junction annotation. This filters out alignments across the exon-exon 
    junctions which typically contaminate the 'true' chimeric alignments 
    originated from the RNA duplexes
-   Deduplication of the identical read alignments
-   Clustering of reads into duplex groups (DGs)

*Optionally*,

-   Annotation of the DGs with transcript or other features
-   Prediction of the base-pairing hybridization energies
-   Computing p-values from testing the probability of random ligation

## Input arguments

Required inputs for minimal run are the following:

-   `data` Chimeric read alignments or coordinates. Object of any
    table type, convertible to `tibble` or `GInteractions`
-   `junctions_gr` `GRanges` object with splice junction
    coordinates.\
-   Library type and type of the read table, if input is not
    `GInteractions`.
    
Optional inputs for executing the full workflow:

-   `anno_gr` a `GRanges` object with genes or transcripts
    annotation object
-   `df_counts` a 2-column table with the `id` in the 1st column and
    raw read count in the 2nd. `id` should correspond to the entries
    in `anno_gr`. Providing this argument triggers p-value
    calculation.
-   `fafile`  path to *.fasta* file with the genome. Providing this
    argument triggers prediction of RNA hybrids.

## Loading example data

Once installed, we can load the example data. Example dataset is based
on the RNA duplex probing of SPLASH ES-cells [Aw. et.al
2016](https://pubmed.ncbi.nlm.nih.gov/27184079/) which was aligned with
STAR [Dobin et.al,
2013](https://academic.oup.com/bioinformatics/article-abstract/29/1/15/272537)
and subset to the chromosome 22.

```{r dataload,echo=T,eval=T,message=FALSE}
library(DuplexDiscovereR)
data(RNADuplexesSampleData)
```

Details on alignment of the reads of example dataset and fetching of the
annotation data are described in the document which can be found at

```{r dataload2 }
system.file("extdata", "scripts", "DD_data_generation.R", package = "DuplexDiscovereR")
```

## Running analysis

```{r run_workflow, echo=T,eval=T,message=FALSE}
genome_fasta <- NULL
result <- DuplexDiscovereR::runDuplexDiscoverer(
    data = RNADuplexesRawChimSTAR,
    junctions_gr = SampleSpliceJncGR,
    anno_gr = SampleGeneAnnoGR,
    df_counts = RNADuplexesGeneCounts,
    sample_name = "example_run",
    lib_type = "SE",
    table_type = "STAR",
    fafile = genome_fasta,
)
```

The `result` is a `DuplexDiscovererResults` object which contains several outputs of different
dimensions.

```{r observe_results1,eval=TRUE}
result
```

The primary output are detected duplex groups. 
This is a `GInteractions`, where each entry represents boundaries 
of the detected RNA duplex. Metadata fields as `n_reads` , `p_val` , `score` and others 
can be used for ranking and sub-setting results for downstream analysis. 

```{r observe_results2,eval=TRUE}
gi_clusters <- dd_get_duplex_groups(result)
head(gi_clusters, 2)
```

Individual read-level data can be accessed by running:

```{r observe_results3,eval=TRUE}
gi_reads <- dd_get_chimeric_reads(result)
head(gi_reads, 2)
```
Each entry in `gi_reads` is a single read. They are tagged by DG with the 
`dg_id`, but not collapsed into groups. This layout is used for
visualization and may be useful for a more detailed review of the structure of 
the individual DGs.

The dimensions of `gi_reads` do not correspond 1:1 to the size of the
to the size of the input, since in general not every element in the input data 
can be represented as a 2-arm split alignment.

To track each input alignment one can query the the read-level classification results, 
which have the same dimension with the input. 

```{r observe_results4a,eval=TRUE}
df_reads <- dd_get_reads_classes(result)
print(dim(df_reads))
print(dim(RNADuplexesRawChimSTAR))
```
`df_reads` has fields such as `read_type`, `map_type` and `junction_type` 
indicating the results of chimeric read classification and
filtering. 

This layout is useful for assessing the statistics and highlighting potential 
problems and optimization points, i.e. how many reads from the input were 
classified as true chimeric and how many of them were assigned to duplex groups. 

```{r observe_results4b,eval=TRUE}
table(df_reads$read_type)
```

Among the 2-arm alignments from `gi_reads`, only unambigously aligned reads 
without self-ovelaps that pass splice-junctions and minimum junction length
filters are subjected to clustering into DGs. To see the determined types of 
2-arm alignments, we can check the `junction_type` metadata field

```{r observe_results5,eval=TRUE}
table(gi_reads$junction_type)
```

## Writing output

The resulting `GInteractions` with duplex groups are ready for
integration into transcriptomic analysis using other Bioconductor
packages.

There are several options available for outputting the results for other
uses.

### Output to table

One can convert the object with duplex groups to dataframe-like object and
write it to file

```{r write_table, eval=TRUE}
clusters_dt <- makeDfFromGi(gi_clusters)
write.table(clusters_dt, file = tempfile(pattern = "dgs_out", fileext = ".tab"))
```

### Output to .sam file

Saving results to .sam file can be a useful tool for downstream
visualization with interactive viewers as IGV

```{r write_sam, eval=TRUE, message = FALSE}
writeGiToSAMfile(gi_clusters, file_out = tempfile(pattern = "dgs_out", fileext = ".sam"), genome = "hg38")
```

## Visualization

Collapsed DGs can be visualized with Gviz -based DuplexTrack, 
allowing overlays of RNA duplexes with the genes of other features

```{r, vis_1, eval=TRUE,message=FALSE}
library(Gviz)
# define plotting region
plotrange <- GRanges(
    seqnames = "chr22",
    ranges = IRanges(
        start = c(37877619),
        end = c(37878053)
    ),
    strand = "+"
)
# make genes track
anno_track <- Gviz::GeneRegionTrack(SampleGeneAnnoGR,
    name = "chr22 Genes",
    range = plotrange
)
# construct DuplexTrack
duplex_track <- DuplexTrack(
    gi = gi_clusters,
    gr_region = plotrange,
    labels.fontsize = 12,
    arcConstrain = 4,
    annotation.column1 = "gene_name.A",
    annotation.column2 = "gene_name.B"
)
plotTracks(list(anno_track, duplex_track),
    sizes = c(1, 3),
    from = start(plotrange) - 100,
    to = end(plotrange) + 100
)
```

## Calcualting hybridization energies

In the example above we have omitted the base-pairing prediction and the
hybridization energies by not passing the path to file with genome as the 
`fafile` argument. However, the prediction of RNA hybrids is
central to the analysis of RNA-RNA interaction data.

To compute the structure formed by two stretches of RNA
`DuplexDiscoverer` uses *RNAduplex* algorithm from ViennaRNA. Note
that ViennaRNA is distributed under its own licence. Please refer to
[ViennaRNA's license](https://viennarna.readthedocs.io/en/latest/license.html)
for details.

If the *RNAduplex* is installed, all steps of the
analysis can be re-run by calling `runDuplexDiscoverer` with the
option `fafile = <path to genome fasta>`. Alternatively, one can predict RNA
hybrids by calling separate function on already existing `GInteractions` object:

```         
sequence <- paste0(
"AGCUAGCGAUAGCUAGCAUCGUAGCAUCGAUCGUAAGCUAGCUAGCUAGCAUCGAUCGUAGCUAGCAUCGAU",
"CGUAGCAUCGUAGCUAGCUAGCUAUGCGAUU")

# Save the sequence to a temp fasta file
fasta_file = tempfile(fileext = '.fa')
chrom <- "test_chrA"
writeLines(c(">test_chrA", sequence), con = fasta_file)

# Create the GInteraction object
# Define start and end positions for the base-pairing regions
regions <- data.frame(
    start1 = c(1, 11, 21, 31, 41),
    end1 = c(10, 20, 30, 40, 50),
    start2 = c(91, 81, 71, 61, 51),
    end2 = c(100, 90, 80, 70, 60)
)
# GRanges objects for the anchors
anchor1 <- GRanges(seqnames = chrom, ranges = IRanges(start = regions$start1, end = regions$end1))
anchor2 <- GRanges(seqnames = chrom, ranges = IRanges(start = regions$start2, end = regions$end2))
interaction <- GInteractions(anchor1, anchor2)
# predict hybrids
getRNAHybrids(interaction,fasta_file)
```

## Comparing multiple samples or replicates

Due to the sparse nature of RNA duplex data, it is desirable to be able
to compare replicates of the same experiment or to compare different
data sets.

`DuplexDiscovereR` provides a method for finding intersections between
multiple samples. Comparisons between ranges can result in one-to-many
relations, i.e. when a region (or pair a of regions) of one sample contains
multiple regions of other samples. `DuplexDiscovereR` addresses this
problem by using the following procedure: first it computes a non-redundant
set of interactions that stacks and collapses interactions from all
input samples. This resulting super-set is then compared in a pairwise
manner with each  entry from each individual sample.

To demonstrate how multiple samples can be compared, we will generate
three pseudo-replicates from the example data which we clustered above.

First, we will select the clustered reads and divide them into three groups

```{r comp1, eval=TRUE}
clust_reads <- gi_reads
clust_reads <- clust_reads[!is.na(clust_reads$dg_id)]

# Create separate pseudo-sample objects for each group
set.seed(123)
group_indices <- sample(rep(1:3, length.out = length(clust_reads)))

group1 <- clust_reads[group_indices == 1]
group2 <- clust_reads[group_indices == 2]
group3 <- clust_reads[group_indices == 3]
```

Since individual reads have already been assigned to duplex groups, 
we can collapse them, ensuring that we have some overlap between 
reads that belong to the same 'true' DGs, but are spread across three
artificial sets.

```{r comp2, eval=TRUE}
group1 <- collapse_duplex_groups(group1)
group2 <- collapse_duplex_groups(group2)
group3 <- collapse_duplex_groups(group3)
```

```{r comp3, eval=TRUE, message=FALSE}
a <- list("sample1" = group1, "sample2" = group2, "sample3" = group3)
res_comp <- compareMultipleInteractions(a)
names(res_comp)
```

Result contains slot for interaction superset and overlap information
information. We can visualize the intersections of the samples using the upset plot

```{r comp4, eval=TRUE}
# first, check if UpSetR is installed
if (!requireNamespace("UpSetR", quietly = TRUE)) {
    stop("Install 'UpSetR' to use this function.")
}
library(UpSetR)
upset(res_comp$dt_upset, text.scale = 1.5)
```

# Building customized RNA duplex data analysis

One can customize the analysis by performing classification, clustering and 
subsequent annotation procedures step by step, adapting the procedures to
the RNA-RNA interaction probing protocol or upstream/downstream analysis.

In this section, we show how users can use the core methods in the 
package to achieve more flexibility in this task. 

## Preprocessing 

Most of the methods in `DuplexDiscovereR` work with
the RNA-RNA interaction data stored in `GInteraction` object. There are
several metadata fields which are not strictly required by the package,
but utilized in procedures run classification: `map_type`, `read_id` and 
`n_reads`,`score` for clustering.

The easiest way to run `DuplexDiscovereR` step-by-step is to start with a
pre-processing routine. It is fairly flexible in input type: it can be either
`GInteractions` ,  *Chimeric.out.Junction* file from *STAR* or generic 
*.bedpe* file with pairs of regions. 

```{r,  preproc ex1, message=FALSE}
data("RNADuplexesSampleData")

new_star <- runDuplexDiscoPreproc(
    data = RNADuplexesRawChimSTAR,
    table_type = "STAR",
    library_type = "SE",
)
```

```{r,  preproc ex2, message=FALSE}
new_bedpe <- runDuplexDiscoPreproc(
    data = RNADuplexesRawBed,
    table_type = "bedpe",
    library_type = "SE", return_gi = TRUE
)
```

By default, to make downstream read filtering slightly more convenient,
`tibble` is returned after the pre-processing. In case of the
`GInteractions` input, input reads already are 2-arm split by design, so
we can return `GInteractions` .

```{r,  preproc ex3, message=FALSE}
# keep only readname in metadata
mcols(RNADuplexSampleGI) <- mcols(RNADuplexSampleGI)["readname"]
new_gi <- runDuplexDiscoPreproc(data = RNADuplexSampleGI, return_gi = TRUE)
head(new_gi, 1)
```

## Classification and filtering

The basic classification routines implemented in the package are
comparing the chimeric junctions to the known splice junctions and
determining the types of the junctions. Both can be called by a 
separate method that adds a corresponding metadata field to the input:

```{r ex_classif1}
gi <- getChimericJunctionTypes(new_gi)
gi <- getSpliceJunctionChimeras(gi, sj_gr = SampleSpliceJncGR)
```

One can inspect detected junction types

```{r ex_classif2}
table(gi$junction_type)
```

and amount of reads which are not likely to come from true RNA
duplexes, because their chimeric junctions are too similar to normal
exon-exon junction

```{r ex_cassif3}
table(gi$splicejnc)
```

For determining the duplex groups, we will leave only chimeric junctions
in a read which do not self-overlap:

```{r }
keep <- which((gi$junction_type == "2arm") & (gi$splicejnc == 0))
gi <- gi[keep]
```

## Clustering reads into duplex groups

The most basic way to find the duplex groups is to simply call

```{r clust1, message=FALSE}
gi <- clusterDuplexGroups(gi)
```

This procedure will:

1.  Find overlaps between all chimeric read pairs.
2.  Represent reads as a directed graph, where edges are weighted with
    the amount of overlap between the reads.
3.  Call a community search algorithm on the graph which finds the read
    clusters.
4.  Add the read cluster IDs as the `dg_id` field to the input object.
    `NA` values are used for reads which are not a part of any read duplex
    group (DG)

```{r ex_clust2}
table(is.na(gi$dg_id))
```

To collapse reads to the DGs with the boundaries, containing
every read of DG, one can call

```{r ex_clist3}
gi_dgs <- collapse_duplex_groups(gi)
head(gi_dgs, 1)
# number of DGs
length(gi_dgs)
```
The above procedure can be applied to any subset of data. 
If the user is interested in the RNA structure in a particular region, they
can select all reads overlapping with this region and repeat the clustering several times. 
By adjusting the parameters (minimum overlap, maximum allowed distance between 
alignment arms, coordinates of coordinates) one can obtain more refined or larger
duplex groups for improving subsequent prediction of RNA secondary structure.

## Customization of the read clustering

### Collapsing identical reads

For large datasets, it is possible to identify and collapse identical
reads – those that map to the same coordinates with the same score. In
overlap-based clustering, these reads would belong to the same duplex
groups. Collapsing them into a single entry reduces the number of nodes
and vertices in the clustering graph, resulting in better time and
memory performance.

```{r , dupl1 }
res_collapse <- collapseIdenticalReads(gi)
# returns new object
gi_unique <- res_collapse$gi_collapsed
```

This procedure adds `duplex_id` column as new id and removes `read_id`,
because single entry in a new object can correspond to multiple reads.
`n_reads` records the number of reads in this temporary duplex group.

```{r dupl2 }
head(res_collapse$stats_df, 1)
```

Note that `collapse_duplex_groups()` method has options to handle
temporary duplex groups, so that merging identical reads before clustering
would not lose information on the read support for duplex group.

```{r dupl3, message=FALSE}
# cluster  gi_unqiue
gi_unique_dg <- collapse_duplex_groups(clusterDuplexGroups(gi_unique))
# check if the get the same amount of reads as in basic clustering
sum(gi_unique_dg$n_reads) == sum(gi_dgs$n_reads)
```

### Custom weights for clustering

There is a possibility for a user to re-define the rules by which
overlaps between reads are weighted before community search.

This might be a customization related to protocols or the size of the
dataset. Possible scenarios are weighting the clustering graph vertices
with hybridization energies calculated directly on the reads instead of
duplex groups, or employing other heuristic assumptions on how RNA
duplexes are translated into chimeric read fragments. e.g in RIL-seq, the
second chimeric arm is more likely to be a target RNA rather than sRNA.

To demonstrate such a modification, we will first compute the overlap graph manually,
using the following procedure, second, modify the graph weights, and third, call the 
clustering. 

First, we find pair overlaps in the input `GInteractions` object

```{r cl1}
graph_df <- computeGISelfOverlaps(gi_unique)
head(graph_df)
```

The size of this dataframe is determined by the presence of pairwise overlap
between both arms of read alignments. The first two columns represent
the element index or `duplex_id` of the corresponding reads. It can be
filtered or supplemented with other relevant metrics of the pairwise
relationship between the reads.

Then we change the rule for how we handle overlap between reads.
In this example we will manually change the span-to-overlap ratio per
pair or overlapping alignment arms.  Read pairs that overlap for less than half 
of their length will not be treated as reliable to be considered part of 
the same DG. 

```{r cl2a}
graph_df <- graph_df[graph_df$ratio.A > 0.5 & graph_df$ratio.B > 0.5, ]
```
We will use the sum of the overlap ratios as the new weights and re-scale them to {0,1}.
```{r cl2b}
rescale_vec <- function(x) {
    return((x - min(x)) / (max(x) - min(x)))
}

graph_df$weight_new <- graph_df$ratio.A + graph_df$ratio.B
graph_df$weight_new <- rescale_vec(graph_df$weight_new)
head(graph_df, 2)
```

This dataframe then can be passed to the clustering function:

```{r cl3, message=FALSE}
gi_clust_adj <- clusterDuplexGroups(gi_unique, graphdf = graph_df, weight_column = "weight_new")
gi_clust_adj_dgs <- collapse_duplex_groups(gi_clust_adj)
```

We get smaller number of the DGs, as some of them are
not assembled after restricting the minimum required overlap

```{r cl4}
length(gi_clust_adj)
```

# Session information

```{r session-info,cache = F,echo=T,message=T,warning=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{RNADuplexSampleClustReads}
\alias{RNADuplexSampleClustReads}
\title{RNA duplex reads of SPLASH, clustered and assigned to duplex groups}
\format{
An object of class \code{StrictGInteractions} of length 2090.
}
\source{
\href{https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943}{SequenceReadArcive}
Reads were aligned with STAR and duplex groups were identified
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details on the data generation proccedure.
}
\usage{
data(RNADuplexesSampleData)
}
\value{
\code{GInteractions} with
\itemize{
\item \code{n_reads_dg } : number of reads in the duplex group (DG)
\item \code{duplex_id  } : temporary id for RNA duplexes which could be found before
clustering (duplicated or shifted by couple of nt )
\item \code{dg_id} :id of the duplex group
\item \code{score} : median alignment score in duplex group
\item other columns inherited from the STAR Chimeric.out.Junction
}
}
\description{
\code{GInteractions} read-level object containing processed reads,annotated with duplex group
ids, read types gene names and p-values
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_preproc.R
\name{trimAroundJunction}
\alias{trimAroundJunction}
\title{Extract regions around chimeric junction}
\usage{
trimAroundJunction(dt, extract_len = 30)
}
\arguments{
\item{dt}{table with the}

\item{extract_len}{the length of the seqeunce to trim}
}
\value{
dataframe with the trimmed alignments
}
\description{
Trim alignements to contain only 'extract len' nucleotides adajcent
to the chimeric junction
}
\details{
In case of the long alignemtns, it may be necessary trim chimeric alignments
to identify RNA duplex. If 'extract_len' is longer than the read
alignemnt length, then no trimming is performed
}
\examples{
data("RNADuplexesSampleData")
dt_preproc <- runDuplexDiscoPreproc(RNADuplexesRawChimSTAR,
    table_type = "STAR", library_type = "SE"
)
trimAroundJunction(dt_preproc, 40)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexTrack.R
\name{drawGD,DuplexTrack-method}
\alias{drawGD,DuplexTrack-method}
\title{Draw method for DuplexTrack}
\usage{
\S4method{drawGD}{DuplexTrack}(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...)
}
\value{
pushes boxes, arcs and labels to viewport
}
\description{
\code{Gviz::AnnotationTrack} stacking algorithm is used to calculate vertical
distribution of boxes for the interactions.
Boxes coordinates are later imported for placing labels and arcs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_preproc.R
\name{preproc_chim_junction_out_se}
\alias{preproc_chim_junction_out_se}
\title{Processing of of the STAR SE Chimeric.junction.out}
\usage{
preproc_chim_junction_out_se(dt, keep_all_columns = FALSE)
}
\arguments{
\item{dt}{Chimeric.out.junction with the correct column names}

\item{keep_all_columns}{\itemize{
\item TRUE or FALSE. Keep CIGAR strings and junction coordinate columns
}}
}
\value{
tibble with annotated reads
}
\description{
Calculates alignment coordinates and returns reads with categories
}
\details{
#' \describe{
\item{multimap}{ multi-mapped read}
\item{multigap}{ more than one junction (more than two 'N' in CIGAR string)}
\item{bad junction}{ Artifacts. I.e alignments for both arms are continious, but with 'backward' chimeric junction was wrongly put}
}
}
\seealso{
\code{\link[=col_check_rename]{col_check_rename()}}
}
\keyword{interal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_annotation.R
\name{.addGeneCounts}
\alias{.addGeneCounts}
\title{Helper function to add count data to metadata of \code{GInteractions}}
\usage{
.addGeneCounts(gi, df_counts, id_col = "gene_id")
}
\arguments{
\item{gi}{\code{GInteractions}}

\item{df_counts}{dataframe with read counts}

\item{id_col}{key to use in merge}
}
\value{
\code{GInteractions} with added counts
}
\description{
Merges the count dataframe and interactions metadata by \code{id_col}
If key is not found, in metadata throws error
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{get_char_count_cigar}
\alias{get_char_count_cigar}
\title{Count the length of the key type in CIGAR string}
\usage{
get_char_count_cigar(strings, s)
}
\arguments{
\item{strings}{CIGAR string vector}

\item{s}{CIGAR operands}
}
\value{
vector with length values
}
\description{
Takes CIGAR operands i.e M,N,S and sums the associated blocks length
It is vectorized. i.e supports vector with CIGAR strings
}
\examples{
# From a vector
get_char_count_cigar(c("4S18M22S", "25S26M"), "S")
get_char_count_cigar(c("18M22S", "20M20S"), "M")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_classification.R
\name{.getStartEndOvl}
\alias{.getStartEndOvl}
\title{Helper func for calculating one-side overlap of SJ and junctions of gi}
\usage{
.getStartEndOvl(gi, gr_chim_c, gr_sj_c, tol = 3)
}
\arguments{
\item{gi}{GInteractions object}

\item{gr_chim_c}{GRanges object with chimeric junctions of gi}

\item{gr_sj_c}{GRanges with chimeric junctions}

\item{tol}{overlap tolerance}
}
\value{
Granges object with the left (A) region
}
\description{
Helper func for calculating one-side overlap of SJ and junctions of gi
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexTrack.R
\name{availableDisplayPars}
\alias{availableDisplayPars}
\title{The default display parameters for a  \code{DuplexTrack} object}
\usage{
availableDisplayPars(class)
}
\arguments{
\item{class}{\code{DuplexTrack} track object
This function allows user to display the
default display parameters for the \code{DuplexTrack} class.}
}
\value{
list of the default display parameters.
}
\description{
\code{DuplexTrack} inherits from \verb{[Gviz::Annotaiontrack()]} and its Gviz parents.
Most likely, user doesn't need all dioplay pars for the parents, so only
parameters relevant to the \code{DuplexTrack} are returned by default.
}
\examples{
library(InteractionSet)
anchor1 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(100, 600, 1100, 1600, 2100),
        end = c(200, 700, 1200, 1700, 2200)
    ),
    strand = "+"
)
anchor2 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(300, 800, 1300, 1800, 2300),
        end = c(400, 900, 1400, 1900, 2400)
    ),
    strand = "+"
)

interactions <- GInteractions(anchor1, anchor2, mode = "strict")
gr_region <- range(anchor1, anchor2)
a <- DuplexTrack(interactions, gr_region = gr_region, stacking = "dense")
availableDisplayPars("DuplexTrack")
DuplexDiscovereR::availableDisplayPars(a)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexTrack.R
\docType{class}
\name{DuplexTrack}
\alias{DuplexTrack}
\title{class for the visualization of RNA duplexes}
\arguments{
\item{gi}{An \code{GInteractions} object}

\item{gr_region}{\code{GRanges} region for plotting}

\item{from}{Integer start coordinate of subset region. Used if \code{gr_region} is not provided}

\item{to}{Integer end coordinate of subset region. Used if \code{gr_region} is not provided}

\item{chromosome}{Chromosome of subset region. Used if \code{gr_region} is not provided}

\item{strand}{Used if \code{gr_region} is not provided}

\item{fill.column}{used for fill. Default is "" (empty) and triggers IGV color pallete.
\strong{Display parameters}
\describe{
\item{arcs.color}{Character. Color of the arcs. Default is "black".}
\item{arc.location}{Character in c('inner','outer','midpoint'). Location of the arcs in X axis relative to range.
Default is "inner"}
\item{labels.v.offset.base}{Numeric. Base vertical offset for the labels. Default is 0.2. Other offesets are added to it.
}
\item{labels.v.offset.trans}{Numeric. Vertical offset for trans labels. Applied when one part of the duplex is outside of the plot.
Recommended ranges are in -0.5 to 0.5 Default is 0.0.   }
\item{labels.h.offset.trans}{Numeric. Horizontal offset for trans labels.
Applied when one part of the duplex is outside of the plot
Value is in nucleotide units. Default is 0.0. }
\item{labels.v.offset.cis}{Numeric. Vertical offset for cis labels.  Recommended ranges are in -0.5 to 0.5 Default is 0.0. Default is 0.0. }
\item{labels.h.offset.cis}{Numeric. Horizontal offset for cis labels.  Value is in nucleotide units. Default is 0.0.}
\item{labels.fontsize}{Numeric. Font size of the labels. Default is 18.}
\item{label.cis.above}{Logical. Whether the cis labels should be above. When set to FALSE, labels are plot for each box separately.
Default is TRUE}
\item{annotation.column1}{Character. First annotation column to use for labels. Default is "group" and generated internally.}
\item{annotation.column2}{Character. Second annotation column to use for labels. Default is "" (empty).}
\item{fill.column}{Character. Column used for fill. Default is "" (empty) and triggers IGV color pallete.}
\item{labels.color}{Character. Color of the labels. Default is 'black'.}
\item{labels.align}{Character. Alignment of the labels. Default is 'center'.
Possible values are in c('left','right','center)}
\item{arcConstrain}{Numeric. Minimum gap distance between arms of the
interaction to draw arcs}
}}
}
\description{
Inherits the \code{Gviz::AnnotationTrack}, plots interaction ranges as boxes.
Arguments from \code{Gviz::AnnotationTrack}, as \code{stacking} which set boxes
layout are accepted. Parent aesthetics for labels are overwritten with Display
parameters of this class.
Accepts \code{GInteractions} object to plot and \code{GRanges} to define plot region

Duplexes which can be displayed on the plot range are connected with arcs.
Duplexes which are partially outside of the range are displayed without arcs.
Labeles and appearance can be controlled with display parameters
}
\examples{
library(InteractionSet)
library(Gviz)
# generate input
anchor1 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(100, 600, 1100, 1600, 2100, 150, 400),
        end = c(200, 700, 1200, 1700, 2200, 250, 500)
    ),
    strand = "+"
)
anchor2 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(300, 800, 1300, 1800, 2300, 1500, 1700),
        end = c(400, 900, 1400, 1900, 2400, 1600, 1800)
    ),
    strand = "+"
)

interactions <- GInteractions(anchor1, anchor2, mode = "strict")
# define plotting range
gr_region <- range(anchor1, anchor2)
interactions$anno_A <- sample(LETTERS, length(interactions))
interactions$anno_B <- interactions$anno_A
a <- DuplexTrack(interactions, gr_region = gr_region, stacking = "dense")
plotTracks(a, stacking = "dense")
plotTracks(a, stacking = "squish", annotation.column1 = "anno_A")

# add interactions which are not fully in plot range: outside the range or on different chromosome()

# one left (A) interaction arm outside of the plot, other on different chromosome
new_anchor1 <- GRanges(
    seqnames = c("chr1", "chr2"),
    ranges = IRanges(
        start = c(10, 600),
        end = c(90, 700)
    ),
    strand = "+"
)
new_anchor2 <- GRanges(
    seqnames = c("chr1", "chr1"),
    ranges = IRanges(
        start = c(1500, 1000),
        end = c(1600, 1200)
    ),
    strand = "+"
)

new_interactions <- GInteractions(new_anchor1, new_anchor2)
new_interactions$anno_A <- c("A.out", "A.out_chr")
new_interactions$anno_B <- c("B.in", "B.in")
all_interactions <- c(interactions, new_interactions)

b <- DuplexDiscovereR::DuplexTrack(all_interactions,
    gr_region = gr_region,
    annotation.column1 = "anno_A",
    annotation.column2 = "anno_B"
)

plotTracks(b)

# to customize plot, one can call, to see options
DuplexDiscovereR::availableDisplayPars(b)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{get_colnames_and_types_for_input}
\alias{get_colnames_and_types_for_input}
\title{Get colnames for expected data types}
\usage{
get_colnames_and_types_for_input(nameset)
}
\arguments{
\item{nameset}{name of the table}
}
\value{
character vector
}
\description{
Get colnames for expected data types
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clustering.R
\name{computeGISelfOverlaps}
\alias{computeGISelfOverlaps}
\title{Find overlaps between entries in \code{GInteractions}}
\usage{
computeGISelfOverlaps(
  gi,
  id_column = "duplex_id",
  maxgap = 40,
  minoverlap = 10
)
}
\arguments{
\item{gi}{input gi object}

\item{id_column}{column which use for using as ids for entries}

\item{maxgap}{parameter for call of \code{\link[InteractionSet:overlaps]{InteractionSet::findOverlaps()}}}

\item{minoverlap}{parameter for call \code{\link[InteractionSet:overlaps]{InteractionSet::findOverlaps()}}}
}
\value{
dataframe with indexes of pairwise overlapsin input and
columns for span, overlap, ratios of either read arm
}
\description{
Utility function to find overlapping reads in the input and calculate
overlap scores. Removes self-hits. Computes overlap/span ratios for
each interaction arm. Sum of the scores is recorded in 'weight' field
}
\examples{
data("RNADuplexesSmallGI")
computeGISelfOverlaps(SampleSmallGI)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{SampleSmallGI}
\alias{SampleSmallGI}
\title{RNA duplex reads of SPLASH derived from chimeric alignments}
\format{
An object of class \code{StrictGInteractions} of length 14.
}
\source{
\href{https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943}{SequenceReadArcive}
}
\usage{
data(RNADuplexesSmallGI)
}
\description{
\code{GInteractions} object containing two-arm chimeric reads extracted from
mapping output and which can be represented in the \code{GInteraction} object
and subset to chr22: 23877144-45562960 '*'
}
\details{
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details on the data generation procedure.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{RNADuplexSampleDGs}
\alias{RNADuplexSampleDGs}
\title{RNA duplex reads of SPLASH, clustered and collapsed to duplex groups}
\format{
An object of class \code{StrictGInteractions} of length 79.
}
\source{
\href{https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943}{SequenceReadArcive}
Reads were aligned with STAR and duplex groups were identified
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details on the data generation procedure.
}
\usage{
data(RNADuplexesSampleData)
}
\value{
\code{GInteractions} with
\itemize{
\item \code{n_reads} : number of reads in the duplex group (DG)
\item \code{dg_id} :id of the duplex group
\item \code{p_val} : BH adjusted p-value of testing to reject hypothesis of DG arising
from random ligation
\item \code{score} : median alignment score in duplex group
\item other columns with \code{.A} and \code{.B} annotating to which genes either arm of the
DG maps
}
}
\description{
\code{GInteractions} duplex group -level object containing detected duplex groups,
annotated with duplex group ids, gene_names and p-values
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_preproc.R
\name{preproc_generic}
\alias{preproc_generic}
\title{Preprocess .bedpe input}
\usage{
preproc_generic(dt, keep_all_columns = TRUE)
}
\arguments{
\item{dt}{dataframe with reads aligned to strictly two loci}

\item{keep_all_columns}{keeep columns apart form the required from .bedpe format}
}
\value{
pre-processed dataframe
}
\description{
Searches for the multi-mapped and bad reads (overlapping arms)
Adds 'multimap', 'bad_junction' columns filled with 0 or 1 and 'multigap' = 0
for consistency with other pre-processing methods
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexDiscovererResults.R
\name{show,DuplexDiscovererResults-method}
\alias{show,DuplexDiscovererResults-method}
\title{Show Method for DuplexDiscovererResults class}
\usage{
\S4method{show}{DuplexDiscovererResults}(object)
}
\arguments{
\item{object}{A \code{DuplexDiscovererResults} object.}
}
\value{
None. Prints a formatted summary.
}
\description{
This method provides a summary of the DuplexDiscovererResults object.
It prints \code{chimeric_reads_stats} followed by the \code{run_stats}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{RNADuplexSampleGI}
\alias{RNADuplexSampleGI}
\title{RNA duplex reads of SPLASH derived from chimeric alignments}
\format{
An object of class \code{StrictGInteractions} of length 2090.
}
\source{
\href{https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943}{SequenceReadArcive}
}
\usage{
data(RNADuplexesSampleData)
}
\value{
\code{GInteractions} with
\itemize{
\item \code{readname} : read name
\item \code{map_type } : type of the mapped read (2arm by design of pre-filtering)
\item \code{junction_type} : if read jucntion is too short, or it not a 'true' ligated
reads because of the jucntoin coincides with splice junction
\item cigar_aln* columns inherited from the STAR Chimeric.out.Junction output
}
}
\description{
\code{GInteractions} read-level object containing two-arm chimeric reads extracted from
mapping output and which can be represented in the \code{GInteraction} object
}
\details{
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details on the data generation proccedure.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{RNADuplexesRawBed}
\alias{RNADuplexesRawBed}
\title{Chimeric reads of SPLASH converted to .bedpe fromat}
\format{
An object of class \code{spec_tbl_df} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 2040 rows and 10 columns.
}
\source{
\href{https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943}{SequenceReadArcive}
Reads were aligned with STAR and filtered to contain only reads
which could be represented as 2-arm chimeric alignments. Converted to the
\href{https://bedtools.readthedocs.io/en/latest/content/general-usage.html#bedpe-format}{bedpe}
format
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details on the pre-processing and sub-setting the
data
}
\usage{
data(RNADuplexesSampleData)
}
\value{
\code{tibble} with columns of bedpe format
}
\description{
A Chimeric.out.Junction file with a subset of chr 22 Chimeric reads
detected by SPLASH protocol in Human embryonic stem cells.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{subset_gi}
\alias{subset_gi}
\title{Subset the \code{GInteractions} object to single interaction}
\usage{
subset_gi(gi, k)
}
\arguments{
\item{gi}{\code{GInteractions} object}
}
\value{
\code{GInteractions} with the range atribure reduced to single interaction
}
\description{
Sub-setting the \code{GInteractions} object does not reduce its \code{ranges} container
This function selects \code{Ginteraction} by index and reduces the \code{ranges}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clustering.R
\name{.DGIdToDuplexId}
\alias{.DGIdToDuplexId}
\title{Accessor for mapping between temporary and final cluster ids}
\usage{
.DGIdToDuplexId(gi)
}
\arguments{
\item{gi}{}
}
\value{
tibble
}
\description{
Accessor for mapping between temporary and final cluster ids
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_classification.R
\name{classifyTwoArmChimeras}
\alias{classifyTwoArmChimeras}
\title{Wrapper for classification of the 2arm chimeric reads}
\usage{
classifyTwoArmChimeras(
  gi,
  min_junction_len = 4,
  junctions_gr,
  max_sj_shift = 4
)
}
\arguments{
\item{gi}{\code{GInteractions} object}

\item{min_junction_len}{minimum allowed distance between two chimeric arms}

\item{junctions_gr}{\code{Granges} object with the splice junctions coordinates}

\item{max_sj_shift}{maximum shift between either donor and acceptor splice
sites and corresponding chimreic junction coordinates to count chimeric
junction as splice junction}
}
\value{
\code{GInteractions} object object of the same size with new columns:
\describe{
\item{splicejnc}{filled with 0 or 1}
\item{junction_type}{factor for the junction types}
}
}
\description{
Wraps two procedures for different types of classification for read alignment:
\describe{
\item{overlap type}{test if chimeric junction map to two non-overlapped regions or shorter than defined minimum distance}
\item{splice junction}{test if chimeric junction is also a splice junction}
}
}
\details{
Calls detection of the chimeric junction type, annotates short junctions
on same chromosome an strand as 'short'. Compares chimeric
junctions with splice junctions. Adds results as the new metadata fields
parallel to the input.
}
\examples{
data("RNADuplexesSampleData")
head(RNADuplexSampleGI)
# remove all metadata
mcols(RNADuplexSampleGI) <- NULL
gi <- classifyTwoArmChimeras(RNADuplexSampleGI,
    min_junction_len = 5,
    junctions_gr = SampleSpliceJncGR, max_sj_shift = 10
)
table(gi$splicejnc)
table(gi$junction_type)
}
\seealso{
\link[=getSpliceJunctionChimeras]{DuplexDiscovereR::getChimericJunctionTypes(), DuplexDiscovereR::getSpliceJunctionChimeras()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_classification.R
\name{getChimericJunctionTypes}
\alias{getChimericJunctionTypes}
\title{Classify chimeric junctions of two-arm reads into types}
\usage{
getChimericJunctionTypes(gi, normal_gap_threshold = 10)
}
\arguments{
\item{gi}{\code{GInteractions} object}

\item{normal_gap_threshold}{minimum allowed distance between chimeric arms}
}
\value{
gi object of the same size with the 'junction_type' field added
}
\description{
Chimeric reads which can be represented ans two-arm interactions can be
divided into several categories based on the distance between the chimeric
fragments and existence of the overlap between these fragments.
}
\details{
Takes \code{GInteractions} object and classifies junctions into following categories
\describe{
\item{2arm}{ normal chimeric read }
\item{2arm_short}{ normal chimeric read with junction < \emph{normal_gap_threshold}}
\item{self_ovl}{ arms overlap}
\item{antisense_ovl}{ arms overlap on the opposite strand}
}
}
\examples{
data("RNADuplexesSampleData")
preproc_df <- runDuplexDiscoPreproc(RNADuplexesRawBed, table_type = "bedpe")
preproc_gi <- makeGiFromDf(preproc_df)
preproc_gi <- getChimericJunctionTypes(preproc_gi)
table(preproc_gi$junction_type)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{get_chimeric_junctions_onestrand}
\alias{get_chimeric_junctions_onestrand}
\title{Get chimeric junctions}
\usage{
get_chimeric_junctions_onestrand(gi_intra)
}
\value{
Granges object with the
}
\description{
Returns chimeric junction defined as range distance between the end and the
start of the first and second range respectively
}
\details{
If the pair of interacting ranges is not on the same strand and chromosome,
returns error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{RNADuplexesRawChimSTAR}
\alias{RNADuplexesRawChimSTAR}
\title{Chimeric reads of SPLASH}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 5000 rows and 21 columns.
}
\source{
\href{https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943}{SequenceReadArcive}
Reads were aligned with STAR
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details on the pre-processing and sub-setting the
data
}
\usage{
data(RNADuplexesSampleData)
}
\value{
\code{tibble} with columns of Chimeric.junction.out
}
\description{
A Chimeric.out.Junction file with a subset of chr 22 Chimeric reads
detected by SPLASH protocol in Human embryonic stem cells.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clustering.R
\name{collapseIdenticalReads}
\alias{collapseIdenticalReads}
\title{Collapses identical interactions}
\usage{
collapseIdenticalReads(gi)
}
\arguments{
\item{gi}{GInteractions(mode='strict') object with chromA, strandA, startA,
endA, chromB, strandB, startB, endB, score columns
Optionally cigar_alnA, cigar_alnB columns are also considered for collapsing
'read_id' column used as the index in the initial objects. Created, if not exists}
}
\value{
result_list object with keys
' gi_collapsed':   New collapsed GInteraction object
' stats_df': tibble with the mapping of the original entries to the new duplex_id
}
\description{
Two entries (reads) are considered identical if they share start, end, strand and score vales
Identical entries are collapsed into the single one.
}
\details{
Adds columns to the collapsed object
duplex_id (int) unique record id
n_reads (int) number of entries collapsed
}
\examples{
# load data
data("RNADuplexesSmallGI")
res_collapse <- collapseIdenticalReads(SampleSmallGI)
gi_new <- res_collapse[["gi_collapsed"]]
# keeps the mapping of the colapsed object to new
read_stats_df <- res_collapse[["stats_df"]]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{get_arm_a}
\alias{get_arm_a}
\title{Get left arm of GInteraction}
\usage{
get_arm_a(gi)
}
\arguments{
\item{gi}{GInteractions object}
}
\value{
Granges object with the left (A) region
}
\description{
Get left arm of GInteraction
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runDuplexDiscoverer.R
\name{runDuplexDiscoverer}
\alias{runDuplexDiscoverer}
\title{Executes all steps of DuplexDiscovereR pipeline}
\usage{
runDuplexDiscoverer(
  data,
  table_type = "",
  junctions_gr = NULL,
  anno_gr = NULL,
  anno_gr_keys = c("gene_id", "gene_name", "gene_type"),
  fafile = NULL,
  df_counts = NULL,
  sample_name = "sample",
  lib_type = "SE",
  min_junction_len = 5,
  max_gap = 50,
  min_arm_ratio = 0.1,
  min_overlap = 10,
  max_sj_shift = 10,
  gap_collapse_similar = 2,
  collapse_n_inter = 5,
  trim_alignments = FALSE,
  trim_length = 40,
  min_arm_len = 9,
  compute_p_values = TRUE,
  compute_odds_test_p_values = FALSE
)
}
\arguments{
\item{data}{dataframe-like object with the split reads. Output of Chimeric.out.junction or dataframe with fileds defined by bedpe format:
c("chromA","startA",'endA',"chromB",'startB','endB','readname','flag','strandA','strandB', ... )
Alternatively, \code{GInteractions} object}

\item{table_type}{one in c("STAR","bedpe") Defines the type of the input dataframe.
ignored if input data is \code{GInteractions}}

\item{junctions_gr}{\pkg{GRanges} object with the splice junction coordinates}

\item{anno_gr}{\pkg{GRanges} object to use for the annotation of the interactions. Optional}

\item{anno_gr_keys}{c() vector with names of metadata fields in anno_gr which will be used for the annotation. Argument passed to
\code{annotateGI()} function. The c('gene_id','gene_name','gene_type') columns in anno_gr are used by default.}

\item{fafile}{path to the genome .fasta file. Used to calculate hybridization energy with \emph{RNADuplex}. Sequence names should correspond to the sequences from which the mapping index was created. Optional}

\item{df_counts}{A two- column dataframe with counts. Counts are used for p-value calculation. The first column should match the 'gene_id' feature in anno_gr. The second column is the respective count. Optional}

\item{sample_name}{A name of the sample, used for assembling the analysis statistics dataframe}

\item{lib_type}{one in c('SE','PE'). Type of the seqeuncing library. Default is 'SE'}

\item{min_junction_len}{a minimum allowed distance between chimeric arms for the read input.
Reads with the junction closer than \code{min_junction_len} are annotated as '2arm_shot' and not clustered to duplex groups}

\item{max_gap}{Parameter for read clustering. Minimum required shift between start and end coordinates of arms for pair of overlapping chimeric reads.
If the shift is longer than \code{max_gap} for either arm, then total read overlap between those reads is zero.}

\item{min_arm_ratio}{Parameter for read clustering.
If the overlap-to-span ratio for either arm (A or B) for pair of chimeric reads is less than \code{min_arm_ratio}, then the total overlap for this pair is set to zero.}

\item{min_overlap}{Parameter for read clustering. Minimum required overlap to for either arm (A or B) for pair of chimeric reads.}

\item{max_sj_shift}{Maximum shift between either donor and acceptor splice sites and chimeric junction coordinates to count chimeric junction as splice junction}

\item{gap_collapse_similar}{Parameter for read clustering (iterative step). Analogous to the max_gap, but applied \code{collapse_n_inter} times during the iterative merging step.
Reduce this to 1 or 2 to lower RAM usage for clustering the library with many similar reads.}

\item{collapse_n_inter}{Parameter for read clustering  (iterative step). Number of iterations to repeat step of collapsing of the highly similar chimeric reads.
Increasing this from i.e 0 to 5 reduces clustering time and memory for the libraries with many overlapping reads.}

\item{trim_alignments}{TRUE or FALSE. Whether to trim arms alignments to
'trim_length' nucleotide around chimeric junction}

\item{trim_length}{target size of trimmed alignment}

\item{min_arm_len}{minimum allowed length of the alignment arm.
Read will be dropped if either arm is shorter}

\item{compute_p_values}{TRUE or FALSE. whether to calcualte random ligation test with binominal ligation model}

\item{compute_odds_test_p_values}{TRUE or FALSE. Whether to calcualte odds-ratio random ligation test.}
}
\value{
a list with the  following keys
\describe{
\item{\code{duplex_groups}}{ \code{GInteractions} object with chimeric reads clustered duplex groups }
\item{\code{chimeric_reads}}{ \code{GInteractions} object with non-collapsed chimeric reads }
\item{\code{reads_classes}}{ \code{tbl_df} dataframe parallel to the the input dataframe, annotated with read categories and duplex groups }
\item{\code{chimeric_reads_stats}}{ \code{tbl_df} dataframe containing read type classification statistics }
\item{\code{run_stats}}{ \code{tbl_df} dataframe with the time and memory info about the run }
}
}
\description{
Generates GInteractions object with duplex groups from the STAR
Chimeric.out.junction or bedpe file.
Classifies reads, annotates reads by overlap with the gene or transcript
features, calculates p-values and hybridization energies.
Additionally, returns mappings from duplex groupd back to genes.
}
\details{
This is a main function to do the initial discovery of the RNA duplexes after
the chimeric read mapping. It wraps following procedures:
\itemize{
\item Classifies the input reads by the mapping type.
Keeps 2-arm chimeric reads for downstream analysis
\item Compares 2arm duplex reads against provided splice junctions
\item Classifies 2arm duplexes into spurious self-overlapping, splice junction categoris
\item Performs clustering of the remaining reads into duplex groups
\itemize{
\item Collapses identically mapped reads
\item Collapses closely located reads, almost identical reads
\item Finds duplex groups throughout whole  data set
}
\item Annotates duplex groups with genomic features if annotation is provided
\item Calculates p-values if gene counts and annotation are provided
\item Calculates hybridization energies if path to the .fasta file is provided
}
}
\examples{

library(DuplexDiscovereR)
# load data
data("RNADuplexesSampleData")
result <- runDuplexDiscoverer(
    data = RNADuplexesRawChimSTAR,
    junctions_gr = SampleSpliceJncGR,
    anno_gr = SampleGeneAnnoGR,
    df_counts = RNADuplexesGeneCounts,
    sample_name = "test clustering",
    fafile = NULL,
    collapse_n_inter = 3,
    lib_type = "SE",
    table_type = "STAR"
)
# see results object
print(result)
# duplex groups
dd_get_duplex_groups(result)
# individual chimeric reads
dd_get_chimeric_reads(result)
# counts of detected read tyoes
dd_get_chimeric_reads_stats(result)
}
\seealso{
\code{\link[=DuplexDiscovererResults]{DuplexDiscovererResults()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{makeGiFromDf}
\alias{makeGiFromDf}
\title{Convert Dataframe to GInteractions}
\usage{
makeGiFromDf(df)
}
\arguments{
\item{df}{dataframe-like object. Should be convertable to tibble::tibble()}
}
\value{
GInteractions(mode='strict')
}
\description{
Converts dataframe-like object to the \code{GInteractions}.
}
\details{
arms will be consistent between different objects of same reference
Following columns are looked up in input dataframe  to parse region coordinates:
c("chromA','startA','endA','strandA',"chromB",'startB','endB','strandB')
\code{GInteractions(mode='strict')} is enforced, to ensure that the order of the regions
Extra columns are stored as metadata fields
}
\examples{
# load example GInteractions
data(RNADuplexesSmallGI)

converted_to_df <- makeDfFromGi(SampleSmallGI)
converted_to_gi <- makeGiFromDf(converted_to_df)
}
\seealso{
\code{\link[=makeDfFromGi]{makeDfFromGi()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_annotation.R
\name{annotateGI}
\alias{annotateGI}
\title{Annotate RNA duplexes with features}
\usage{
annotateGI(
  gi,
  anno_gr,
  keys = c("gene_name", "gene_type", "gene_id"),
  ambig_key = keys[1],
  save_ambig = TRUE,
  order_key = NULL,
  order_vec = NULL
)
}
\arguments{
\item{gi}{\code{GInteraction} object to annotate}

\item{anno_gr}{\code{GRanges} object with the \code{keys} columns in the metadata}

\item{keys}{vectors with the names of the features to use for annotation.}

\item{ambig_key}{a which feature to use for recording the annotation ambiguity. Determines the values in \code{ambig_list.A} and \code{ambig_list.B}.}

\item{save_ambig}{When RNA duplex overlaps multiple features, ambiguous annotation for a single key
can be stored in \code{ambig_list.A} and \code{ambig_list.B}. \code{ambig.A} and `ambig.B' fields will be added as the 0/1 label}

\item{order_key}{Experimental. In case RNA duplex overlaps multiple features, this key will be used to sort the overlapping features.}

\item{order_vec}{Experimental. An ordered vector of values in \code{order_key} annotation feature, which sets the priority in case of feature overlap.}
}
\value{
\code{GInteractions} object with new fields
}
\description{
Overlays RNA duplexes with \code{GRanges} annotation object.
}
\details{
For each annotation feature in \code{keys}, i.e if keys=c(keyname1),
then \verb{<keyname1>.A}, \verb{<keyname1>.B} annotation fields will be created, containing the
names of overlapping features
If no overlap is found for the feature, then filed will have NA
}
\examples{
data("RNADuplexesSampleData")
annotateGI(gi = RNADuplexSampleDGs, anno_gr = SampleGeneAnnoGR)

# Prioritisation of the snRNA and lncRNA before mRNA if genes overlap
annotateGI(
    gi = RNADuplexSampleDGs, anno_gr = SampleGeneAnnoGR,
    keys = c("gene_id", "gene_name", "gene_type"),
    order_key = "gene_type",
    order_vec = c("snRNA", "lncRNA", "protein_coding"),
    save_ambig = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexDiscovereR-package.R
\docType{package}
\name{DuplexDiscovereR}
\alias{DuplexDiscovereR-package}
\alias{DuplexDiscovereR}
\title{Analysis of the data from RNA duplex probing experiments}
\description{
DuplexDiscovereR is a package for analysing data from RNA cross-linking and
proximity ligation protocols such as SPLASH, PARIS, LIGR-seq and others,
which provide information about intra-molecular RNA-RNA interactions through
chimeric RNA-seq reads. Chimerically aligned fragments in these experiments
correspond to the base-paired stretches (RNA duplexes) of RNA molecules .
DuplexDiscovereR takes input in the form of chimericly or split -aligned reads,
It implements procedures for alignment classification, filtering and efficient
clustering  of  individual chimeric reads into duplex groups (DGs).
Once DGs are found, RNA duplex formation and their hybridization energies are
predicted. Additional metrics, such as p-values or mean DG alignment scores,
can be calculated to rank and analyse the final set of RNA duplexes.
Data from multiple experiments or replicates can be processed separately
and further compared to check the reproducibility of the experimental method.
}
\details{
DuplexDiscovereR
}
\seealso{
\href{`browseVignettes("DuplexDiscovereR")`}{DuplexDiscovereR vignette}
}
\author{
Egor Semenchenko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clustering.R
\name{.addDGidsForTmpDGs}
\alias{.addDGidsForTmpDGs}
\title{Helper function to add ids to the duplex groups missed during global clustering}
\usage{
.addDGidsForTmpDGs(gi_input)
}
\arguments{
\item{gi_input}{\pkg{GInteractions} with the  \code{dg_id},  \code{duplex_id} and  \code{n_reads} column}
}
\value{
\pkg{GInteractions} object with new \code{dg_id} for rows with  \code{n_reads > 1}
}
\description{
Check if there are a temporary duplex records with \code{duplex_id}, which consist of more than one read  \code{n_reads > 1} , but
does not have assigned any \code{dg_id} as the duplex group (DG) index. Creates new  \code{dg_id} if \code{n_reads > 1}
}
\details{
Meant to be used in the situations when previous collapsing steps merged two or more reads to the temporary DG with \code{duplex_id}, but
global clustering has not identified any overlap between this temporary group and other duplexes, resulting in undefined \code{dg_id}.
This function looks up for these cases and creates new \code{dg_id} for temporary DGs, marking them as the final DGs.
New \code{dg_id} values are unique and allocated sequentially after the  maximum value of \code{dg_id}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{makeDfFromGi}
\alias{makeDfFromGi}
\title{Convert GInteractions to tibble}
\usage{
makeDfFromGi(gi)
}
\arguments{
\item{gi}{GInteracttions}
}
\value{
tibble preserving metadata columns
}
\description{
Converts \code{GInteractions} to tibble, preserves metadata
}
\details{
Following naming conventions is used  for region coordinates:
\code{c('chromA','startA','endA','strandA', 'chromB','startB','endB','strandB')}
}
\examples{
data(RNADuplexesSmallGI)
converted_to_df <- makeDfFromGi(SampleSmallGI)
converted_to_gi <- makeGiFromDf(converted_to_df)
}
\seealso{
\code{\link[=makeGiFromDf]{makeGiFromDf()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexDiscovererResults.R
\docType{methods}
\name{dd_get_run_stats}
\alias{dd_get_run_stats}
\alias{dd_get_run_stats,DuplexDiscovererResults-method}
\title{Accessor for \code{run_stats} Slot}
\usage{
dd_get_run_stats(object)

\S4method{dd_get_run_stats}{DuplexDiscovererResults}(object)
}
\arguments{
\item{object}{A \code{DuplexDiscovererResults} object.}
}
\value{
\code{tibble} from the \code{run_stats} slot.
}
\description{
Retrieves the value of the \code{run_stats} slot in a \code{DuplexDiscovererResults} object.
}
\examples{

# load example input
data("RNADuplexesSmallGI")
data("RNADuplexesSampleData")
# run whole pipeline
result <- runDuplexDiscoverer(
    data = SampleSmallGI,
    junctions_gr = SampleSpliceJncGR,
    anno_gr = SampleGeneAnnoGR,
    sample_name = "run_example",
    lib_type = "SE",
    table_type = "STAR"
)
# access results
show(result)
gi_clusters <- dd_get_duplex_groups(result)
gi_reads <- dd_get_chimeric_reads(result)
df_reads <- dd_get_reads_classes(result)
dd_get_reads_classes(result)
dd_get_run_stats(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_preproc.R
\name{col_check_rename}
\alias{col_check_rename}
\title{Check the column names and types in read dataframe}
\usage{
col_check_rename(df, table_type = "STAR")
}
\arguments{
\item{df}{input}

\item{table_type}{one in c("STAR","bedpe")}
}
\value{
dataframe with the properly formatted columns
}
\description{
Function to check the correct column names and types in dataframe input.
Tries to guess the column names, if colnames are not provided,
but the types are correct
}
\details{
\itemize{
\item Expected column names for bedpe file
\code{c("chromA","startA",'endA',"chromB", 'startB','endB','readname','flag','strandA','strandB')}
\item Expected colnames for STAR Chimeric junction input:
\item For the 'old' chimeric detection scheme: \code{ c("chr_donorA","brkpt_donorA",
"strand_donorA","chr_acceptorB",
"brkpt_acceptorB","strand_acceptorB",
"junction_type","repeat_left_lenA",
"repeat_right_lenB","read_name",
"start_alnA","cigar_alnA",
"start_alnB","cigar_alnB",
"num_chim_aln","max_poss_aln_score",
"non_chim_aln_score","this_chim_aln_score",
"bestall_chim_aln_score") }
\item For the 'new' chimeric detection scheme
\code{c("chr_donorA","brkpt_donorA",
                  "strand_donorA","chr_acceptorB",
                  "brkpt_acceptorB","strand_acceptorB",
                  "junction_type","repeat_left_lenA",
                  "repeat_right_lenB","read_name",
                  "start_alnA","cigar_alnA",
                  "start_alnB","cigar_alnB")}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeGiToSAMfile.R
\name{writeGiToSAMfile}
\alias{writeGiToSAMfile}
\title{Write reads to sam file}
\usage{
writeGiToSAMfile(
  gi_coords,
  file_out,
  distance_chim_junction = 10000,
  read_name_column = "readname",
  id_column = "dg_id",
  genome = "",
  sample_name = "noname_sample"
)
}
\arguments{
\item{gi_coords}{input Ginteraction object}

\item{file_out}{path to write output file}

\item{distance_chim_junction}{maximum distance between input
duplex groups/reads, which will be represented as the single-line in .sam file.
Junction will be output as N- gap. For the interactions with longer distances,
chimeric junction will be represented as MR:Z:i tag}

\item{read_name_column}{character field, pointing out to read names.
Read names are generated automatically if not provided.}

\item{id_column}{character name of the field containing integer duplex group ids.
NA are replaced with zeros}

\item{genome}{character. Genome version. Required for the retrieval of sequence lengths
for sam file header-  SQ and SN tags.
For convenience, hg38 and hg19 chromosome lengths will be assigned
automatically.
If the value is not in c('hg38','hg19'), seqlengths will be looked for be in
attribute in seqlengths() of regions(gi_coords)}

\item{sample_name}{name to use in RG SAM tag in header}
}
\value{
no object is returned
}
\description{
Writes interactions to the sam file for visualization in extrnal browsers.
Takes input as GInteractions object containing reads or duplex groups.
}
\examples{
# Load test data
data("RNADuplexesSampleData")
# if the input is read-based, it should have integer duplex group ids
# here, we have 2090 reads
length(RNADuplexSampleGI)
# among them 300 reads does not belong to any DG
# missing ids will be converted to 0
table(is.na(RNADuplexSampleGI$dg_id))
tmpf <- tempfile(".sam")
writeGiToSAMfile(
    gi_coords = RNADuplexSampleGI,
    id_column = "dg_id",
    file_out = tmpf,
    distance_chim_junction = 1e5,
    genome = "hg38"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{refresh_gi}
\alias{refresh_gi}
\title{Refresh the \code{GInteractions} object}
\usage{
refresh_gi(gi)
}
\arguments{
\item{gi}{\code{GInteractions} object}
}
\value{
\code{GInteractions} object with new ranges attribute
}
\description{
Sub-setting the \code{GInteractions} object does not reduce its \code{ranges} container
For some applications, to save memory, we can safely reduce the size of the
object by re-creating it.
Also, it can be used to ensure the 'strict' mode of the regions in \code{ranges}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_annotation.R
\name{.annotateCisTrans}
\alias{.annotateCisTrans}
\title{Annotate RNA-RNA interactions as cis- and trans-}
\usage{
.annotateCisTrans(gi, id_col_base = "gene_id")
}
\arguments{
\item{gi}{\code{GInteractions} object containing two metadata columns as feature annotation}

\item{id_col_base}{base name of the feature id columns to use. Function will
look for <id_col_base>.A and <id_col_base>.B columns and compare them}
}
\value{
gi \code{GInteractions} object containing \code{cis} field with 0/1 values
}
\description{
Annotated each entry gi object as cis, if the .A and .B arms correspond to
the same feature (i.e transcript_id or gene_id)
If the values are are equal, then annotated with value: \code{cis} = 1, If not equal
or \code{NA}: \code{cis} = 0
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{SampleGeneAnnoGR}
\alias{SampleGeneAnnoGR}
\title{Gene coordinates on human chromosome 22}
\format{
An object of class \code{GRanges} of length 1445.
}
\source{
\href{https://www.gencodegenes.org/human/release_44.html}{GENCODEv44}
}
\usage{
data(RNADuplexesSampleData)
}
\value{
statdatd GENCODE gtf fields
}
\description{
\code{Granges} containing gene coordinates of human chromosome 22 obtained from
GENCODEv44 annotaion
}
\details{
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapseDuplexGroups.R
\name{collapse_duplex_groups}
\alias{collapse_duplex_groups}
\title{Collapse the reads into the duplex groups after clustering}
\usage{
collapse_duplex_groups(
  gi,
  return_unclustered = FALSE,
  return_collapsed = TRUE,
  keep_meta = TRUE
)
}
\arguments{
\item{gi}{\code{GInteractions} with the 'dg_id' metadata field}

\item{return_unclustered}{add unclustered reads to output}

\item{return_collapsed}{add duplex groups, which were created as temporary
with n_reads > 1 but was not clustered to the DG golabally. This parameter
is used internally and should be kept default in most situations.}

\item{keep_meta}{whether to keep metadata, which only unclustered reads have, in case of a mixed output}
}
\value{
\code{GInteractions} object with collapsed duplex groups
}
\description{
Collapse each interaction in the input to the duplex group based on the pre-computed dg_id
}
\details{
'dg_id' is used as the identifier for the duplex group
Reads belonging to the same duplex group are collapsed into a single entry
with start and end are set as min() and max() coordinate of the reads in within the duplex group.
The 'score' column is averaged across the duplex group reads is calculated and
put as the 'score' for the collapsed duplex group
Behavior in case 'dg_id'  = NA: Option '\code{return_unclustered}' -
whether unclustered reads with should be added to the output gi
\describe{
\item{return_unclustered == FALSE}{Interaction is not returned in the output. Default. }
\item{return_unclustered == TRUE}{Interaction is returned in the output,
output is mixed duplex groups and individual reads}
}
Internally used argument
#' \describe{
\item{return_collapsed == FALSE}{In case interaction already collapsed and n_read > 1,
interaction will not be returned as duplex group }
\item{return_collapsed == TRUE}{In case interaction has n_read > 1, interaction will be treated
as duplex group }
}
}
\examples{
# load example of clustered data
data("RNADuplexesSampleData")
# some reads assigned to DG, some are not
table(is.na(RNADuplexSampleGI$dg_id))
# Return only DGs
gicollapsed <- collapse_duplex_groups(RNADuplexSampleGI, return_unclustered = FALSE)
# Return DGs and unclustered reads as well
gimixed <- collapse_duplex_groups(RNADuplexSampleGI, return_unclustered = TRUE)

# load small sample GInteractions and process it manually
data("RNADuplexesSmallGI")
# First, collapse duplicated reads. This adds n_reads and duplex ids
ginodup <- collapseIdenticalReads(SampleSmallGI)$gi_collapsed
# Second, run clustering, get DG ids
ginodup <- clusterDuplexGroups(ginodup)
# Return all DGs result in n=3 DGS, one of them formed by
# identical duplicated alignments
collapse_duplex_groups(ginodup, return_collapsed = TRUE)
# Return DGs, but drop duplicated returns n=2 DGs
collapse_duplex_groups(ginodup, return_collapsed = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{convert_gi_to_ranges}
\alias{convert_gi_to_ranges}
\title{Convert \code{GInteractions} object to \code{Granges}}
\usage{
convert_gi_to_ranges(gi)
}
\arguments{
\item{gi}{\code{GInteractions}}
}
\value{
\code{GRanges} twice the length of the input
}
\description{
Creates the 'long' \code{GRanges} by stacking the A and B arms one 'on top' of the other.
Adds \code{id} and  \code{group} fields as indicators of original index and interaction
arm (A- left arm, B- right arm)
}
\examples{
data("RNADuplexesSmallGI")
convert_gi_to_ranges(SampleSmallGI)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexTrack.R
\name{.gv_plotboxes}
\alias{.gv_plotboxes}
\title{Plots distributed boxes}
\usage{
.gv_plotboxes(box, lwd, lty, alpha)
}
\value{
pushes boxes to the viewport
}
\description{
Non-exported from Gviz, contains logic for calcualting boxes alignment on plot
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterDuplexGroups.R
\name{.compute_clusters_comp}
\alias{.compute_clusters_comp}
\title{Helper function to call clustering on each component}
\usage{
.compute_clusters_comp(graph, index)
}
\value{
sub-graph with clusters labelled with
sample-wise unique ids (cluster_group)
}
\description{
Helper function to call clustering on each component
}
\details{
Call clustering on each independent graph component.
Used when decompose==TRUE
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterDuplexGroups.R
\name{clusterDuplexGroups}
\alias{clusterDuplexGroups}
\title{Cluster RNA duplexes in \code{GInteractions} object}
\usage{
clusterDuplexGroups(
  gi,
  graphdf = NULL,
  maxgap = 40,
  minoverlap = 10,
  id_column = "duplex_id",
  weight_column = "weight",
  fast_greedy = FALSE,
  decompose = FALSE,
  id_columns_grapdf = paste(id_column, c(1, 2), sep = "."),
  min_arm_ratio = 0.3,
  dump_graph = FALSE,
  dump_path = ""
)
}
\arguments{
\item{gi}{\code{GInteractions} object}

\item{graphdf}{Optional. Dataframe representing connection edges between entries in gi
If not provided, graphdf is created inside the function}

\item{maxgap}{For graph creation only. Max shift between arms starts and ends for pair of overlapping reads}

\item{minoverlap}{For graph creation only. Minimum required overlap between either arm for pair of overlapping reads
Other optional arguments, which are not relevant, unless user want to modify clustering
weights or modify clustering in some other way}

\item{id_column}{Optional. Column name in the  \code{GInteractions} metadata,
which was used to index temporary duplex groups, if they are present}

\item{weight_column}{Optional. If graphdf is provided,  field to use for weight overlaps}

\item{fast_greedy}{Optional. Run the fast_greedy algorithm instead of Louvain.
Can speed up calcualtion for the large graphs.}

\item{decompose}{Decompose graph into separate sub-graphs before clustering.}

\item{id_columns_grapdf}{Column in the graph dataframe, which was used for index}

\item{min_arm_ratio}{For graph creation only. Span-to-overlap ratio threshold. If smaller than this value, then edge is not drawn}

\item{dump_graph}{For debug. Export the graph elements. not used}

\item{dump_path}{For debug. PArt to export the graph elements. not used}
}
\value{
\code{GInteractions} object with new \code{dg_id} column
}
\description{
Main method to find duplex groups from the individual interactions
}
\details{
Accepts or creates the connections graphdf dataframe, creates graph with
\code{igraph} package, uses community detection algoritm to call clusters.
New field \code{dg_id} is added to label the clusters (duplex groups).
If no community is found for the read, \code{dg_id} is \code{NA}
}
\examples{
data("RNADuplexesSampleData")
# run preprocessing and filtering
preproc_df <- runDuplexDiscoPreproc(RNADuplexesRawBed, table_type = "bedpe")
preproc_gi <- makeGiFromDf(preproc_df)
preproc_gi <- classifyTwoArmChimeras(preproc_gi,
    min_junction_len = 5,
    junctions_gr = SampleSpliceJncGR, max_sj_shift = 10
)
# collapse duplicates
gi <- collapseIdenticalReads(preproc_gi)$gi
# run global clustering
gi <- clusterDuplexGroups(gi)
# check dg_ids
table(is.na(gi$dg_id))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clustering.R
\name{collapseSimilarChimeras}
\alias{collapseSimilarChimeras}
\title{Call clustering multiple times to collapse similar reads into duplex groups}
\usage{
collapseSimilarChimeras(
  gi,
  read_stats_df,
  maxgap = 5,
  niter = 2,
  minoverlap = 10,
  min_nodes = 10
)
}
\arguments{
\item{gi}{\code{GInteractions} object}

\item{read_stats_df}{\code{tibble} with the mapping 'read_id' and 'duplex_id' fields
'read_id' refers to the unique read, 'duplex_id' refers to the entry collapsed
identical reads i.e two identical reads will will correspond to two unique read_id and
the single duplex_id with n_reads=2}

\item{maxgap}{Maximum relative shift between the overlapping read arms}

\item{niter}{Number of times clustering will be called}

\item{minoverlap}{Minimum required overlap between either read arm}

\item{min_nodes}{Minimum count of nodes to finish the interaction merging}
}
\value{
a list with the following keys
\describe{
\item{gi_updated}{ \code{GInteractions} object with both collapsed duplex groups
and not-collapsed unchanged reads}
\item{stats_df}{ \code{tibble} With the mapping from the unique read -
with the the infromation about time and memory reaquired for the function
call
}
}
}
\description{
Function calls clustering algorithm several times and collapses highly similar
reads to the temporary duplex groups (DGs).
}
\details{
Calling this procedure before global read clustering
substantially reduces time required for calling DGs.
Collapsed duplex groups are aggregated only from the reads which are shifted
by only a few nucleotides from each other.
These DGs are temporary until full library clustering is called.
To keep track of the mapping of the temprary DGs to the input, dedicated
dataframe is returned. The 'duplex_id' column will be added or updated as
identifier for the temporary duplex group.
The number of reads under single 'duplex_id' is recorded in the 'n_reads' fields
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexTrack.R
\name{show,DuplexTrack-method}
\alias{show,DuplexTrack-method}
\title{Show method for DuplexTrack}
\usage{
\S4method{show}{DuplexTrack}(object)
}
\arguments{
\item{object}{DuplexTrack.}
}
\value{
class representation
}
\description{
Show method for DuplexTrack
}
\examples{
library(InteractionSet)
anchor1 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(100, 600, 1100, 1600, 2100),
        end = c(200, 700, 1200, 1700, 2200)
    ),
    strand = "+"
)
anchor2 <- GRanges(
    seqnames = "chr1",
    ranges = IRanges(
        start = c(300, 800, 1300, 1800, 2300),
        end = c(400, 900, 1400, 1900, 2400)
    ),
    strand = "+"
)

interactions <- GInteractions(anchor1, anchor2, mode = "strict")
gr_region <- range(anchor1, anchor2)
a <- DuplexTrack(interactions, gr_region = gr_region, stacking = "dense")
show(a)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{get_arm_b}
\alias{get_arm_b}
\title{Get right arm of GInteraction}
\usage{
get_arm_b(gi)
}
\arguments{
\item{gi}{GInteractions object}
}
\value{
Granges object with the right (B) region
}
\description{
Get right arm of GInteraction
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_preproc.R
\name{runDuplexDiscoPreproc}
\alias{runDuplexDiscoPreproc}
\title{Run pre-processing of chimeric reads input}
\usage{
runDuplexDiscoPreproc(
  data,
  table_type,
  library_type = "SE",
  keep_metadata = TRUE,
  return_gi = FALSE,
  min_arm_len = 15
)
}
\arguments{
\item{data}{Either dataframe-like object: \emph{Chimeric.out.junction} from \emph{STAR} or
\emph{.bedpe} - formatted or \code{GInteractions} object from \pkg{InteractionSet} package}

\item{table_type}{in \code{c("STAR","bedpe")} for Chimeric.out.Junction or generic input}

\item{library_type}{\code{c("SE","PE")} for pair- or single- end input}

\item{keep_metadata}{\code{c(TRUE,FALSE)} Whether extra fields like
CIGAR strings and junction coordinates should be kept}

\item{return_gi}{if the return object should be \code{GInteractions}}

\item{min_arm_len}{minimum allowed length of the alignment arm.
Read will be dropped if either arm is shorter}
}
\value{
tibble with new metadata fields OR GInteractions if \code{return_gi} is
set to TRUE
}
\description{
Imports dataframe with reads (\emph{.bedpe} or \emph{Chimeric.out.junction} ) or \code{GInteractions}
object.
Checks column names or tries to quess them if not provided.
Adds necessary annotation depending on the input type,
For \emph{STAR} input, calculates length of the alignments
and marks unique 2-arm alignments.
For the \emph{.bedpe} or \code{GInteractions} input, all entries are already represented as reads
with two different aligned parts (2-arm), so only check for unique readname
is performed.
}
\details{
If not existed, adds fields required for the downstream steps:
'readname', 'map_type', 'score', 'n_reads'.
'map_type' field determines the type of the chimeric read:
\describe{
\item{multimap}{ multi-mapped read}
\item{multigap}{ more than one junction (more than two 'N' in CIGAR string)}
\item{bad junction}{ Artifacts or possibly unaccounted types.
I.e alignments for both arms are continuous, but with 'backward' chimeric
junction was wrongly introduced in the mapping}
}
}
\examples{
# load data
data(RNADuplexesSampleData)
# with bedpe input
preproc_reads <- runDuplexDiscoPreproc(RNADuplexesRawBed, table_type = "bedpe")
# with STAR input
preproc_reads_star <- runDuplexDiscoPreproc(RNADuplexesRawChimSTAR,
    table_type = "STAR",
    keep_metadata = FALSE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexTrack.R
\name{.gv_updatepars}
\alias{.gv_updatepars}
\title{Set Gviz graphical parameters}
\usage{
.gv_updatepars(x, class)
}
\value{
set default values inside class upon calling withn class constructor
}
\description{
Set Gviz graphical parameters
}
\details{
Non-exported from Gviz. Uses the same procedure as in AnnotationTrack
to set defaults.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_annotation.R
\name{calculateLigationPvalues}
\alias{calculateLigationPvalues}
\title{Calculate ligation-based significance statistics for RNA duplexes}
\usage{
calculateLigationPvalues(
  gi,
  df_counts,
  id_col = "gene_id",
  compute_binom = TRUE,
  compute_odds = FALSE
)
}
\arguments{
\item{gi}{\code{GInteractions} object annotated with gene/transcript names.}

\item{df_counts}{\code{data.frame}. A two-column data frame with gene/transcript
counts. The first column should match the \verb{<id_col>.A} / \verb{<id_col>.B}
identifiers in \code{gi}. The second column is the corresponding count.}

\item{id_col}{\code{character(1)}. Prefix for gene/transcript metadata fields in
\code{gi}. Two fields, \verb{<id_col>.A} and \verb{<id_col>.B}, are expected.}

\item{compute_binom}{\code{logical(1)}. Whether to compute the binomial
random-ligation test. Default is \code{TRUE}.}

\item{compute_odds}{\code{logical(1)}. Whether to compute the odds-ratio /
Fisher exact test. Default is \code{TRUE}.}
}
\value{
\code{GInteractions} object with added metadata columns.
}
\description{
Calculates significance statistics for each gene/transcript pair represented
in a \code{GInteractions} object. Two optional models are supported:
}
\details{
\enumerate{
\item A binomial random-ligation model, where pair probability is derived from
per-gene/transcript counts and tested using the right-tail binomial test.
\item An odds-ratio / Fisher exact test based on a 2x2 contingency table using
chimeric counts and singleton counts. This test is used in RBP-dependent and
RNA probing methods like RIL-seq
}

The function also outputs abundance fractions for each duplex pair and counts
of other RNA duplexes formed by either or neither gene/transcript in the pair.

\strong{Binomial model}

Null hypothesis: an observed duplex is generated by random ligation.

Alternative hypothesis: the duplex is enriched above the random-ligation
expectation.

The random-ligation probability \eqn{P(a, b)} is defined as:

\deqn{
P(a, b) \propto
\begin{cases}
    2 \cdot P(a) \cdot P(b) & \text{if } a:b \text{ is observed and } a \neq b \\
    P(a)^2                  & \text{if } a:b \text{ is observed and } a = b \\
    0                       & \text{otherwise}
\end{cases}
}

where:

\deqn{
P(a) = \frac{\text{N reads}(a)}{\text{total N reads}}
}

The observed-pair probabilities are renormalized to sum to one and the
p-value is computed from the right tail of the binomial distribution.

\strong{Odds-ratio / Fisher model}

For a pair \verb{(A, B)}, the contingency table is:

\tabular{lll}{
\tab B present \tab B absent\cr
A present     \tab C1        \tab C2\cr
A absent      \tab C3        \tab C4\cr
}

with:
\itemize{
\item \code{C1}: chimeric fragments containing both \code{A} and \code{B}
\item \code{C2}: fragments containing \code{A} but not \code{B}
\item \code{C3}: fragments containing \code{B} but not \code{A}
\item \code{C4}: fragments containing neither \code{A} nor \code{B}
}

A one-sided Fisher exact test (\code{alternative = "greater"}) is used.
}
\examples{
data("RNADuplexesSampleData")
gi <- calculateLigationPvalues(
  RNADuplexSampleDGs,
  df_counts = RNADuplexesGeneCounts
)
hist(gi$p.adj, breaks = 20)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRNAHybrids.R
\name{getRNAHybrids}
\alias{getRNAHybrids}
\title{Run prediciton of RNA hybridization}
\usage{
getRNAHybrids(gi, fafile)
}
\arguments{
\item{gi}{\code{Ginteraction} with pairs of regions}

\item{fafile}{path to the .fasta file with genome}
}
\value{
object parallel to input with added energy GC content,
dot-format base-pairings and lenghts of RNA hybrids
will return the input, if RNAhybrids cannot be run
}
\description{
Calls RNAduplex from ViennaRNA to find base-pairs for every entry in the
input, throws a message and system warning if it is not installed
}
\examples{
sequence <- paste0(
    "AGCUAGCGAUAGCUAGCAUCGUAGCAUCGAUCGUAAGCUAGCUAGCUAGCAUCGAUCGUAGCUAGCAUCGAU",
    "CGUAGCAUCGUAGCUAGCUAGCUAUGCGAUU"
)

# Save the sequence to a temp fasta file
fasta_file <- tempfile(fileext = ".fa")
chrom <- "test_chrA"
writeLines(c(">test_chrA", sequence), con = fasta_file)

# Create the GInteraction object
# Define start and end positions for the base-pairing regions
regions <- data.frame(
    start1 = c(1, 11, 21, 31, 41),
    end1 = c(10, 20, 30, 40, 50),
    start2 = c(91, 81, 71, 61, 51),
    end2 = c(100, 90, 80, 70, 60)
)
# GRanges objects for the anchors
anchor1 <- GRanges(seqnames = chrom, ranges = IRanges(start = regions$start1, end = regions$end1))
anchor2 <- GRanges(seqnames = chrom, ranges = IRanges(start = regions$start2, end = regions$end2))
interaction <- GInteractions(anchor1, anchor2)
# predict hybrids
# In case ViennaRNA is installed
\dontrun{
gi_with_hybrids <- getRNAHybrids(interaction, fasta_file)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_classification.R
\name{getSpliceJunctionChimeras}
\alias{getSpliceJunctionChimeras}
\title{Identify chimeric junctions coinciding with the splice junctions}
\usage{
getSpliceJunctionChimeras(
  gi,
  sj_gr,
  sj_tolerance = 20,
  sj_tolerance_strict = 10
)
}
\arguments{
\item{gi}{\pkg{GInteractions} object}

\item{sj_gr}{\pkg{Granges} object with the splice junctions data}

\item{sj_tolerance}{maximum shift between either donor and acceptor splice
sites and corresponding chimreic junction coordinates to count chimeric
junction as splice junction}

\item{sj_tolerance_strict}{maximum shift between either donor and acceptor splice
sites irrespective of the particular splice junction. If both chimeric junction start
and end correspond to donor or acceptor of any known junction, it is marked
as splice junction. Used to catch novel combinations of known 3' and 5' sites}
}
\value{
gi object with added 'splicejnc' and  field
Additionally 'splicejnc_donor' 'splicejnc_acceptor' fields are added
}
\description{
Marks interactions which starts/ends within specified shift from the
known splice junctions.
}
\examples{
data("RNADuplexesSampleData")
gi <- getSpliceJunctionChimeras(RNADuplexSampleGI, SampleSpliceJncGR)
table(gi$splicejnc)
table(gi$splicejnc_acceptor, gi$splicejnc_donor)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_comparisons.R
\name{compareMultipleInteractions}
\alias{compareMultipleInteractions}
\title{Compare multiple RNA-RNA interactions sets}
\usage{
compareMultipleInteractions(
  gi_samples_list,
  min_ratio = 0.3,
  minoverlap = 5,
  maxgap = 50,
  niter = 3,
  gi_superset = NULL,
  anno_gr = NULL
)
}
\arguments{
\item{gi_samples_list}{anmes list with the \code{GInteractions} entries list('sample1'=gi1,'sample2'='gi2)}

\item{min_ratio}{If the overlap-to-span ratio for either arm (A or B) for pair
of chimeric reads is less than \code{min_arm_ratio}, then the total overlap for this pair is set to zero.
Relevant to comparison of superset vs individual samples}

\item{minoverlap}{Parameter for read clustering to create a superset. Minimum required overlap to for either arm (A or B) for pair of entries.}

\item{maxgap}{Parameter for read clustering. Minimum required shift between start and end coordinates of arms for pair of overlapping entries..
If the shift is longer than \code{max_gap} for either arm, then total read overlap between those reads is zero.}

\item{niter}{Internal parameter for debugging. Number of cluster& collapse iterations to find superset}

\item{gi_superset}{Optional. Superset defining the space (all) of the interactions, against which inputs from the list will be compared.}

\item{anno_gr}{Optional. \code{Granges} to annotate superset.}
}
\value{
dataframe recodding the overlaps between samples and supeset
}
\description{
Combines all interaction into single superset by clustering & collapsing.
Then compares every input entry with the superset. Overlaps between superset and
inputs are recorded in a table as 0/1
}
\examples{
# Create test set of RNA interactions
chrom <- "chr1"
start1 <- c(1, 11, 21, 31, 41, 51, 61, 71, 81, 91)
end1 <- start1 + 9
start2 <- c(101, 111, 121, 131, 141, 151, 161, 171, 181, 191)
end2 <- start2 + 9

anchor1 <- GRanges(seqnames = chrom, ranges = IRanges(start = start1, end = end1))
anchor2 <- GRanges(seqnames = chrom, ranges = IRanges(start = start2, end = end2))

interaction <- GInteractions(anchor1, anchor2)

# Ensure some overlaps
n <- length(interaction)
group_size <- ceiling(n / 2)
group_indices1 <- sort(sample(seq_len(n), group_size))
group_indices2 <- sort(sample(seq_len(n), group_size))
group_indices3 <- sort(sample(seq_len(n), group_size))

# Create separate GInteractions objects for each group
group1 <- interaction[group_indices1]
group2 <- interaction[group_indices2]
group3 <- interaction[group_indices3]

# format input and call comparison
a <- list("sample1" = group1, "sample2" = group2, "sample3" = group3)
res <- compareMultipleInteractions(a)
# comparison result
head(res$dt_upset)
# superset
res$gi_all
# dataframe for the Upset plot
res$dt_upset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{RNADuplexesGeneCounts}
\alias{RNADuplexesGeneCounts}
\title{Gene counts on human chromosome 22, embryonic stem cells}
\format{
An object of class \code{spec_tbl_df} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 1445 rows and 2 columns.
}
\source{
\href{https://www.ncbi.nlm.nih.gov/sra/?term=SRR3404943}{SequenceReadArcive}
}
\usage{
data(RNADuplexesSampleData)
}
\value{
\code{tibble} with columns of Chimeric.junction.out
}
\description{
File generated by mapping with STAR using \verb{--quantMode GeneCounts}
see \code{system.file("extdata/scripts", "DD_data_generation.R", package =
"DuplexDiscovereR")} for details on the pre-processing and sub-setting the
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DuplexDiscovererResults.R
\docType{class}
\name{DuplexDiscovererResults-class}
\alias{DuplexDiscovererResults-class}
\alias{DuplexDiscovererResults}
\title{DuplexDiscovererResults}
\usage{
DuplexDiscovererResults(
  duplex_groups,
  chimeric_reads,
  reads_classes,
  chimeric_reads_stats,
  run_stats
)
}
\arguments{
\item{duplex_groups}{\pkg{GInteractions} object with duplex groups}

\item{chimeric_reads}{\pkg{GInteractions} object with chimeric reads}

\item{reads_classes}{\code{tibble} (tbl_df) with read classification data.}

\item{chimeric_reads_stats}{\code{tibble} (tbl_df)  read type statistics.}

\item{run_stats}{\code{tibble} (tbl_df)  runtime and memory info}
}
\value{
A \code{DuplexDiscovererResults} object.
}
\description{
A helper S4 class to store the results of the full
DuplexDiscovereR analysis.
This class contains the following output:
\itemize{
\item \code{duplex_groups}:  clustered duplex groups.
\item \code{chimeric_reads}: individual two-regions chimeric reads.
Contains both clustered and unclustered reads.
Clustered reads are linked to the duplex groups though 'dg_id' field in metadata
\item \code{reads_classes}: dataframe parallel to the the input containing classification result and detected mapping type for each entry in the input
\item \code{chimeric_reads_stats}: dataframe containing read type classification statistics
\item \code{run_stats}: data frame containing statistics about the time and memory used by the pipeline
}
}
\details{
Each output type has a corresponding accessor:
\itemize{
\item \code{\link[=dd_get_duplex_groups]{dd_get_duplex_groups()}}
\item \code{\link[=dd_get_chimeric_reads]{dd_get_chimeric_reads()}}
\item \code{\link[=dd_get_reads_classes]{dd_get_reads_classes()}}
\item \code{\link[=dd_get_chimeric_reads_stats]{dd_get_chimeric_reads_stats()}}
\item \code{\link[=dd_get_run_stats]{dd_get_run_stats()}}
}
}
\section{Slots}{

\describe{
\item{\code{duplex_groups}}{\pkg{GInteractions} object with duplex groups}

\item{\code{chimeric_reads}}{\pkg{GInteractions} object with chimeric reads}

\item{\code{reads_classes}}{\code{tibble} (tbl_df) with read classification data.}

\item{\code{chimeric_reads_stats}}{\code{tibble} (tbl_df)  read type statistics.}

\item{\code{run_stats}}{\code{tibble} (tbl_df)  runtime and memory info}
}}

\examples{

# load example input
data("RNADuplexesSmallGI")
data("RNADuplexesSampleData")
# run whole pipeline
result <- runDuplexDiscoverer(
    data = SampleSmallGI,
    junctions_gr = SampleSpliceJncGR,
    anno_gr = SampleGeneAnnoGR,
    sample_name = "run_example",
    lib_type = "SE",
    table_type = "STAR"
)
# access results
show(result)
gi_clusters <- dd_get_duplex_groups(result)
gi_reads <- dd_get_chimeric_reads(result)
df_reads <- dd_get_reads_classes(result)
dd_get_reads_classes(result)
dd_get_run_stats(result)
}
\seealso{
\code{\link[=dd_get_duplex_groups]{dd_get_duplex_groups()}} ,
\code{\link[=dd_get_chimeric_reads]{dd_get_chimeric_reads()}} ,
\code{\link[=dd_get_reads_classes]{dd_get_reads_classes()}} ,
\code{\link[=dd_get_chimeric_reads_stats]{dd_get_chimeric_reads_stats()}} ,
\code{\link[=dd_get_run_stats]{dd_get_run_stats()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_preproc.R
\name{preproc_generic_gi}
\alias{preproc_generic_gi}
\title{Preprocess GInteractions input}
\usage{
preproc_generic_gi(gi_raw, keep_all_columns = TRUE)
}
\arguments{
\item{gi_raw}{\code{GInteractions} with inpit RNA interactions}

\item{keep_all_columns}{keep columns apart from those required by .bedpe format}
}
\value{
pre-processed dataframe
}
\description{
Searches for the multi-mapped reads (overlapping arms)
Adds 'multimap', 'bad_junction' columns filled with 0/1 and 'multigap' = 0
for consistency with other pre-processing methods.
}
\keyword{internal}
